@@ -185,9 +185,6 @@ private:
     QPrinter	 *printer;
     QButtonGroup *bgroup;
     QPushButton	 *print;
-    QRadioButton *rb_wheel;
-    QRadioButton *rb_fonts;
-    QRadioButton *rb_patht;
     int		  drawindex;
     int		  maxindex;
 };
