@@ -4,6 +4,9 @@
 // Definition of the Hello widget.
 //
 
+#ifndef HELLO_H
+#define HELLO_H
+
 #include <qwidget.h>
 
 
@@ -23,3 +26,5 @@ private:
     QString t;
     int     b;
 };
+
+#endif
