@@ -8,15 +8,15 @@
 #include <qradiobt.h>
 #include <qbttngrp.h>
 #include <qlcdnum.h>
-#include <qscrbar.h>
+#include <qslider.h>
 #include <qmenubar.h>
 
 #include <qpainter.h>
 #include <qpixmap.h>
-#include <qstring.h>
 
 #include <stdlib.h>
 
+
 class FontSelect;
 
 
@@ -38,9 +38,9 @@ private slots:
     void fontSelected( const QFont & );
     void toggleMode();
 private:
-    QScrollBar	*rotS;		       // Rotation angle scroll bar
-    QScrollBar	*shearS;	       // Shear value scroll bar
-    QScrollBar	*magS;		       // Magnification value scroll bar
+    QSlider	*rotS;		       // Rotation angle scroll bar
+    QSlider	*shearS;	       // Shear value scroll bar
+    QSlider	*magS;		       // Magnification value scroll bar
     QLCDNumber	*rotLCD;	       // Rotation angle LCD display
     QLCDNumber	*shearLCD;	       // Shear value LCD display
     QLCDNumber	*magLCD;	       // Magnification value LCD display
@@ -53,8 +53,9 @@ private:
 };
 
 
-/* ShowXForm displays a text or an image (QPixmap) using a coordinate
-   transformation matrix (QWMatrix)
+/*
+  ShowXForm displays a text or a pixmap (QPixmap) using a coordinate
+  transformation matrix (QWMatrix)
 */
 
 class ShowXForm : public QWidget
@@ -63,22 +64,22 @@ class ShowXForm : public QWidget
 public:
     ShowXForm( QWidget *parent=0, const char *name=0 );
    ~ShowXForm() {}
-    void showIt();		 // (Re)displays text or image
+    void showIt();			// (Re)displays text or pixmap
+    bool pixmapMode() const { return usingPixmap; }
 public slots:
     void setText( const char* );
     void setMatrix( QWMatrix );
     void setFont( const QFont &f );
     void setPixmap( QPixmap );
     void setPixmapMode( bool b );
-    bool imageMode() { return usingImage; }
 private:
     void paintEvent( QPaintEvent * );
     void resizeEvent( QResizeEvent * );
-    QString   text;		       // text to be displayed
-    QWMatrix  mtx;		       // coordinate transform matrix
-    QPixmap   img;		       // Image to be displayed
-    bool      usingImage;	       // TRUE if img is being displayed
-    QRect     eraseRect;	       // covers last displayed text/image
+    QString   text;			// text to be displayed
+    QWMatrix  mtx;			// coordinate transform matrix
+    QPixmap   pix;			// pixmap to be displayed
+    bool      usingPixmap;		// TRUE if pix is being displayed
+    QRect     eraseRect;		// covers last displayed text/pixmap
 };
 
 class FontSelect : public QDialog
@@ -96,7 +97,6 @@ private slots:
     void	 newSize( int );
     void	 slidingSize( int );
     void	 doApply();
-    void	 fileActivated( int );
     void	 familyActivated( int );
     void	 pSizeActivated( int );
 signals:
@@ -113,7 +113,7 @@ private:
     QCheckBox	 *strikeOut;
     QPushButton	 *apply;
     QPushButton	 *stat;
-    QScrollBar	 *sizeS;
+    QSlider	 *sizeS;
     QRadioButton *rb[5];
     QButtonGroup *weight;
     QGroupBox	 *mGroup;
@@ -128,11 +128,11 @@ XFormControl::XFormControl( QWidget *parent, const char *name )
 	: QFrame( parent, name )
 {
     rotLCD	= new QLCDNumber( 4, this, "rotateLCD" );
-    rotS	= new QScrollBar( QScrollBar::Horizontal, this,
-				  "rotateScrollBar" );
+    rotS	= new QSlider( QSlider::Horizontal, this,
+				  "rotateSlider" );
     shearLCD	= new QLCDNumber( 5,this, "shearLCD" );
-    shearS	= new QScrollBar( QScrollBar::Horizontal, this,
-				  "shearScrollBar" );
+    shearS	= new QSlider( QSlider::Horizontal, this,
+				  "shearSlider" );
     mirror	= new QCheckBox( this, "mirrorCheckBox" );
     textEd	= new QLineEdit( this, "text" );
     f		= new QPushButton( this, "text" );
@@ -235,21 +235,22 @@ void XFormControl::selectFontDestroyed()
     fs = 0;
 }
 
-void XFormControl::fontSelected( const QFont &f )
+void XFormControl::fontSelected( const QFont &font )
 {
     imgCheckBox->setChecked( FALSE );
-    emit newFont( f );
+    emit newFont( font );
     toggleMode();
 }
 
 /*
     Toggles between image and text mode.
 */
+
 void XFormControl::toggleMode()
 {
     if ( magS == 0 ) {
-	magS = new QScrollBar( QScrollBar::Horizontal, this,
-			       "magnifyScrollBar" );
+	magS = new QSlider( QSlider::Horizontal, this,
+			       "magnifySlider" );
 	magS->setGeometry( 10, 375, 100, 15 );
 	magS->setRange( 0, 400 );
 	magS->setValue( 100 );
@@ -278,7 +279,7 @@ ShowXForm::ShowXForm( QWidget *parent, const char *name )
 {
     setFont( QFont( "Charter", 48, QFont::Bold ) );
     setBackgroundColor( white );
-    usingImage = FALSE;
+    usingPixmap = FALSE;
     eraseRect = QRect( 0, 0, 0, 0 );
 }
 
@@ -306,33 +307,33 @@ void ShowXForm::setMatrix( QWMatrix w )
 
 void ShowXForm::setFont( const QFont &f )
 {
-    usingImage = FALSE;
+    usingPixmap = FALSE;
     QWidget::setFont( f );
 }
 
 void ShowXForm::setPixmap( QPixmap pm )
 {
-    img	       = pm;
-    usingImage = TRUE;
+    pix		= pm;
+    usingPixmap = TRUE;
     showIt();
 }
 
 void ShowXForm::setPixmapMode( bool enable )
 {
-    usingImage = enable;
+    usingPixmap = enable;
 }
 
 void ShowXForm::showIt()
 {
     QPainter p;
-    QRect r;	  // rectangle covering new text/image in virtual coordinates
+    QRect r;	  // rectangle covering new text/pixmap in virtual coordinates
     QWMatrix m;	  // copy user specified transform
     int textYPos = 0; // distance from boundingRect y pos to baseline
     int textXPos = 0; // distance from boundingRect x pos to text start
     QRect br;
     QFontMetrics fm( fontMetrics() );	// get widget font metrics
-    if ( imageMode() ) {
-	r = img.rect();
+    if ( pixmapMode() ) {
+	r = pix.rect();
     } else {
 	br = fm.boundingRect( text );	// rectangle covering text
 	r  = br;
@@ -343,20 +344,20 @@ void ShowXForm::showIt()
     r.moveTopLeft( QPoint(-r.width()/2, -r.height()/2) );
 	  // compute union of new and old rect
 	  // the resulting rectangle will cover what is already displayed
-	  // and have room for the new text/image
+	  // and have room for the new text/pixmap
     eraseRect = eraseRect.unite( mtx.map(r) );
     eraseRect.moveBy( -1, -1 ); // add border for matrix round off
     eraseRect.setSize( QSize( eraseRect.width() + 2,eraseRect.height() + 2 ) );
     int pw = QMIN(eraseRect.width(),width());
     int ph = QMIN(eraseRect.height(),height());
-    QPixmap pix( pw, ph );		// off-screen drawing pixmap
-    pix.fill( backgroundColor() );
-    if ( imageMode() ) {
-	QPixmap rotated = img.xForm(mtx);
-	bitBlt( &pix, pix.width()/2 - rotated.width()/2,
-		pix.height()/2 - rotated.height()/2, &rotated );
+    QPixmap pm( pw, ph );		// off-screen drawing pixmap
+    pm.fill( backgroundColor() );
+    if ( pixmapMode() ) {
+	QPixmap rotated = pix.xForm(mtx);
+	bitBlt( &pm, pm.width()/2 - rotated.width()/2,
+		pm.height()/2 - rotated.height()/2, &rotated );
     } else {
-	p.begin( &pix );
+	p.begin( &pm );
 	p.setFont( font() );		// use widget font
 	m.translate( pw/2, ph/2 );	// 0,0 is center
 	m = mtx * m;
@@ -371,15 +372,16 @@ void ShowXForm::showIt()
     int xpos = width()/2  - pw/2;
     int ypos = height()/2 - ph/2;
     bitBlt( this, xpos, ypos,			// copy pixmap to widget
-	    &pix, 0, 0, -1, -1 );
+	    &pm, 0, 0, -1, -1 );
     eraseRect =	 mtx.map( r );
 }
 
+
 FontSelect::FontSelect( QWidget *parent, const char *name)
     : QDialog( parent, name, 0 ), f( "Charter", 48, QFont::Bold )
 {
     static const char *radios[] = {
-    	"Light (25)", "Normal (50)", "DemiBold (63)",
+	"Light (25)", "Normal (50)", "DemiBold (63)",
 	"Bold (75)", "Black (87)"
     };
     int i;
@@ -396,8 +398,8 @@ FontSelect::FontSelect( QWidget *parent, const char *name)
     underline  = new QCheckBox(	 this, "underline" );
     strikeOut  = new QCheckBox(	 this, "strikeOut" );
     apply      = new QPushButton( this, "apply" );
-    sizeS      = new QScrollBar( QScrollBar::Horizontal, this,
-			      "pointSizeScrollBar" );
+    sizeS      = new QSlider( QSlider::Horizontal, this,
+			      "pointSizeSlider" );
 
     familyL->setGeometry( 10, yOff + 10, 100,20 );
     familyL->setText( "Family :" );
@@ -455,19 +457,15 @@ FontSelect::FontSelect( QWidget *parent, const char *name)
 	"Symbol", "Terminal", "Times", "Utopia", "Arial", 0
     };
 
-    static const char *pSizes[]  = {
+    static const char *pSizes[]	 = {
 	"8", "10", "12", "14", "18", "24", "36", "48", "72", "96", 0
     };
 
-
-    QPopupMenu *file = new QPopupMenu;
-    file->insertItem( "Quit\tC-x" );
-
-    QPopupMenu *family = new QPopupMenu;
+    QPopupMenu *familyPopup = new QPopupMenu;
     const char **tmp;
     tmp = familys;
     while( *tmp )
-	family->insertItem( *tmp++ );
+	familyPopup->insertItem( *tmp++ );
 
     QPopupMenu *pSize = new QPopupMenu;
     tmp = pSizes;
@@ -477,12 +475,10 @@ FontSelect::FontSelect( QWidget *parent, const char *name)
     menu = new QMenuBar( this );
     menu->move( 0, 0 );
     menu->resize( 350, 30 );
-    menu->insertItem( "File", file );
-    menu->insertItem( "Family", family );
+    menu->insertItem( "Family", familyPopup );
     menu->insertItem( "Point size", pSize );
 
-    connect( file, SIGNAL(activated(int)), SLOT(fileActivated(int)) );
-    connect( family, SIGNAL(activated(int)), SLOT(familyActivated(int)) );
+    connect( familyPopup, SIGNAL(activated(int)), SLOT(familyActivated(int)) );
     connect( pSize, SIGNAL(activated(int)), SLOT(pSizeActivated(int)) );
 
     static const char *mLabelStr[] = {
@@ -518,7 +514,6 @@ void FontSelect::doFont()
     xyz.setPointSize( f.pointSize() );
     fontInternal->setFont( xyz );
     updateMetrics();
-    emit newFont( xyz );
 }
 
 void FontSelect::newUnderline()
@@ -593,11 +588,8 @@ void FontSelect::doApply()
     }
     f.setFamily( family->text() );
     doFont();
-}
 
-void FontSelect::fileActivated( int )
-{
-    qApp->quit();
+    emit newFont( fontInternal->font() );
 }
 
 void FontSelect::familyActivated( int id )
@@ -621,7 +613,7 @@ void FontSelect::updateMetrics()
     metrics[0][1]->setText( fi.family() );
     metrics[1][1]->setNum( fi.pointSize() );
     metrics[2][1]->setNum( fi.weight() );
-    metrics[3][1]->setNum( fi.italic() );
+    metrics[3][1]->setNum( (int)fi.italic() );
 }
 
 /*
@@ -653,7 +645,7 @@ void XFormCenter::newMode( bool showPix )
 {
     static bool first = TRUE;
 
-    if ( sx->imageMode() == showPix )
+    if ( sx->pixmapMode() == showPix )
 	return;
     if ( showPix && first ) {
 	first = FALSE;
@@ -679,7 +671,7 @@ XFormCenter::XFormCenter( QWidget *parent, const char *name )
     connect( xc, SIGNAL(newMatrix(QWMatrix)),
 	     sx, SLOT(setMatrix(QWMatrix)) );
     connect( xc, SIGNAL(newFont(const QFont&)), sx,
-	         SLOT(setFont(const QFont&)) );
+		 SLOT(setFont(const QFont&)) );
     connect( xc, SIGNAL(newMode(bool)), SLOT(newMode(bool)) );
     sx->setText( "Troll" );
 }
