@@ -1,5 +1,5 @@
 /****************************************************************************
-** $Id: qbuffer.h,v 2.0 1996/09/24 10:16:08 agulbra Exp $
+** $Id: qbuffer.h,v 2.2 1997/08/06 15:05:49 hanord Exp $
 **
 ** Definition of QBuffer class
 **
@@ -7,13 +7,16 @@
 **
 ** Copyright (C) 1992-1997 Troll Tech AS.  All rights reserved.
 **
-** This file is part of the non-commercial distribution of Qt 1.2.
+** This file is part of the free edition of Qt 1.33.
 **
 ** See the file LICENSE included in the distribution for the usage
 ** and distribution terms, or http://www.troll.no/qt/license.html.
 **
-** (This file differs from the one in the commercial distribution of
-** Qt only by this comment.)
+** IMPORTANT NOTE: You may NOT copy this file or any part of it into
+** your own programs or libraries.
+**
+** (This file differs from the one in the commercial edition of Qt
+** only by this comment.)
 **
 *****************************************************************************/
 
@@ -31,7 +34,7 @@ public:
     QBuffer( QByteArray );
    ~QBuffer();
 
-    QByteArray buffer() const { return a; }
+    QByteArray buffer() const;
     bool  setBuffer( QByteArray );
 
     bool  open( int );
@@ -63,6 +66,9 @@ private:	// Disabled copy constructor and operator=
 };
 
 
+inline QByteArray QBuffer::buffer() const
+{ return a; }
+
 inline uint QBuffer::size() const
 { return a.size(); }
 
