@@ -1,5 +1,5 @@
 /****************************************************************************
-** $Id: qevent.h,v 2.2 1997/03/22 16:20:02 hanord Exp $
+** $Id: qevent.h,v 2.5 1997/09/08 14:16:57 hanord Exp $
 **
 ** Definition of event classes
 **
@@ -7,13 +7,16 @@
 **
 ** Copyright (C) 1992-1997 Troll Tech AS.  All rights reserved.
 **
-** This file is part of the non-commercial distribution of Qt 1.2.
+** This file is part of the free edition of Qt 1.33.
 **
 ** See the file LICENSE included in the distribution for the usage
 ** and distribution terms, or http://www.troll.no/qt/license.html.
 **
-** (This file differs from the one in the commercial distribution of
-** Qt only by this comment.)
+** IMPORTANT NOTE: You may NOT copy this file or any part of it into
+** your own programs or libraries.
+**
+** (This file differs from the one in the commercial edition of Qt
+** only by this comment.)
 **
 *****************************************************************************/
 
@@ -33,6 +36,7 @@
 #define Event_KeyPress		    6		// key pressed
 #define Event_KeyRelease	    7		// key released
 #define Event_FocusIn		    8		// keyboard focus received
+#define Event_FocusRestore	   21		// restore original focus
 #define Event_FocusOut		    9		// keyboard focus lost
 #define Event_Enter		   10		// mouse enters widget
 #define Event_Leave		   11		// mouse leaves widget
@@ -48,6 +52,9 @@
 #define Event_Accel		   30		// accelerator event
 #define Event_Clipboard		   40		// internal clipboard event
 #define Event_SockAct		   50		// socket activation
+#define Event_DragMove		   60		// drag moves (into) widget
+#define Event_DragLeave		   61		// drag leaves or is cancelled
+#define	Event_Drop		   62		// actual drop
 #define Event_User		 1000		// first user event id
 
 
