@@ -1,5 +1,5 @@
 /****************************************************************************
-** $Id: qfont.h,v 2.3 1997/03/14 16:50:44 hanord Exp $
+** $Id: qfont.h,v 2.6 1997/09/10 14:36:35 agulbra Exp $
 **
 ** Definition of QFont class
 **
@@ -7,13 +7,16 @@
 **
 ** Copyright (C) 1992-1997 Troll Tech AS.  All rights reserved.
 **
-** This file is part of the non-commercial distribution of Qt 1.2.
+** This file is part of the free edition of Qt 1.33.
 **
 ** See the file LICENSE included in the distribution for the usage
 ** and distribution terms, or http://www.troll.no/qt/license.html.
 **
-** (This file differs from the one in the commercial distribution of
-** Qt only by this comment.)
+** IMPORTANT NOTE: You may NOT copy this file or any part of it into
+** your own programs or libraries.
+**
+** (This file differs from the one in the commercial edition of Qt
+** only by this comment.)
 **
 *****************************************************************************/
 
@@ -40,11 +43,21 @@ public:
 		     Decorative = OldEnglish};
     enum Weight	   { Light = 25, Normal = 50, DemiBold = 63,
 		     Bold  = 75, Black	= 87 };
-    enum CharSet   { Latin1, ISO_8859_1 = Latin1, AnyCharSet };
+    enum CharSet   { Latin1, ISO_8859_1 = Latin1, AnyCharSet,
+		     Latin2, ISO_8859_2 = Latin2,
+		     Latin3, ISO_8859_3 = Latin3,
+		     Latin4, ISO_8859_4 = Latin4,
+		     Latin5, ISO_8859_5 = Latin5,
+		     Latin6, ISO_8859_6 = Latin6,
+		     Latin7, ISO_8859_7 = Latin7,
+		     Latin8, ISO_8859_8 = Latin8,
+		     Latin9, ISO_8859_9 = Latin9 };
 
     QFont();					// default font
     QFont( const char *family, int pointSize = 12,
 	   int weight = Normal, bool italic = FALSE );
+    QFont( const char *family, int pointSize,
+	   int weight, bool italic, CharSet charSet );
     QFont( const QFont & );
     virtual ~QFont();
     QFont      &operator=( const QFont & );
