@@ -1,5 +1,5 @@
 /**********************************************************************
-** $Id: qlined.h,v 2.7 1997/03/02 12:32:05 paul Exp $
+** $Id: qlined.h,v 2.18 1997/09/22 17:38:25 paul Exp $
 **
 ** Definition of QLineEdit widget class
 **
@@ -7,13 +7,16 @@
 **
 ** Copyright (C) 1992-1997 Troll Tech AS.  All rights reserved.
 **
-** This file is part of the non-commercial distribution of Qt 1.2.
+** This file is part of the free edition of Qt 1.33.
 **
 ** See the file LICENSE included in the distribution for the usage
 ** and distribution terms, or http://www.troll.no/qt/license.html.
 **
-** (This file differs from the one in the commercial distribution of
-** Qt only by this comment.)
+** IMPORTANT NOTE: You may NOT copy this file or any part of it into
+** your own programs or libraries.
+**
+** (This file differs from the one in the commercial edition of Qt
+** only by this comment.)
 **
 *****************************************************************************/
 
@@ -24,7 +27,7 @@
 #include "qstring.h"
 
 class QComboBox;
-
+class QValidator;
 
 class QLineEdit : public QWidget
 {
@@ -41,14 +44,21 @@ public:
     bool	frame() const;
 
     enum	EchoMode { Normal, NoEcho, Password };
-    void	setEchoMode( enum EchoMode );
-    QLineEdit::EchoMode echoMode() const;
+    void	setEchoMode( EchoMode );
+    EchoMode 	echoMode() const;
+
+    void	setValidator( QValidator * );
+    QValidator * validator() const;
+
+    QSize	sizeHint() const;
 
 public slots:
     void	setText( const char * );
     void	selectAll();
     void	deselect();
 
+    void	clearValidator();
+
 signals:
     void	textChanged( const char * );
     void	returnPressed();
@@ -72,8 +82,8 @@ private slots:
     void	clipboardChanged();
 
 private:
-    void	paint( bool frame = FALSE );
-    void	pixmapPaint();
+    void	paint( const QRect& clip, bool frame = FALSE );
+    void	pixmapPaint( const QRect& clip );
     void	paintText( QPainter *, const QSize &, bool frame = FALSE );
     void	cursorLeft( bool mark, int steps = 1 );
     void	cursorRight( bool mark, int steps = 1 );
