@@ -1,5 +1,5 @@
 /****************************************************************************
-** $Id: qpicture.h,v 2.0 1996/09/24 10:15:57 agulbra Exp $
+** $Id: qpicture.h,v 2.2 1997/08/06 15:05:41 hanord Exp $
 **
 ** Definition of QPicture class
 **
@@ -7,13 +7,16 @@
 **
 ** Copyright (C) 1992-1997 Troll Tech AS.  All rights reserved.
 **
-** This file is part of the non-commercial distribution of Qt 1.2.
+** This file is part of the free edition of Qt 1.33.
 **
 ** See the file LICENSE included in the distribution for the usage
 ** and distribution terms, or http://www.troll.no/qt/license.html.
 **
-** (This file differs from the one in the commercial distribution of
-** Qt only by this comment.)
+** IMPORTANT NOTE: You may NOT copy this file or any part of it into
+** your own programs or libraries.
+**
+** (This file differs from the one in the commercial edition of Qt
+** only by this comment.)
 **
 *****************************************************************************/
 
@@ -30,6 +33,12 @@ public:
     QPicture();
    ~QPicture();
 
+    bool	isNull() const;
+
+    uint	size() const;
+    const char *data() const;
+    void	setData( const char *data, uint size );
+
     bool	play( QPainter * );
 
     bool	load( const char *fileName );
@@ -51,4 +60,20 @@ private:	// Disabled copy constructor and operator=
 };
 
 
+inline bool QPicture::isNull() const
+{
+    return pictb.buffer().isNull();
+}
+
+inline uint QPicture::size() const
+{
+    return pictb.buffer().size();
+}
+
+inline const char *QPicture::data() const
+{
+    return pictb.buffer().data();
+}
+
+
 #endif // QPICTURE_H
