@@ -1,5 +1,5 @@
 /****************************************************************************
-** $Id: qregion.h,v 2.1 1996/11/22 18:31:54 agulbra Exp $
+** $Id: qregion.h,v 2.5 1997/10/09 13:11:05 agulbra Exp $
 **
 ** Definition of QRegion class
 **
@@ -7,13 +7,16 @@
 **
 ** Copyright (C) 1992-1997 Troll Tech AS.  All rights reserved.
 **
-** This file is part of the non-commercial distribution of Qt 1.2.
+** This file is part of the free edition of Qt 1.33.
 **
 ** See the file LICENSE included in the distribution for the usage
 ** and distribution terms, or http://www.troll.no/qt/license.html.
 **
-** (This file differs from the one in the commercial distribution of
-** Qt only by this comment.)
+** IMPORTANT NOTE: You may NOT copy this file or any part of it into
+** your own programs or libraries.
+**
+** (This file differs from the one in the commercial edition of Qt
+** only by this comment.)
 **
 *****************************************************************************/
 
@@ -49,7 +52,24 @@ public:
     QRegion unite( const QRegion & )	const;
     QRegion intersect( const QRegion &) const;
     QRegion subtract( const QRegion & ) const;
+
+// Work around clash with the ANSI C++ keyword "xor".
+//
+// Use of QRegion::xor() is deprecated - you should use QRegion::eor().
+// Calls to QRegion::xor() will work for now, but give a warning.
+//
+// If possible, compile the Qt library without this ANSI C++ feature enabled,
+// thus including both the old xor() and new eor() in the library, so old
+// binaries will continue to work (with the warning).
+//
+// We also hide the xor() function if there is a #define for xor, in
+// case someone is using #define xor ^ to work around deficiencies in
+// their compiler that cause problems with some other header files.
+//
+#if !(defined(__STRICT_ANSI__) && defined(_CC_GNU_)) && !defined(_CC_EDG_) && !defined(_CC_HP_) && !defined(_CC_HP_ACC_) && !defined(xor)
     QRegion xor( const QRegion & )	const;
+#endif
+    QRegion eor( const QRegion & )	const;
 
     bool    operator==( const QRegion & )  const;
     bool    operator!=( const QRegion &r ) const
