@@ -1,5 +1,5 @@
 /****************************************************************************
-** $Id: qstring.h,v 2.5 1997/03/26 18:58:58 hanord Exp $
+** $Id: qstring.h,v 2.8 1997/08/06 15:05:54 hanord Exp $
 **
 ** Definition of extended char array operations, and QByteArray and
 ** QString classes
@@ -8,13 +8,16 @@
 **
 ** Copyright (C) 1992-1997 Troll Tech AS.  All rights reserved.
 **
-** This file is part of the non-commercial distribution of Qt 1.2.
+** This file is part of the free edition of Qt 1.33.
 **
 ** See the file LICENSE included in the distribution for the usage
 ** and distribution terms, or http://www.troll.no/qt/license.html.
 **
-** (This file differs from the one in the commercial distribution of
-** Qt only by this comment.)
+** IMPORTANT NOTE: You may NOT copy this file or any part of it into
+** your own programs or libraries.
+**
+** (This file differs from the one in the commercial edition of Qt
+** only by this comment.)
 **
 *****************************************************************************/
 
@@ -149,6 +152,7 @@ public:
     QString( int size );			// allocate size incl. \0
     QString( const QString &s ) : QByteArray( s ) {}
     QString( const char *str );			// deep copy
+    QString( const char *str, uint maxlen );	// deep copy, max length
 
     QString    &operator=( const QString &s );	// shallow copy
     QString    &operator=( const char *str );	// deep copy
