@@ -1,5 +1,5 @@
 /****************************************************************************
-** $Id: qapp.cpp,v 2.10 1997/02/26 14:16:33 hanord Exp $
+** $Id: qapp.cpp,v 2.35 1997/09/26 17:37:56 warwick Exp $
 **
 ** Implementation of QApplication class
 **
@@ -7,13 +7,16 @@
 **
 ** Copyright (C) 1992-1997 Troll Tech AS.  All rights reserved.
 **
-** This file is part of the non-commercial distribution of Qt 1.2.
+** This file is part of the free edition of Qt 1.33.
 **
 ** See the file LICENSE included in the distribution for the usage
 ** and distribution terms, or http://www.troll.no/qt/license.html.
 **
-** (This file differs from the one in the commercial distribution of
-** Qt only by this comment.)
+** IMPORTANT NOTE: You may NOT copy this file or any part of it into
+** your own programs or libraries.
+**
+** (This file differs from the one in the commercial edition of Qt
+** only by this comment.)
 **
 *****************************************************************************/
 
@@ -23,7 +26,7 @@
 #include "qwidcoll.h"
 #include "qpalette.h"
 
-RCSTAG("$Id: qapp.cpp,v 2.10 1997/02/26 14:16:33 hanord Exp $");
+RCSTAG("$Id: qapp.cpp,v 2.35 1997/09/26 17:37:56 warwick Exp $");
 
 
 /*!
@@ -74,11 +77,15 @@ RCSTAG("$Id: qapp.cpp,v 2.10 1997/02/26 14:16:33 hanord Exp $");
 
 void qt_init( int *, char ** );			// defined in qapp_xyz.cpp
 void qt_cleanup();
+#if defined(_WS_X11_)
+void qt_init( Display* dpy );			// defined in qapp_x11.cpp
+#endif
 
 QApplication *qApp = 0;				// global application object
 QPalette *QApplication::app_pal	       = 0;	// default application palette
 QFont	 *QApplication::app_font       = 0;	// default application font
 QCursor	 *QApplication::app_cursor     = 0;	// default application cursor
+int	  QApplication::app_tracking   = 0;	// global mouse tracking
 bool	  QApplication::is_app_running = FALSE;	// app starting up if FALSE
 bool	  QApplication::is_app_closing = FALSE;	// app closing down if TRUE
 int	  QApplication::loop_level     = 0;	// event loop level
@@ -86,12 +93,8 @@ QWidget	 *QApplication::main_widget    = 0;	// main application widget
 QWidget	 *QApplication::focus_widget   = 0;	// has keyboard input focus
 
 
-#if defined(_WS_MAC_)
-GUIStyle QApplication::app_style = MacStyle;	// default style for Mac
-#elif defined(_WS_WIN_)
+#if defined(_WS_WIN_)
 GUIStyle QApplication::app_style = WindowsStyle;// default style for Windows
-#elif defined(_WS_PM_)
-GUIStyle QApplication::app_style = PMStyle;	// default style for OS/2 PM
 #elif defined(_WS_X11_)
 GUIStyle QApplication::app_style = MotifStyle;	// default style for X Windows
 #endif
@@ -99,25 +102,31 @@ GUIStyle QApplication::app_style = MotifStyle;	// default style for X Windows
 int	 QApplication::app_cspec = QApplication::NormalColor;
 
 
-static QPalette *motifPalette = 0;
+static QPalette *stdPalette = 0;
+static QColor winHighlightColor = darkBlue;
+static int mouseDoubleClickInterval = 400;
+
 
 static void create_palettes()			// creates default palettes
 {
-    QColorGroup motif_nor( black, lightGray,
-			   white, lightGray.dark(), gray,
-			   black, white );
-    QColorGroup motif_dis( darkGray, lightGray,
-			   white, lightGray.dark(), gray,
-			   darkGray, motif_nor.background() );
-    QColorGroup motif_act( black, lightGray,
-			   white, lightGray.dark(), gray,
-			   black, white );
-    motifPalette = new QPalette( motif_nor, motif_dis, motif_act );
+    QColor standardLightGray( 192, 192, 192 );
+    QColor light( 255, 255, 255 );
+    QColor dark( standardLightGray.dark() );
+    QColorGroup std_nor( black, standardLightGray,
+			 light, dark, gray,
+			 black, white );
+    QColorGroup std_dis( darkGray, standardLightGray,
+			 light, dark, gray,
+			 darkGray, std_nor.background() );
+    QColorGroup std_act( black, standardLightGray,
+			 light, dark, gray,
+			 black, white );
+    stdPalette = new QPalette( std_nor, std_dis, std_act );
 }
 
 static void destroy_palettes()
 {
-    delete motifPalette;
+    delete stdPalette;
 }
 
 
@@ -160,6 +169,13 @@ static void destroy_palettes()
   <li> \c -title \e title, sets the application title (caption).
   <li> \c -style= \e style, sets the application GUI style. Possible values
        are \c motif and \c windows 
+  <li> \c -visual \c TrueColor, forces the application to use a TrueColor visual
+       on an 8-bit display.
+  <li> \c -ncols \e count, limits the number of colors allocated in the
+       color cube on a 8-bit display, if the application is using the
+       \c QApplication::ManyColor color specification.
+  <li> \c -cmap, causes the application to install a private color map
+       on an 8-bit display.
   </ul>
 
   \sa argc(), argv()
@@ -173,6 +189,36 @@ QApplication::QApplication( int &argc, char **argv )
 #endif
     qApp = this;
     qt_init( &argc, argv );
+    initialize( argc, argv );
+}
+
+
+#if defined(_WS_X11_)
+
+/*!
+  Create an application, given an already open display.  X11 only.
+*/
+
+QApplication::QApplication( Display* dpy )
+{
+#if defined(CHECK_STATE)
+    if ( qApp )
+	warning( "QApplication: There should be only one application object" );
+#endif
+    qApp = this;
+    qt_init( dpy );
+    initialize( 0, 0 );
+}
+
+#endif // _WS_X11_
+
+
+/*!
+  Initializes the QApplication object, called from the constructors.
+*/
+
+void QApplication::initialize( int argc, char **argv )
+{
     initMetaObject();
     app_argc = argc;
     app_argv = argv;
@@ -180,7 +226,7 @@ QApplication::QApplication( int &argc, char **argv )
     quit_code = 0;
     if ( !app_pal ) {				// palette not already set
 	create_palettes();
-	app_pal = new QPalette( *motifPalette );
+	app_pal = new QPalette( *stdPalette );
 	CHECK_PTR( app_pal );
     }
     if ( !app_font ) {				// font not already set
@@ -191,9 +237,11 @@ QApplication::QApplication( int &argc, char **argv )
     is_app_running = TRUE;			// no longer starting up
 }
 
+
 /*!
-  Closes all widgets and cleans up all window system resources.
-  Sets \c qApp to 0.
+  Deletes all remaining widgets and cleans up any window system
+  resources that were allocated by this application.  Sets the global \c
+  qApp to null.
 */
 
 QApplication::~QApplication()
@@ -271,9 +319,7 @@ QApplication::~QApplication()
 /*!
   Sets the application GUI style to \e style.
 
-  The style parameter can be \c MacStyle, \c WindowsStyle, \c PMStyle
-  or \c MotifStyle.  Only \c MotifStyle and \c WindowsStyle is
-  supported in the current version of Qt.
+  The style parameter can be \c WindowsStyle or \c MotifStyle.
 
   \sa style(), QWidget::setStyle()
 */
@@ -317,51 +363,49 @@ int QApplication::colorSpec()
 
   The choices are:
   <ul>
-  <li> \c QApplication::NormalColor. This is the default color allocation
-  strategy. The application allocates system global colors. This work fine
-  for most applications under X11, but Windows dithers to the 20 standard
-  colors unless the display has true color support (more than 256 colors).
+  <li> \c QApplication::NormalColor.
+    This is the default color allocation strategy.
+    Use this choice if your application uses buttons, menus,
+    texts and pixmaps with few colors.
+    With this choice, the application allocates system global colors.
+    This work fine for most applications under X11, but Windows dithers to
+    the 20 standard colors unless the display has true color support (more
+    than 256 colors).
 
-  <li> \c QApplication::CustomColor. Under X11 this is the same as \c
-  NormalColor. Under Windows, Qt creates a Windows palette if the display
-  supports 256 colors.
+  <li> \c QApplication::CustomColor.
+    Use this choice if your application needs a small number of
+    custom colors.  This choice only makes a difference on Windows
+    - the application gets more colors when it is active, but the
+    background windows look less good.
+    Under X11 this is the same as \c
+    NormalColor. Under Windows, Qt creates a Windows palette if the display
+    supports 256 colors.
 
-  <li> \c QApplication::PrivateColor. Under Windows this is the same as \c
-  CustomColor. Under X11, Qt uses a private colormap for the application.
-
-  <li> \c QApplication::TrueColor. Under Windows, this is equal to
-  \c NormalColor. Under X11, this option makes the application use
-  a true color visual if one exists but is not the default visual.
-  Silicon Graphics X servers have this feature. They provide an 8
-  bit visual as default but can deliver true color when asked.
+  <li> \c QApplication::ManyColor.
+    Use this choice if your application is very color hungry
+    (e.g. it wants thousands of colors).
+    Under Windows, this is equal to \c CustomColor.
+    Under X11 the effect is:
+    <ul>
+      <li> For 256-color displays which have at best a 256 color true color
+	    visual, the default visual is used, and a color cube is
+	    preallocated. The number of colors in the cube is as many
+	    as possible, but can be forced lower by the \e -ncols option.
+	    The user can force the application to use the true color visual by
+	    the \link QApplication::QApplication() <em>-visual</em> \endlink
+	    option.
+      <li> For 256-color displays which have a true color visual with more
+	    than 256 colors, use that visual.  Silicon Graphics X servers
+	    have this feature. They provide an 8 bit visual as default but
+	    can deliver true color when asked. 
+    </ul>
   </ul>
 
-  The settings are bit-coded and can be combined using | or +.
-
-  If you have an application that uses buttons, menus, texts and
-  pixmaps with few colors, you can probably use the \c NormalColor
-  specification (i.e. keep the default setting).
-
-  If your application needs some custom colors you should specify \c
-  CustomColor.  This only makes a difference on Windows - the application
-  gets more colors when it is active, but the background windows look
-  less good.
-
-  If the application is color hungry (e.g. it wants 200 colors), \c
-  PrivateColor is best.  On Windows, it is the same as \c CustomColor.
-  Under X, you get a private colormap.
-
-  If your application is very color hungry (e.g. it wants thousands of
-  colors), you can specify a combination such as \c
-  (TrueColor+PrivateColor) or \c (TrueColor+CustomColor).  These
-  combinations mean to use 24-bit color if that is available, and fall
-  back to \c PrivateColor or \c CustomColor respectively if not.
-  
   Example:
   \code
   int main( int argc, char **argv )
   {
-      QApplication::setColorSpec( QApplication::PrivateColor );
+      QApplication::setColorSpec( QApplication::ManyColor );
       QApplication a( argc, argv );
       ...
   }
@@ -373,12 +417,10 @@ int QApplication::colorSpec()
 
   To see what mode you end up with, you can call QColor::numBitPlanes()
   once the QApplication object exists.  A value greater than 8 (typically
-  16,24 or 32) means true color.
+  16, 24 or 32) means true color.
 
-  \warning The X.h header file for X11 contains <code>#define TrueColor
-  </code>. If you include the X headers files (which is necessary only
-  when you bypass Qt and write Xlib-specific code), you must <code>#undef
-  TrueColor</code> to be able to specify \c QApplication::TrueColor.
+  The color cube used by Qt are all those colors with red, green, and blue
+  components of either 0x00, 0x33, 0x66, 0x99, 0xCC, or 0xFF.
 
   \sa colorSpec(), QColor::numBitPlanes(), QColor::enterAllocContext()
 */
@@ -479,6 +521,8 @@ void QApplication::setFont( const QFont &font,	bool updateAllWidgets )
 /*!
   Returns a list of the top level widgets in the application.
 
+  The list is created using new and must be deleted by the caller.
+
   The list is \link QList::isEmpty() empty \endlink if there are no
   top level widgets.
 
@@ -506,7 +550,8 @@ void QApplication::setFont( const QFont &font,	bool updateAllWidgets )
   You can get in serious trouble if you for instance try to access
   a widget that has been deleted.
 
-  \sa QWidget::isTopLevel(), QWidget::isVisible(), QList::isEmpty()
+  \sa allWidgets(), QWidget::isTopLevel(), QWidget::isVisible(),
+      QList::isEmpty()
 */
 
 QWidgetList *QApplication::topLevelWidgets()
@@ -514,6 +559,45 @@ QWidgetList *QApplication::topLevelWidgets()
     return QWidget::tlwList();
 }
 
+/*!
+  Returns a list of all the widgets in the application.
+
+  The list is created using new and must be deleted by the caller.
+
+  The list is \link QList::isEmpty() empty \endlink if there are no
+  widgets.
+
+  Note that some of the widgets may be hidden.
+
+  Example:
+  \code
+    //
+    // Updates all widgets.
+    //
+    QWidgetList	 *list = QApplication::allWidgets();
+    QWidgetListIt it( *list );		// iterate over the widgets
+    while ( it.current() ) {		// for each top level widget...
+        it.current()->update();
+	++it;
+    }
+    delete list;			// delete the list, not the widgets
+  \endcode
+
+  The QWidgetList class is defined in the qwidcoll.h header file.
+
+  \warning
+  Delete the list away as soon you have finished using it.
+  You can get in serious trouble if you for instance try to access
+  a widget that has been deleted.
+
+  \sa topLevelWidgets(), QWidget::isVisible(), QList::isEmpty(), 
+*/
+
+QWidgetList *QApplication::allWidgets()
+{
+    return QWidget::wList();
+}
+
 
 /*!
   \fn QWidget *QApplication::focusWidget() const
@@ -553,7 +637,7 @@ void QApplication::exit( int retcode )
 	return;
     if ( qApp->quit_now )			// don't overwrite quit code
 	return;
-    qApp->quit_now = TRUE;
+    qApp->quit_now  = TRUE;
     qApp->quit_code = retcode;
 }
 
@@ -606,12 +690,20 @@ void QApplication::quit()
   Sends \e event to \e receiver: <code>receiver->event( event )</code>
   Returns the value that is returned from the receiver's event handler.
 
-  All Qt events are sent using the notify function. Since this function
-  is virtual, you can make a subclass of QApplication and reimplement
-  notify() to get total control of Qt events.
-
-  Installing an event filter on \c qApp is another way of making an
-  application-global event hook.
+  Reimplementing this virtual function is one of five ways to process
+  an event: <ol> <li> Reimplementing this function.  Very powerful,
+  you get \e complete control, but of course only one subclass can be
+  qApp.  <li> Installing an event filter on qApp.  Such an event
+  filter gets to process all events for all widgets, so it's just as
+  powerful as reimplementing notify(), and in this way it's possible
+  to have more than one application-global event filter. <li>
+  Reimplementing QObject::event() (as QWidget does).  If you do this
+  you get tab key-presses, and you get to see the events before any
+  widget-specific event filters.  <li> Installing an event filter on
+  the object.  Such an even filter gets all the events except Tab and
+  Shift-Tab key presses. <li> Finally, reimplementing paintEvent(),
+  mousePressEvent() and so on.  This is the normal, easist and least
+  powerful way. </ol>
 
   \sa QObject::event(), installEventFilter()
 */
@@ -624,6 +716,7 @@ bool QApplication::notify( QObject *receiver, QEvent *event )
 #endif
 	return FALSE;
     }
+
     if ( eventFilters ) {
 	QObjectListIt it( *eventFilters );
 	register QObject *obj = it.current();
@@ -634,6 +727,14 @@ bool QApplication::notify( QObject *receiver, QEvent *event )
 	    obj = it.current();
 	}
     }
+
+    // throw away any mouse-tracking-only mouse events
+    if ( event->type() == Event_MouseMove &&
+	 (((QMouseEvent*)event)->state() & MouseButtonMask) == 0 &&
+	 ( receiver->isWidgetType() &&
+	   !((QWidget *)receiver)->hasMouseTracking() ) )
+	return TRUE;
+
     return receiver->event( event );
 }
 
@@ -658,6 +759,32 @@ bool QApplication::closingDown()
     return is_app_closing;
 }
 
+/*!
+  Processes pending events, for 3 seconds or until there
+  are no more events to process, then return.
+
+  You can call this function occasionally when your program is busy doing a
+  long operation (e.g. copying a file).
+*/
+
+void QApplication::processEvents()
+{
+    processEvents( 3000 );
+}
+
+/*!
+  Waits for an event to occur, processes it, then returns.
+
+  This function is useful for adapting Qt to situations where the event
+  processing must be grafted into existing program loops.  Beware
+  that using this function in new applications may be an indication
+  of design problems.
+*/
+void QApplication::processOneEvent()
+{
+    processNextEvent(TRUE);
+}
+
 
 #if !defined(_WS_X11_)
 
@@ -668,3 +795,70 @@ void QApplication::flushX()	{}		// do nothing
 void QApplication::syncX()	{}		// do nothing
 
 #endif
+
+
+
+/*!
+  Sets the color used to mark selections in windows style for all widgets
+  in the application. Will repaint all widgets if the color is changed.
+
+  The default color is \c darkBlue.
+  \sa winStyleHighlightColor()
+*/
+
+void QApplication::setWinStyleHighlightColor( const QColor &c )
+{
+    if ( winHighlightColor == c )
+	return;
+
+    winHighlightColor = c;
+
+    if ( is_app_running && !is_app_closing ) {
+	QWidgetIntDictIt it( *((QWidgetIntDict*)QWidget::mapper) );
+	register QWidget *w;
+	while ( (w=it.current()) ) {		// for all widgets...
+	    ++it;
+	    if ( w->style() == WindowsStyle &&
+		 !w->testWFlags(WType_Desktop) )// (except desktop)
+		w->repaint( FALSE );
+	}
+    }
+}
+
+
+/*!
+  Returns the color used to mark selections in windows style.
+  \sa setWinStyleHighlightColor()
+*/
+const QColor& QApplication::winStyleHighlightColor()
+{
+    return winHighlightColor;
+}
+
+
+/*!
+  Sets the time limit that distinguishes a double click from two
+  consecutive mouse clicks to \a ms milliseconds. This value is
+  ignored under Windows (the control panel value is used.)
+
+  The default value is 400 milliseconds.
+
+  \sa doubleClickInterval()
+*/
+
+void QApplication::setDoubleClickInterval( int ms )
+{
+    mouseDoubleClickInterval = ms;
+}
+
+
+/*!
+  Returns the maximum duration for a double click.
+
+  \sa setDoubleClickInterval()
+*/
+
+int QApplication::doubleClickInterval()
+{
+    return mouseDoubleClickInterval;
+}
