@@ -1,5 +1,5 @@
 /****************************************************************************
-** $Id: qapp.h,v 2.10 1997/03/14 16:50:43 hanord Exp $
+** $Id: qapp.h,v 2.22 1997/09/26 11:11:11 paul Exp $
 **
 ** Definition of QApplication class
 **
@@ -7,13 +7,16 @@
 **
 ** Copyright (C) 1992-1997 Troll Tech AS.  All rights reserved.
 **
-** This file is part of the non-commercial distribution of Qt 1.2.
+** This file is part of the free edition of Qt 1.33.
 **
 ** See the file LICENSE included in the distribution for the usage
 ** and distribution terms, or http://www.troll.no/qt/license.html.
 **
-** (This file differs from the one in the commercial distribution of
-** Qt only by this comment.)
+** IMPORTANT NOTE: You may NOT copy this file or any part of it into
+** your own programs or libraries.
+**
+** (This file differs from the one in the commercial edition of Qt
+** only by this comment.)
 **
 *****************************************************************************/
 
@@ -28,7 +31,6 @@
 #error "TrueColor already #defined by X11/X.h. Include X11/X.h after qapp.h"
 #endif
 
-
 extern QApplication *qApp;			// global application object
 
 
@@ -37,6 +39,9 @@ class QApplication : public QObject		// application class
     Q_OBJECT
 public:
     QApplication( int &argc, char **argv );
+#if defined(_WS_X11_)
+    QApplication( Display* dpy );
+#endif
     virtual ~QApplication();
 
     int		    argc()	const;
@@ -47,12 +52,12 @@ public:
 
 #if 1	/* OBSOLETE */
     enum ColorMode { NormalColors, CustomColors };
-    static QApplication::ColorMode colorMode();
+    static ColorMode colorMode();
     static void      setColorMode( QApplication::ColorMode );
 #endif
 
-    enum ColorSpec { NormalColor=0, CustomColor=1, PrivateColor=3,
-		     TrueColor=4 };
+    enum ColorSpec { NormalColor=0, CustomColor=1, PrivateColor=4,
+		     ManyColor=4, TrueColor=4 };
     static int	     colorSpec();
     static void      setColorSpec( int );
 
@@ -60,6 +65,9 @@ public:
     static void	     setOverrideCursor( const QCursor &, bool replace=FALSE );
     static void	     restoreOverrideCursor();
 
+    static bool	     hasGlobalMouseTracking();
+    static void	     setGlobalMouseTracking( bool enable );
+
     static QPalette *palette();
     static void	     setPalette( const QPalette &,bool updateAllWidgets=FALSE);
 
@@ -70,9 +78,11 @@ public:
     QWidget	    *mainWidget()  const;
     void	     setMainWidget( QWidget * );
 
+    static QWidgetList *allWidgets();
     static QWidgetList *topLevelWidgets();
-
     static QWidget  *desktop();
+    static QWidget  *activePopupWidget();
+    static QWidget  *activeModalWidget();
     static QClipboard *clipboard();
     QWidget	    *focusWidget() const;
 
@@ -81,6 +91,8 @@ public:
 
     int		     exec();
     void	     processEvents();
+    void	     processEvents( int maxtime );
+    void	     processOneEvent();
     int		     enter_loop();
     void	     exit_loop();
     static void	     exit( int retcode=0 );
@@ -99,6 +111,13 @@ public:
 
     static void	     beep();
 
+    static void      setWinStyleHighlightColor( const QColor & );
+    static const QColor&   winStyleHighlightColor();
+
+    static void	    setDoubleClickInterval( int );
+    static int      doubleClickInterval();
+
+
 #if defined(_WS_MAC_)
     virtual bool     macEventFilter( MSG * );
 #elif defined(_WS_WIN_)
@@ -107,6 +126,7 @@ public:
     virtual bool     pmEventFilter( QMSG * );
 #elif defined(_WS_X11_)
     virtual bool     x11EventFilter( XEvent * );
+    int              x11ProcessEvent( XEvent* );
 #endif
 
 #if defined(_WS_WIN_)
@@ -121,6 +141,7 @@ public slots:
 
 private:
     bool	     processNextEvent( bool );
+    void	     initialize( int, char ** );
 
     int		     app_argc;
     char	   **app_argv;
@@ -128,9 +149,10 @@ private:
     int		     quit_code;
     static GUIStyle  app_style;
     static int	     app_cspec;
-    static QPalette  *app_pal;
+    static QPalette *app_pal;
     static QFont    *app_font;
     static QCursor  *app_cursor;
+    static int	     app_tracking;
     static bool	     is_app_running;
     static bool	     is_app_closing;
     static int	     loop_level;
@@ -166,6 +188,11 @@ inline QCursor *QApplication::overrideCursor()
     return app_cursor;
 }
 
+inline bool QApplication::hasGlobalMouseTracking()
+{
+    return app_tracking > 0;
+}
+
 inline QFont *QApplication::font()
 {
     return app_font;
