@@ -1,5 +1,5 @@
 /****************************************************************************
-** $Id: qclb_x11.cpp,v 2.1 1997/01/15 11:22:07 hanord Exp $
+** $Id: qclb_x11.cpp,v 2.3 1997/08/19 09:32:31 hanord Exp $
 **
 ** Implementation of QClipboard class for X11
 **
@@ -7,13 +7,16 @@
 **
 ** Copyright (C) 1992-1997 Troll Tech AS.  All rights reserved.
 **
-** This file is part of the non-commercial distribution of Qt 1.2.
+** This file is part of the free edition of Qt 1.33.
 **
 ** See the file LICENSE included in the distribution for the usage
 ** and distribution terms, or http://www.troll.no/qt/license.html.
 **
-** (This file differs from the one in the commercial distribution of
-** Qt only by this comment.)
+** IMPORTANT NOTE: You may NOT copy this file or any part of it into
+** your own programs or libraries.
+**
+** (This file differs from the one in the commercial edition of Qt
+** only by this comment.)
 **
 *****************************************************************************/
 
@@ -27,7 +30,7 @@
 #include <X11/Xos.h>
 #include <X11/Xatom.h>
 
-RCSTAG("$Id: qclb_x11.cpp,v 2.1 1997/01/15 11:22:07 hanord Exp $");
+RCSTAG("$Id: qclb_x11.cpp,v 2.3 1997/08/19 09:32:31 hanord Exp $");
 
 
 /*****************************************************************************
@@ -318,6 +321,7 @@ void QClipboard::setData( const char *format, void *data )
 	ASSERT( XGetSelectionOwner(dpy,XA_PRIMARY) == win );
 #endif
 	d->setData( format, data );
+	emit dataChanged();
 	return;
     }
 
