@@ -1,5 +1,5 @@
 /****************************************************************************
-** $Id: qcol_x11.cpp,v 2.9 1997/02/27 15:39:53 hanord Exp $
+** $Id: qcol_x11.cpp,v 2.14.2.2 1997/11/19 12:20:32 warwick Exp $
 **
 ** Implementation of QColor class for X11
 **
@@ -7,13 +7,16 @@
 **
 ** Copyright (C) 1992-1997 Troll Tech AS.  All rights reserved.
 **
-** This file is part of the non-commercial distribution of Qt 1.2.
+** This file is part of the free edition of Qt 1.33.
 **
 ** See the file LICENSE included in the distribution for the usage
 ** and distribution terms, or http://www.troll.no/qt/license.html.
 **
-** (This file differs from the one in the commercial distribution of
-** Qt only by this comment.)
+** IMPORTANT NOTE: You may NOT copy this file or any part of it into
+** your own programs or libraries.
+**
+** (This file differs from the one in the commercial edition of Qt
+** only by this comment.)
 **
 *****************************************************************************/
 
@@ -26,7 +29,7 @@
 #include <X11/Xutil.h>
 #include <X11/Xos.h>
 
-RCSTAG("$Id: qcol_x11.cpp,v 2.9 1997/02/27 15:39:53 hanord Exp $");
+RCSTAG("$Id: qcol_x11.cpp,v 2.14.2.2 1997/11/19 12:20:32 warwick Exp $");
 
 
 /*****************************************************************************
@@ -52,41 +55,80 @@ static QColorDict *colorDict  = 0;		// dict of allocated colors
 static bool	   colorAvail = TRUE;		// X colors available
 
 static bool	color_init = FALSE;		// module was initialized
+static bool	colors_frozen = FALSE;		// allocating disabled
 static int	current_alloc_context = 0;	// current color alloc context
 static Visual  *g_vis	= 0;			// visual
 static XColor  *g_carr	= 0;			// color array
+static bool	g_carr_fetch = TRUE;		// perform XQueryColors?
+static int	g_cells = 0;			// number of entries in g_carr
+static bool    *g_our_alloc = 0;		// our allocated colors
 static bool	g_truecolor;
 static uint	red_mask , green_mask , blue_mask;
 static int	red_shift, green_shift, blue_shift;
 
+extern int	qt_ncols_option;		// defined in qapp_x11.cpp
+extern int	qt_visual_option;
+extern bool	qt_cmap_option;
 
 /*
   This function is called from the event loop. It resets the colorAvail
   flag so that the application can retry to allocate read-only colors
   that other applications may have deallocated lately.
+
+  The g_our_alloc and g_carr are global arrays that optimize color
+  approximation when there are no more colors left to allocate.
 */
 
 void qt_reset_color_avail()
 {
     colorAvail = TRUE;
-    if ( g_carr ) {				// color array was allocated
-	delete [] g_carr;
-	g_carr = 0;				// reset
-    }
+    g_carr_fetch = TRUE;		// do XQueryColors if !colorAvail
 }
 
 
 /*
-  Returns a truecolor visual (if there is one). The SGI X server usually
-  has an 8 bit default visual, but the application can also ask for a
-  truecolor visual. This is what we do if QApplication::colorSpec() includes
-  QApplication::TrueColor.
+  Finds the nearest color.  If colors_frozen is TRUE, we only search among
+  the colors we have already allocated and not colors that have been
+  allocated by other applications.
+*/
+
+static int find_nearest_color( int r, int g, int b )
+{
+    int mincol = -1;
+    int mindist = 200000;
+    int rx, gx, bx, dist;
+    XColor *xc = &g_carr[0];
+    for ( int i=0; i<g_cells; i++ ) {
+	if ( colors_frozen && !g_our_alloc[i] ) {
+	    xc++;				// skip color we didn't alloc
+	    continue;
+	}
+	rx = r - (xc->red >> 8);
+	gx = g - (xc->green >> 8);
+	bx = b - (xc->blue>> 8);
+	dist = rx*rx + gx*gx + bx*bx;		// calculate distance
+	if ( dist < mindist ) {			// minimal?
+	    mindist = dist;
+	    mincol = i;
+	}
+	xc++;
+    }
+    return mincol;
+}
+
+
+/*
+  Returns a truecolor visual (if there is one). 8-bit TrueColor visuals
+  are ignored, unless the user has explicitly requested -visual TrueColor.
+  The SGI X server usually has an 8 bit default visual, but the application
+  can also ask for a truecolor visual. This is what we do if
+  QApplication::colorSpec() is QApplication::ManyColor.
 */
 
 static Visual *find_truecolor_visual( Display *dpy, int *depth, int *ncols )
 {
     XVisualInfo *vi, rvi;
-    int best=-1, n, i;
+    int best=0, n, i;
     int scr = DefaultScreen(dpy);
     rvi.c_class = TrueColor;
     rvi.screen  = scr;
@@ -94,12 +136,14 @@ static Visual *find_truecolor_visual( Display *dpy, int *depth, int *ncols )
 			 &rvi, &n );
     if ( vi ) {
 	for ( i=0; i<n; i++ ) {
-	    if ( (vi[i].depth == 24) || (vi[i].depth > 24 && best<0) )
+	    if ( vi[i].depth > vi[best].depth )
 		best = i;
 	}
     }
     Visual *v = DefaultVisual(dpy,scr);
-    if ( best < 0 || (vi[best].visualid == XVisualIDFromVisual(v)) ) {
+    if ( !vi || (vi[best].visualid == XVisualIDFromVisual(v))
+       || (vi[best].depth <= 8 && qt_visual_option != TrueColor) )
+    {
 	*depth = DefaultDepth(dpy,scr);
 	*ncols = DisplayCells(dpy,scr);	
     } else {
@@ -173,9 +217,13 @@ void QColor::initialize()
     int	     depth, ncols;
     Colormap cmap;    
     const int tc = TrueColor;
-#undef  TrueColor				// defined in X.h
 
-    if ( spec & (int)QApplication::TrueColor ) {
+#if QT_VERSION == 200
+#error "Remove old colorspecs"
+    if ( spec == (int)QApplication::ManyColor ) {
+#else /* } bracket match */
+    if ( spec & ((int)QApplication::ManyColor | 2) ) {
+#endif
 	g_vis = find_truecolor_visual( dpy, &depth, &ncols );
     } else {
 	g_vis = DefaultVisual(dpy,scr);
@@ -190,14 +238,31 @@ void QColor::initialize()
     if ( g_truecolor )
 	defCmap = defVis;
     else
-	defCmap = (spec & QApplication::PrivateColor)
-	    != QApplication::PrivateColor;
+	defCmap = !qt_cmap_option;
 
     if ( defCmap )
 	cmap = DefaultColormap(dpy,scr);
     else
 	cmap = XCreateColormap(dpy, RootWindow(dpy,scr), g_vis, AllocNone );
 
+    if ( !g_truecolor ) {
+	// Create the g_our_alloc array, which remembers which color pixels
+	// we allocated.
+	g_cells = QMIN(ncols,256);
+	g_carr  = new XColor[g_cells];
+	CHECK_PTR( g_carr );
+	memset( g_carr, 0, g_cells*sizeof(XColor) );
+	g_carr_fetch = TRUE;		// run XQueryColors on demand
+	g_our_alloc = new bool[g_cells];	
+	CHECK_PTR( g_our_alloc );
+	memset( g_our_alloc, FALSE, g_cells*sizeof(bool) );
+	XColor *xc = &g_carr[0];
+	for ( int i=0; i<g_cells; i++ ) {
+	    xc->pixel = i;		// g_carr[i] = color i
+	    xc++;
+	}
+    }
+
     QPaintDevice::x_display   = dpy;
     QPaintDevice::x_screen    = scr;
     QPaintDevice::x_depth     = depth;
@@ -235,6 +300,82 @@ void QColor::initialize()
 	((QColor*)(&white))->alloc();
     }
 
+#if QT_VERSION == 200
+#error "Remove old colorspecs"
+    if ( spec == (int)QApplication::ManyColor && !g_truecolor ) {
+#else /* } bracket match */
+    if ( (spec & ((int)QApplication::ManyColor | 2)) && !g_truecolor ) {
+#endif
+	// Allocate a color cube, starting from the most common and extreme
+	// colours in the cube, then each most-distant color.
+	static QRgb cmap[216] = {
+	    0x000000, 0xffffff, 0x0000ff, 0x00ff00,
+	    0xff0000, 0x00ffff, 0xffff00, 0xff00ff,
+	    0x00ff99, 0xff0099, 0xcccc00, 0x6699ff,
+	    0x666666, 0x33ff00, 0xff3300, 0xff66ff,
+	    0x9900ff, 0x99ff99, 0xff9966, 0x009933,
+	    0x990033, 0x33ffff, 0x0099cc, 0x003399,
+	    0x660099, 0x669900, 0x99ffff, 0xff00ff,
+	    0xffff99, 0xcccccc, 0x9966cc, 0x66cc66,
+	    0xcc3366, 0xffff33, 0x99ff33, 0x333333,
+	    0x00ccff, 0xcc99ff, 0x3366ff, 0xcc33ff,
+	    0x6633ff, 0x66ffcc, 0x33cccc, 0xff99cc,
+	    0xff33cc, 0x3333cc, 0xcc00cc, 0xcc9999,
+	    0x669999, 0xff6699, 0x336699, 0x993399,
+	    0xccff66, 0x33ff66, 0x00cc66, 0x999966,
+	    0x339966, 0x006666, 0x330066, 0x00ff33,
+	    0x33cc33, 0xcc9933, 0xff6633, 0x996633,
+	    0xff0033, 0x00cc00, 0xff9900, 0xcc6600,
+	    0x336600, 0x993300, 0xcc0000, 0x660000,
+	    0xccffff, 0x66ffff, 0x00ffff, 0xffccff,
+	    0xccccff, 0x99ccff, 0x66ccff, 0x33ccff,
+	    0xff99ff, 0x9999ff, 0x3399ff, 0x0099ff,
+	    0xcc66ff, 0x9966ff, 0x6666ff, 0x0066ff,
+	    0xff33ff, 0x9933ff, 0x3333ff, 0x0033ff,
+	    0xcc00ff, 0x6600ff, 0x3300ff, 0xffffcc,
+	    0xccffcc, 0x99ffcc, 0x33ffcc, 0x00ffcc,
+	    0xffcccc, 0x99cccc, 0x66cccc, 0x00cccc,
+	    0xcc99cc, 0x9999cc, 0x6699cc, 0x3399cc,
+	    0xff66cc, 0xcc66cc, 0x6666cc, 0x3366cc,
+	    0x0066cc, 0xcc33cc, 0x9933cc, 0x6633cc,
+	    0x0033cc, 0xff00cc, 0x9900cc, 0x6600cc,
+	    0x3300cc, 0x0000cc, 0xccff99, 0x66ff99,
+	    0x33ff99, 0xffcc99, 0xcccc99, 0x99cc99,
+	    0x66cc99, 0x33cc99, 0x00cc99, 0xff9999,
+	    0x999999, 0x339999, 0x009999, 0xcc6699,
+	    0x996699, 0x666699, 0x006699, 0xff3399,
+	    0xcc3399, 0x663399, 0x333399, 0xcc0099,
+	    0x990099, 0x330099, 0x000099, 0xffff66,
+	    0x99ff66, 0x66ff66, 0x00ff66, 0xffcc66,
+	    0xcccc66, 0x99cc66, 0x33cc66, 0xcc9966,
+	    0x669966, 0x009966, 0xff6666, 0xcc6666,
+	    0x996666, 0x336666, 0xff3366, 0x993366,
+	    0x663366, 0x333366, 0x003366, 0xff0066,
+	    0xcc0066, 0x990066, 0x660066, 0x000066,
+	    0xccff33, 0x66ff33, 0x33ff33, 0xffcc33,
+	    0xcccc33, 0x99cc33, 0x66cc33, 0x00cc33,
+	    0xff9933, 0x999933, 0x669933, 0x339933,
+	    0xcc6633, 0x666633, 0x336633, 0x006633,
+	    0xff3333, 0xcc3333, 0x993333, 0x663333,
+	    0x003333, 0xcc0033, 0x660033, 0x330033,
+	    0x000033, 0xffff00, 0xccff00, 0x99ff00,
+	    0x66ff00, 0x00ff00, 0xffcc00, 0x99cc00,
+	    0x66cc00, 0x33cc00, 0xcc9900, 0x999900,
+	    0x339900, 0x009900, 0xff6600, 0x996600,
+	    0x666600, 0x006600, 0xcc3300, 0x663300,
+	};
+
+	if (qt_ncols_option>216) qt_ncols_option=216;
+
+	for ( int i = 0; i < qt_ncols_option; i++ ) {
+	    QColor c(cmap[i]);
+	    c.alloc();
+        }
+
+	// No more custom allocations: user has set limit
+	colors_frozen = TRUE;
+    }
+
 #if 0 /* 0 == allocate colors on demand */
     setLazyAlloc( FALSE );			// allocate global colors
     ((QColor*)(&darkGray))->	alloc();
@@ -267,6 +408,8 @@ void QColor::cleanup()
     if ( !color_init )
 	return;
     color_init = FALSE;
+    delete [] g_carr;
+    delete [] g_our_alloc;
     if ( !QPaintDevice::x_defcmap )
 	XFreeColormap( QPaintDevice::x__Display(), QPaintDevice::x_colormap );
     if ( colorDict ) {
@@ -285,9 +428,9 @@ void QColor::cleanup()
 /*!
   Constructs a color with a RGB value and a custom pixel value.
 
-  If the \e pix = 0xffffffff, then the color uses the RGB value in a
-  standard way.	 If \e pix is something else, then the pixel value will
-  be set directly to \e pix (skips the standard allocation procedure).
+  If the \e pixel = 0xffffffff, then the color uses the RGB value in a
+  standard way.	 If \e pixel is something else, then the pixel value will
+  be set directly to \e pixel (skips the standard allocation procedure).
 */
 
 QColor::QColor( QRgb rgb, uint pixel )
@@ -342,8 +485,10 @@ uint QColor::alloc()
     if ( c ) {					// found color in dictionary
 	rgbVal &= RGB_MASK;			// color ok
 	pix = c->pix;				// use same pixel value
-	if ( c->context != current_alloc_context )
-	    c->context = 0;			// convert to standard context
+	if ( c->context != current_alloc_context ) {
+	    c->context = 0;			// convert to default context
+	    g_our_alloc[pix] = TRUE;		// reuse without XAllocColor
+	}
 	return pix;
     }
     XColor col;
@@ -353,67 +498,90 @@ uint QColor::alloc()
     col.red   = r << 8;
     col.green = g << 8;
     col.blue  = b << 8;
-    if ( colorAvail && XAllocColor(dpy, QPaintDevice::x11Colormap(), &col) ) {
-	pix = (uint)col.pixel;			// allocated X11 color
-	rgbVal &= RGB_MASK;
-    } else {					// get closest color
-	int mincol = -1;
-	int mindist = 200000;
-	int rx, gx, bx, dist;
-	int i, maxi = QMIN(QPaintDevice::x11Cells(),256);
-	register XColor *xc;
-	colorAvail = FALSE;			// no more avail colors
-	if ( !g_carr ) {			// get colors in colormap
-	    g_carr = new XColor[maxi];
-	    CHECK_PTR( g_carr );
-	    xc = &g_carr[0];
-	    for ( i=0; i<maxi; i++ ) {
-		xc->pixel = i;			// carr[i] = color i
-		xc++;
-	    }
-	    XQueryColors( dpy, QPaintDevice::x11Colormap(), g_carr, maxi );
-	}
-	xc = &g_carr[0];
-	for ( i=0; i<maxi; i++ ) {		// find closest color
-	    rx = r - (xc->red >> 8);
-	    gx = g - (xc->green >> 8);
-	    bx = b - (xc->blue>> 8);
-	    dist = rx*rx + gx*gx + bx*bx;	// calculate distance
-	    if ( dist < mindist ) {		// minimal?
-		mindist = dist;
-		mincol = i;
-	    }
-	    xc++;
-	}
-	if ( mincol == -1 ) {			// there are no colors, yuck
+
+    bool try_again = FALSE;
+    int  try_count = 0;
+
+    if ( colors_frozen ) {
+	// Find the nearest color we have already allocated
+	pix = find_nearest_color( r, g, b );
+	if ( (int)pix == -1 ) {			// no nearest color?!
 	    rgbVal |= RGB_INVALID;
 	    pix = BlackPixel( dpy, scr );
 	    return pix;
+	} else {
+	    rgbVal &= RGB_MASK;
 	}
-	XAllocColor( dpy, QPaintDevice::x11Colormap(), &g_carr[mincol] );
-	pix = g_carr[mincol].pixel;		// allocated X11 color
-	rgbVal &= RGB_MASK;
+    }
+
+    else  do {
+	// This loop is run until we manage to either allocate or
+	// find an approximate color, it stops after 100 iterations.
+
+	if ( colorAvail &&
+	     XAllocColor(dpy,QPaintDevice::x11Colormap(),&col) ) {
+
+	    // We could allocate the color
+	    pix = (uint)col.pixel;
+	    rgbVal &= RGB_MASK;
+	    g_carr[pix] = col;			// update color array
+	    if ( current_alloc_context == 0 )
+		g_our_alloc[pix] = TRUE;	// reuse without XAllocColor
+
+	} else {
+
+	    // No available colors
+	    int i;
+	    colorAvail = FALSE;		// no more available colors
+	    if ( g_carr_fetch ) {		// refetch color array
+		g_carr_fetch = FALSE;
+		XQueryColors(dpy, QPaintDevice::x11Colormap(), g_carr,g_cells);
+	    }
+	    i = find_nearest_color( r, g, b );
+	    if ( i == -1 ) {			// no nearest color?!
+		rgbVal |= RGB_INVALID;
+		pix = BlackPixel( dpy, scr );
+		return pix;
+	    }
+	    if ( g_our_alloc[i] ) {		// we've already allocated it
+		pix = g_carr[i].pixel;
+	    } else {
+		if ( XAllocColor(dpy,QPaintDevice::x11Colormap(),&g_carr[i])){
+		    if ( current_alloc_context == 0 )
+			g_our_alloc[i] = TRUE;	// only in the default context
+		} else {
+		    try_count++;
+		    try_again    = TRUE;
+		    colorAvail = TRUE;
+		    g_carr_fetch = TRUE;
+		}
+	    }
+	    ASSERT(i==(int)g_carr[i].pixel);
+	    if ( !try_again ) {			// got it
+		pix = g_carr[i].pixel;		// allocated X11 color
+		rgbVal &= RGB_MASK;
+	    }
+	}
+
+    } while ( try_again && try_count < 100 );
+
+    if ( try_again ) {				// no hope of allocating color
+	rgbVal |= RGB_INVALID;
+	pix = BlackPixel( dpy, scr );
+	return pix;
     }
     if ( colorDict->count() < colorDict->size() * 8 ) {
 	c = new QColorData;			// insert into color dict
 	CHECK_PTR( c );
 	c->pix	   = pix;
-	c->context = current_alloc_context;
+	c->context = colors_frozen ? 0 : current_alloc_context;
 	colorDict->insert( (long)rgbVal, c );	// store color in dict
     }
     return pix;
 }
 
 
-/*!
-  Sets the RGB value to that of the named color.
-
-  This function searches the X color database for the color and sets the
-  RGB value.  The color will be set to invalid if such a color does not
-  exist.
-*/
-
-void QColor::setNamedColor( const char *name )
+void QColor::setSystemNamedColor( const char *name )
 {
     bool ok = FALSE;
     if ( color_init	 ) {			// initialized
@@ -589,7 +757,7 @@ int QColor::currentAllocContext()
 void QColor::destroyAllocContext( int context )
 {
     init_context_stack();
-    if ( !color_init || g_truecolor )
+    if ( !color_init || g_truecolor || colors_frozen )
 	return; 
     ulong *pixels = new ulong[colorDict->count()];
     QColorData   *d;
@@ -601,6 +769,7 @@ void QColor::destroyAllocContext( int context )
 	++it;
 	if ( d->context == context || context == -1 ) {
 	    pixels[i++] = d->pix;		// delete this color
+	    g_our_alloc[d->pix] = FALSE;	// don't reuse it
 	    colorDict->remove( (long)rgbVal );	// remove from dict
 	}
     }
