@@ -1,5 +1,5 @@
 /****************************************************************************
-** $Id: qcolor.cpp,v 2.3 1997/03/10 12:03:20 hanord Exp $
+** $Id: qcolor.cpp,v 2.11 1997/09/19 13:12:32 hanord Exp $
 **
 ** Implementation of QColor class
 **
@@ -7,20 +7,24 @@
 **
 ** Copyright (C) 1992-1997 Troll Tech AS.  All rights reserved.
 **
-** This file is part of the non-commercial distribution of Qt 1.2.
+** This file is part of the free edition of Qt 1.33.
 **
 ** See the file LICENSE included in the distribution for the usage
 ** and distribution terms, or http://www.troll.no/qt/license.html.
 **
-** (This file differs from the one in the commercial distribution of
-** Qt only by this comment.)
+** IMPORTANT NOTE: You may NOT copy this file or any part of it into
+** your own programs or libraries.
+**
+** (This file differs from the one in the commercial edition of Qt
+** only by this comment.)
 **
 *****************************************************************************/
 
 #include "qcolor.h"
 #include "qdstream.h"
+#include <stdlib.h>
 
-RCSTAG("$Id: qcolor.cpp,v 2.3 1997/03/10 12:03:20 hanord Exp $");
+RCSTAG("$Id: qcolor.cpp,v 2.11 1997/09/19 13:12:32 hanord Exp $");
 
 
 /*!
@@ -164,7 +168,7 @@ void QColor::initglobals()
 
   Constructs a color with the RGB value (r,g,b).
 
-  \e r, \e g and \e b must be in the rangle 0..255.
+  \e r, \e g and \e b must be in the range 0..255.
 
   \sa setRgb()
 */
@@ -214,6 +218,36 @@ QColor::QColor( const QColor &c )
     pix	   = c.pix;
 }
 
+
+/*!
+  Sets the RGB value to that of the named color.
+
+  The "#RRGGBB" color format is supported on all platforms, where
+  RR, GG, and BB and hex digits.
+
+  A color name may also be a color in the X color database (rgb.txt) ,
+  e.g.  "steelblue" or "gainsboro".  The X color names also work under
+  Qt for Windows.
+*/
+
+void QColor::setNamedColor( const char *name )
+{
+    if ( strlen(name) == 0 ) {
+	setRgb( 0, 0, 0 );
+    } else if ( name[0]=='#' ) {
+	QString hex = name+1;
+	int bpc = (strlen(name)-1)/3;
+	int i=0;
+	int r = strtol(hex.mid(i,bpc),0,16)*256/(1<<(4*bpc)); i+=bpc;
+	int g = strtol(hex.mid(i,bpc),0,16)*256/(1<<(4*bpc)); i+=bpc;
+	int b = strtol(hex.mid(i,bpc),0,16)*256/(1<<(4*bpc));
+	setRgb( r, g, b );
+    } else {
+	setSystemNamedColor( name );
+    }
+}
+
+
 /*!
   Assigns a copy of the color \c and returns a reference to this color.
 */
@@ -434,11 +468,12 @@ QColor QColor::light( int factor ) const
 	return *this;
     else if ( factor < 100 )			// makes color darker
 	return dark( 10000/factor );
+
     int h, s, v;
     hsv( &h, &s, &v );
     v = (factor*v)/100;
     if ( v > 255 ) {				// overflow
-	s -= (factor*106)/100;			// adjust saturation
+	s -= v-255;				// adjust saturation
 	if ( s < 0 )
 	    s = 0;
 	v = 255;
