@@ -1,5 +1,5 @@
 /****************************************************************************
-** $Id: qdrawutl.cpp,v 2.4 1997/03/22 16:18:36 hanord Exp $
+** $Id: qdrawutl.cpp,v 2.15 1997/09/25 15:57:13 hanord Exp $
 **
 ** Implementation of draw utilities
 **
@@ -7,13 +7,16 @@
 **
 ** Copyright (C) 1992-1997 Troll Tech AS.  All rights reserved.
 **
-** This file is part of the non-commercial distribution of Qt 1.2.
+** This file is part of the free edition of Qt 1.33.
 **
 ** See the file LICENSE included in the distribution for the usage
 ** and distribution terms, or http://www.troll.no/qt/license.html.
 **
-** (This file differs from the one in the commercial distribution of
-** Qt only by this comment.)
+** IMPORTANT NOTE: You may NOT copy this file or any part of it into
+** your own programs or libraries.
+**
+** (This file differs from the one in the commercial edition of Qt
+** only by this comment.)
 **
 *****************************************************************************/
 
@@ -21,7 +24,7 @@
 #include "qbitmap.h"
 #include "qpmcache.h"
 
-RCSTAG("$Id: qdrawutl.cpp,v 2.4 1997/03/22 16:18:36 hanord Exp $");
+RCSTAG("$Id: qdrawutl.cpp,v 2.15 1997/09/25 15:57:13 hanord Exp $");
 
 
 /*!
@@ -193,13 +196,12 @@ void qDrawShadeRect( QPainter *p, int x, int y, int w, int h,
 	p->drawLineSegments( a );		// draw bottom/right lines
     }
     else {					// more complicated
-	int t = lineWidth*2+midLineWidth;
 	int m = lineWidth+midLineWidth;
 	int i, j=0, k=m;
 	for ( i=0; i<lineWidth; i++ ) {		// draw top shadow
 	    p->drawLine( x1+j, y2-j, x1+j, y1+j );
 	    p->drawLine( x1+j, y1+j, x2-j, y1+j );
-	    p->drawLine( x1+t, y2-k, x2-k, y2-k );
+	    p->drawLine( x1+k, y2-k, x2-k, y2-k );
 	    p->drawLine( x2-k, y2-k, x2-k, y1+k );
 	    j++;
 	    k++;
@@ -219,7 +221,7 @@ void qDrawShadeRect( QPainter *p, int x, int y, int w, int h,
 	for ( i=0; i<lineWidth; i++ ) {		// draw bottom shadow
 	    p->drawLine( x1+1+j,y2-j, x2-j, y2-j );
 	    p->drawLine( x2-j,	y2-j, x2-j, y1+j+1 );
-	    p->drawLine( x1+k,	y2-m, x1+k, y1+k );
+	    p->drawLine( x1+k,	y2-k, x1+k, y1+k );
 	    p->drawLine( x1+k,	y1+k, x2-k, y1+k );
 	    j++;
 	    k++;
@@ -329,6 +331,15 @@ void qDrawShadePanel( QPainter *p, int x, int y, int w, int h,
   \internal
   This function draws a rectangle with two pixel line width.
   It is called from qDrawWinButton() and qDrawWinPanel().
+
+  c1..c4 and fill are used:
+
+    1 1 1 1 1 2
+    1 3 3 3 4 2
+    1 3 F F 4 2
+    1 3 F F 4 2
+    1 4 4 4 4 2
+    2 2 2 2 2 2
 */
 
 static void qDrawWinShades( QPainter *p,
@@ -404,7 +415,7 @@ void qDrawWinButton( QPainter *p, int x, int y, int w, int h,
 		       black, g.light(), g.dark(), g.background(), fill );
     else
 	qDrawWinShades( p, x, y, w, h,
-		       g.light(), black, g.background(), g.dark(), fill );
+		       g.light(), black, g.midlight(), g.dark(), fill );
 }
 
 /*!
@@ -412,10 +423,7 @@ void qDrawWinButton( QPainter *p, int x, int y, int w, int h,
 
   Draws a Windows-style panel given by \e (x,y,w,h) using the painter \e p.
 
-  The color group argument \e g specifies the shading colors
-  (\link QColorGroup::light() light\endlink,
-  \link QColorGroup::dark() dark\endlink and
-  \link QColorGroup::mid() middle\endlink colors).
+  The color group argument \e g specifies the shading colors.
 
   The panel appears sunken if \e sunken is TRUE, or raised if \e sunken is
   FALSE.
@@ -438,10 +446,10 @@ void qDrawWinPanel( QPainter *p, int x, int y, int w, int h,
 {
     if ( sunken )
 	qDrawWinShades( p, x, y, w, h,
-		       g.dark(), g.light(), black, g.background(), fill );
+		       g.dark(), g.light(), black, g.midlight(), fill );
     else
 	qDrawWinShades( p, x, y, w, h,
-		       g.background(), black, g.light(), g.dark(), fill );
+		       g.midlight(), black, g.light(), g.dark(), fill );
 }
 
 
@@ -488,6 +496,39 @@ void qDrawPlainRect( QPainter *p, int x, int y, int w, int h, const QColor &c,
 }
 
 
+QRect qItemRect( QPainter *p, GUIStyle gs,
+		int x, int y, int w, int h,
+		int flags, 
+		bool enabled,
+		const QPixmap *pixmap,
+		const char *text, int len )
+{
+    QRect result;
+
+    if ( pixmap ) {
+	if ( (flags & AlignVCenter) == AlignVCenter )
+	    y += h/2 - pixmap->height()/2;
+	else if ( (flags & AlignBottom) == AlignBottom)
+	    y += h - pixmap->height();
+	if ( (flags & AlignRight) == AlignRight )
+	    x += w - pixmap->width();
+	else if ( (flags & AlignHCenter) == AlignHCenter )
+	    x += w/2 - pixmap->width()/2;
+	result = QRect(x, y, pixmap->width(), pixmap->height());
+    } else if ( text && p ) {
+	result = p->boundingRect( x, y, w, h, flags, text, len );
+	if ( gs == WindowsStyle && !enabled ) {
+	    result.setWidth(result.width()+1);
+	    result.setHeight(result.height()+1);
+	}
+    } else {
+	result = QRect(x, y, w, h);
+    }
+
+    return result;
+}
+
+
 void qDrawItem( QPainter *p, GUIStyle gs,
 		int x, int y, int w, int h,
 		int flags, 
@@ -514,31 +555,27 @@ void qDrawItem( QPainter *p, GUIStyle gs,
 	else if ( (flags & AlignHCenter) == AlignHCenter )
 	    x += w/2 - pm.width()/2;
 	if ( !enabled ) {
-	    if ( pm.mask() ) {
-		if ( !pm.selfMask() ) {
+	    if ( pm.mask() ) {			// pixmap with a mask
+		if ( !pm.selfMask() ) {		// mask is not pixmap itself
 		    QPixmap pmm( *pm.mask() );
 		    pmm.setMask( *((QBitmap *)&pmm) );
 		    pm = pmm;
 		}
-	    } else if ( pm.depth() == 1 ) {
+	    } else if ( pm.depth() == 1 ) {	// monochrome pixmap, no mask
 		pm.setMask( *((QBitmap *)&pm) );
-	    } else if ( pm.depth() > 1 ) {
-		if ( pm.mask() ) {
-		    pm = *pm.mask();
-		} else {
-		    QString k;
-		    k.sprintf( "$qt-drawitem-%x", pm.serialNumber() );
-		    QPixmap *mask = QPixmapCache::find(k);
-		    if ( !mask ) {
-			mask = new QPixmap( pm.createHeuristicMask() );
-			mask->setMask( *((QBitmap*)mask) );
-			QPixmapCache::insert( k, mask );
-		    }
-		    pm = *mask;
+	    } else {				// color pixmap, no mask
+		QString k;
+		k.sprintf( "$qt-drawitem-%x", pm.serialNumber() );
+		QPixmap *mask = QPixmapCache::find(k);
+		if ( !mask ) {
+		    mask = new QPixmap( pm.createHeuristicMask() );
+		    mask->setMask( *((QBitmap*)mask) );
+		    QPixmapCache::insert( k, mask );
 		}
+		pm = *mask;
 	    }
 	    if ( gs == WindowsStyle ) {
-		p->setPen( white );
+		p->setPen( g.light() );
 		p->drawPixmap( x+1, y+1, pm );
 		p->setPen( g.text() );
 	    }
@@ -548,7 +585,7 @@ void qDrawItem( QPainter *p, GUIStyle gs,
 	    p->setClipping( FALSE );
     } else if ( text ) {
 	if ( gs == WindowsStyle && !enabled ) {
-	    p->setPen( white );
+	    p->setPen( g.light() );
 	    p->drawText( x+1, y+1, w, h, flags, text, len );
 	    p->setPen( g.text() );
 	}
@@ -630,3 +667,200 @@ void qDrawPlainRect( QPainter *p, const QRect &r, const QColor &c,
     qDrawPlainRect( p, r.x(), r.y(), r.width(), r.height(), c,
 		    lineWidth, fill );
 }
+
+
+static void qDrawWinArrow( QPainter *p, ArrowType type, bool down,
+			   int x, int y, int w, int h,
+			   const QColorGroup &g, bool enabled )
+{
+    QPointArray a;				// arrow polygon
+    switch ( type ) {
+    case UpArrow:
+	a.setPoints( 7, -3,1, 3,1, -2,0, 2,0, -1,-1, 1,-1, 0,-2 );
+	break;
+    case DownArrow:
+	a.setPoints( 7, -3,-1, 3,-1, -2,0, 2,0, -1,1, 1,1, 0,2 );
+	break;
+    case LeftArrow:
+	a.setPoints( 7, 1,-3, 1,3, 0,-2, 0,2, -1,-1, -1,1, -2,0 );
+	break;
+    case RightArrow:
+	a.setPoints( 7, -1,-3, -1,3, 0,-2, 0,2, 1,-1, 1,1, 2,0 );
+	break;
+    }
+    if ( a.isNull() )
+	return;
+
+    if ( down ) {
+	x++;
+	y++;
+    }
+
+    QPen savePen = p->pen();			// save current pen
+    p->fillRect( x, y, w, h, g.background() );
+    if ( enabled ) {
+	a.translate( x+w/2, y+h/2 );
+	p->setPen( g.foreground() );
+	p->drawLineSegments( a, 0, 3 );		// draw arrow
+	p->drawPoint( a[6] );
+    } else {
+	a.translate( x+w/2+1, y+h/2+1 );
+	p->setPen( g.light() );
+	p->drawLineSegments( a, 0, 3 );		// draw arrow
+	p->drawPoint( a[6] );
+	a.translate( -1, -1 );
+	p->setPen( g.mid() );
+	p->drawLineSegments( a, 0, 3 );		// draw arrow
+	p->drawPoint( a[6] );
+    }
+    p->setPen( savePen );			// restore pen
+}
+
+
+#if defined(_CC_MSVC_)
+#pragma warning(disable: 4244)
+#endif
+
+
+// motif arrows look the same whether they are used or not
+// is this correct?
+static void qDrawMotifArrow( QPainter *p, ArrowType type, bool down,
+			     int x, int y, int w, int h,
+			     const QColorGroup &g, bool ) 
+{
+    QPointArray bFill;				// fill polygon
+    QPointArray bTop;				// top shadow.
+    QPointArray bBot;				// bottom shadow.
+    QPointArray bLeft;				// left shadow.
+    QWMatrix	matrix;				// xform matrix
+    bool vertical = type == UpArrow || type == DownArrow;
+    bool horizontal = !vertical;
+    int	 dim = w < h ? w : h;
+    int	 colspec = 0x0000;			// color specification array
+
+    if ( dim < 2 )				// too small arrow
+	return;
+
+    if ( dim > 3 ) {
+	if ( dim > 6 )
+	    bFill.resize( dim & 1 ? 3 : 4 );
+	bTop.resize( (dim/2)*2 );
+	bBot.resize( dim & 1 ? dim + 1 : dim );
+	bLeft.resize( dim > 4 ? 4 : 2 );
+	bLeft.putPoints( 0, 2, 0,0, 0,dim-1 );
+	if ( dim > 4 )
+	    bLeft.putPoints( 2, 2, 1,2, 1,dim-3 );
+	bTop.putPoints( 0, 4, 1,0, 1,1, 2,1, 3,1 );
+	bBot.putPoints( 0, 4, 1,dim-1, 1,dim-2, 2,dim-2, 3,dim-2 );
+
+	for( int i=0; i<dim/2-2 ; i++ ) {
+	    bTop.putPoints( i*2+4, 2, 2+i*2,2+i, 5+i*2, 2+i );
+	    bBot.putPoints( i*2+4, 2, 2+i*2,dim-3-i, 5+i*2,dim-3-i );
+	}
+	if ( dim & 1 )				// odd number size: extra line
+	    bBot.putPoints( dim-1, 2, dim-3,dim/2, dim-1,dim/2 );
+	if ( dim > 6 ) {			// dim>6: must fill interior
+	    bFill.putPoints( 0, 2, 1,dim-3, 1,2 );
+	    if ( dim & 1 )			// if size is an odd number
+		bFill.setPoint( 2, dim - 3, dim / 2 );
+	    else
+		bFill.putPoints( 2, 2, dim-4,dim/2-1, dim-4,dim/2 );
+	}
+    }
+    else {
+	if ( dim == 3 ) {			// 3x3 arrow pattern
+	    bLeft.setPoints( 4, 0,0, 0,2, 1,1, 1,1 );
+	    bTop .setPoints( 2, 1,0, 1,0 );
+	    bBot .setPoints( 2, 1,2, 2,1 );
+	}
+	else {					// 2x2 arrow pattern
+	    bLeft.setPoints( 2, 0,0, 0,1 );
+	    bTop .setPoints( 2, 1,0, 1,0 );
+	    bBot .setPoints( 2, 1,1, 1,1 );
+	}
+    }
+
+    if ( type == UpArrow || type == LeftArrow ) {
+	matrix.translate( x, y );
+	if ( vertical ) {
+	    matrix.translate( 0, h - 1 );
+	    matrix.rotate( -90 );
+	} else {
+	    matrix.translate( w - 1, h - 1 );
+	    matrix.rotate( 180 );
+	}
+	if ( down )
+	    colspec = horizontal ? 0x2334 : 0x2343;
+	else
+	    colspec = horizontal ? 0x1443 : 0x1434;
+    }
+    else if ( type == DownArrow || type == RightArrow ) {
+	matrix.translate( x, y );
+	if ( vertical ) {
+	    matrix.translate( w-1, 0 );
+	    matrix.rotate( 90 );
+	}
+	if ( down )
+	    colspec = horizontal ? 0x2443 : 0x2434;
+	else
+	    colspec = horizontal ? 0x1334 : 0x1343;
+    }
+
+    QColor *cols[5];
+    cols[0] = 0;
+    cols[1] = (QColor *)&g.background();
+    cols[2] = (QColor *)&g.mid();
+    cols[3] = (QColor *)&g.light();
+    cols[4] = (QColor *)&g.dark();
+#define CMID	*cols[ (colspec>>12) & 0xf ]
+#define CLEFT	*cols[ (colspec>>8) & 0xf ]
+#define CTOP	*cols[ (colspec>>4) & 0xf ]
+#define CBOT	*cols[ colspec & 0xf ]
+
+    QPen     savePen   = p->pen();		// save current pen
+    QBrush   saveBrush = p->brush();		// save current brush
+    QWMatrix wxm = p->worldMatrix();
+    QPen     pen( NoPen );
+    QBrush   brush( CMID );
+
+    p->setPen( pen );
+    p->setBrush( brush );
+    p->setWorldMatrix( matrix, TRUE );		// set transformation matrix
+    p->drawPolygon( bFill );			// fill arrow
+    p->setBrush( NoBrush );			// don't fill
+
+    p->setPen( CLEFT );
+    p->drawLineSegments( bLeft );
+    p->setPen( CTOP );
+    p->drawLineSegments( bTop );
+    p->setPen( CBOT );
+    p->drawLineSegments( bBot );
+
+    p->setWorldMatrix( wxm );
+    p->setBrush( saveBrush );			// restore brush
+    p->setPen( savePen );			// restore pen
+
+#undef CMID
+#undef CLEFT
+#undef CTOP
+#undef CBOT
+}
+
+
+void qDrawArrow( QPainter *p, ArrowType type, GUIStyle style, bool down,
+		 int x, int y, int w, int h,
+		 const QColorGroup &g, bool enabled )
+{
+    switch ( style ) {
+	case WindowsStyle:
+	    qDrawWinArrow( p, type, down, x, y, w, h, g, enabled );
+	    break;
+	case MotifStyle:
+	    qDrawMotifArrow( p, type, down, x, y, w, h, g, enabled );
+	    break;
+	default:
+#if defined(CHECK_RANGE)
+	    warning( "qDrawArrow: Requested GUI style not supported" );
+#endif
+    }
+}
