@@ -1,5 +1,5 @@
 /****************************************************************************
-** $Id: qfnt_x11.cpp,v 2.14 1997/04/04 15:54:13 hanord Exp $
+** $Id: qfnt_x11.cpp,v 2.18.2.1 1997/11/18 14:16:05 paul Exp $
 **
 ** Implementation of QFont, QFontMetrics and QFontInfo classes for X11
 **
@@ -7,13 +7,16 @@
 **
 ** Copyright (C) 1992-1997 Troll Tech AS.  All rights reserved.
 **
-** This file is part of the non-commercial distribution of Qt 1.2.
+** This file is part of the free edition of Qt 1.33.
 **
 ** See the file LICENSE included in the distribution for the usage
 ** and distribution terms, or http://www.troll.no/qt/license.html.
 **
-** (This file differs from the one in the commercial distribution of
-** Qt only by this comment.)
+** IMPORTANT NOTE: You may NOT copy this file or any part of it into
+** your own programs or libraries.
+**
+** (This file differs from the one in the commercial edition of Qt
+** only by this comment.)
 **
 *****************************************************************************/
 
@@ -31,7 +34,7 @@
 #include <X11/Xos.h>
 #include <X11/Xatom.h>
 
-RCSTAG("$Id: qfnt_x11.cpp,v 2.14 1997/04/04 15:54:13 hanord Exp $");
+RCSTAG("$Id: qfnt_x11.cpp,v 2.18.2.1 1997/11/18 14:16:05 paul Exp $");
 
 
 static const int fontFields = 14;
@@ -443,11 +446,28 @@ void QFont::initFontInfo() const
     f->s.pointSize = atoi(tokens[PointSize]);
     f->s.styleHint = QFont::AnyStyle;	// ### any until we match families
 
-    if ( strcmp( tokens[CharsetRegistry], "iso8859" ) == 0 &&
-	 strcmp( tokens[CharsetEncoding], "1"	    ) == 0 )
-	f->s.charSet = QFont::Latin1;
-    else
+    if ( strcmp( tokens[CharsetRegistry], "iso8859" ) == 0 ) {
+	if ( strcmp( tokens[CharsetEncoding], "1" ) == 0 )
+	    f->s.charSet = QFont::Latin1;
+	else if ( strcmp( tokens[CharsetEncoding], "2" ) == 0 )
+	    f->s.charSet = QFont::Latin2;
+	else if ( strcmp( tokens[CharsetEncoding], "3" ) == 0 )
+	    f->s.charSet = QFont::Latin3;
+	else if ( strcmp( tokens[CharsetEncoding], "4" ) == 0 )
+	    f->s.charSet = QFont::Latin4;
+	else if ( strcmp( tokens[CharsetEncoding], "5" ) == 0 )
+	    f->s.charSet = QFont::Latin5;
+	else if ( strcmp( tokens[CharsetEncoding], "6" ) == 0 )
+	    f->s.charSet = QFont::Latin6;
+	else if ( strcmp( tokens[CharsetEncoding], "7" ) == 0 )
+	    f->s.charSet = QFont::Latin7;
+	else if ( strcmp( tokens[CharsetEncoding], "8" ) == 0 )
+	    f->s.charSet = QFont::Latin8;
+	else if ( strcmp( tokens[CharsetEncoding], "9" ) == 0 )
+	    f->s.charSet = QFont::Latin9;
+    } else {
 	f->s.charSet = QFont::AnyCharSet;
+    }
 
     char slant	= tolower( tokens[Slant][0] );
     f->s.italic = (slant == 'o' || slant == 'i');
@@ -501,7 +521,7 @@ void QFont::load( HANDLE ) const
 	if ( !d->fin ) {			// font was never loaded
 	    d->fin = new QFontInternal( n );
 	    CHECK_PTR( d->fin );
-	    fontDict->insert( n, d->fin );
+	    fontDict->insert( d->fin->name(), d->fin );
 	}
     }
     XFontStruct *f = d->fin->f;
@@ -521,7 +541,7 @@ void QFont::load( HANDLE ) const
 	// If we get a cache overflow, we make room for this font only
 	if ( size > fontCache->maxCost() + reserveCost )
 	    fontCache->setMaxCost( size + reserveCost );
-	if ( !fontCache->insert(n, d->fin, size) ) {
+	if ( !fontCache->insert(d->fin->name(), d->fin, size) ) {
 #if defined(DEBUG)
 	    fatal( "QFont::load: Cache overflow error" );
 #endif
@@ -592,17 +612,69 @@ int QFont_Private::fontMatchScore( char	 *fontName,	 QString &buffer,
 	 IS_ZERO(tokens[AverageWidth]) )
 	*scalable = TRUE;			// scalable font
 
-    switch( charSet() ) {
-	case Latin1 :
-	    if ( strcmp( tokens[CharsetRegistry], "iso8859" ) == 0 &&
-		 strcmp( tokens[CharsetEncoding], "1"	 ) == 0 )
+    if ( charSet() == AnyCharSet ) {
+	score |= CharSetScore;
+    } else if ( strcmp( tokens[CharsetRegistry], "iso8859" ) == 0 ) {
+	// need to mask away non-8859 charsets here
+	switch( charSet() ) {
+	case Latin1:
+	    if ( strcmp( tokens[CharsetEncoding], "1" ) == 0 )
 		score |= CharSetScore;
 	    else
 		exactMatch = FALSE;
 	    break;
-	case AnyCharSet :
-	    score |= CharSetScore;
+	case Latin2:
+	    if ( strcmp( tokens[CharsetEncoding], "2" ) == 0 )
+		score |= CharSetScore;
+	    else
+		exactMatch = FALSE;
 	    break;
+	case Latin3:
+	    if ( strcmp( tokens[CharsetEncoding], "3" ) == 0 )
+		score |= CharSetScore;
+	    else
+		exactMatch = FALSE;
+	    break;
+	case Latin4:
+	    if ( strcmp( tokens[CharsetEncoding], "4" ) == 0 )
+		score |= CharSetScore;
+	    else
+		exactMatch = FALSE;
+	    break;
+	case Latin5:
+	    if ( strcmp( tokens[CharsetEncoding], "5" ) == 0 )
+		score |= CharSetScore;
+	    else
+		exactMatch = FALSE;
+	    break;
+	case Latin6:
+	    if ( strcmp( tokens[CharsetEncoding], "6" ) == 0 )
+		score |= CharSetScore;
+	    else
+		exactMatch = FALSE;
+	    break;
+	case Latin7:
+	    if ( strcmp( tokens[CharsetEncoding], "7" ) == 0 )
+		score |= CharSetScore;
+	    else
+		exactMatch = FALSE;
+	    break;
+	case Latin8:
+	    if ( strcmp( tokens[CharsetEncoding], "8" ) == 0 )
+		score |= CharSetScore;
+	    else
+		exactMatch = FALSE;
+	    break;
+	case Latin9:
+	    if ( strcmp( tokens[CharsetEncoding], "9" ) == 0 )
+		score |= CharSetScore;
+	    else
+		exactMatch = FALSE;
+	    break;
+	default:
+	    // ### should not get here
+	    break;
+	}
     }
 
     char pitch = tolower( tokens[Spacing][0] );
@@ -972,11 +1044,11 @@ int QFontMetrics::width( const char *str, int len ) const
 
   If \e len is negative (default value), the whole string is used.
 
-  Note that the bounding rectangle may extend to the left of (0,0) and
-  that the text output may cover \e all pixels in the bounding rectangle.
+  Note that the bounding rectangle may extend to the left of (0,0),
+  e.g. for italicized fonts, and that the text output may cover \e all
+  pixels in the bounding rectangle.
 
-  \sa width()
-*/
+  \sa width() */
 
 QRect QFontMetrics::boundingRect( const char *str, int len ) const
 {
