@@ -1,5 +1,5 @@
 /****************************************************************************
-** $Id: qfont.cpp,v 2.9 1997/04/04 15:54:13 hanord Exp $
+** $Id: qfont.cpp,v 2.17 1997/08/06 15:05:36 hanord Exp $
 **
 ** Implementation of QFont, QFontMetrics and QFontInfo classes
 **
@@ -7,13 +7,16 @@
 **
 ** Copyright (C) 1992-1997 Troll Tech AS.  All rights reserved.
 **
-** This file is part of the non-commercial distribution of Qt 1.2.
+** This file is part of the free edition of Qt 1.33.
 **
 ** See the file LICENSE included in the distribution for the usage
 ** and distribution terms, or http://www.troll.no/qt/license.html.
 **
-** (This file differs from the one in the commercial distribution of
-** Qt only by this comment.)
+** IMPORTANT NOTE: You may NOT copy this file or any part of it into
+** your own programs or libraries.
+**
+** (This file differs from the one in the commercial edition of Qt
+** only by this comment.)
 **
 *****************************************************************************/
 
@@ -28,7 +31,7 @@
 #include "qdstream.h"
 #include <ctype.h>
 
-RCSTAG("$Id: qfont.cpp,v 2.9 1997/04/04 15:54:13 hanord Exp $");
+RCSTAG("$Id: qfont.cpp,v 2.17 1997/08/06 15:05:36 hanord Exp $");
 
 
 /*!
@@ -121,7 +124,8 @@ void QFont::init()
 
 /*!
   \internal
-  Constructs a font that gets a deep copy of \e data.
+  Constructs a font that gets a 
+  \link shclass.html deep copy\endlink of \e data.
 */
 
 QFont::QFont( QFontData *data )
@@ -154,10 +158,12 @@ QFont::QFont()
     d->ref();
 }
 
+
 /*!
   Constructs a font object with the specified \e family, \e pointSize,
-  \e weight and \e italic settings.
-  If \e pointSize is less than or equal to 0 it is set to 1.
+  \e weight and \e italic settings. If \e pointSize is less than or
+  equal to 0 it is set to 1.
+
   \sa setFamily(), setPointSize(), setWeight(), setItalic()
 */
 
@@ -172,6 +178,31 @@ QFont::QFont( const char *family, int pointSize, int weight, bool italic )
     d->req.italic    = italic;
 }
 
+
+
+/*!
+  Constructs a font object with the specified \e family, \e pointSize,
+  \e weight, \e italic and \a charSet settings.  If \e pointSize is
+  less than or equal to 0 it is set to 1.
+
+  \sa setFamily(), setPointSize(), setWeight(), setItalic()
+*/
+
+QFont::QFont( const char *family, int pointSize, int weight, bool italic,
+	      CharSet charSet)
+{
+    init();
+    d->req.family    = family;
+    if ( pointSize <= 0 )
+	pointSize = 1;
+    d->req.pointSize = pointSize * 10;
+    d->req.weight    = weight;
+    d->req.italic    = italic;
+    d->req.charSet   = charSet;
+}
+
+
+
 /*!
   Constructs a font that is a copy of \e font.
 */
@@ -188,8 +219,9 @@ QFont::QFont( const QFont &font )
 
 QFont::~QFont()
 {
-    if ( d->deref() )
+    if ( d->deref() ) {
 	delete d;
+    }
 }
 
 
@@ -1075,7 +1107,7 @@ void QFontMetrics::reset( const QPainter *p )
   <ol>
   <li> The QFontMetrics constructor with a QFont
   creates a font metrics object for a screen-compatible font,
-  i.e. the font must not a printer font.
+  i.e. the font must not be a printer font.
   <li> QWidget::fontMetrics() returns the font metrics for a
   widget's current font. The font metrics object is automatically
   updated if somebody sets a new widget font.
@@ -1351,7 +1383,7 @@ void QFontInfo::reset( const QPainter *p )
   <ol>
   <li> The QFontInfo constructor with a QFont
   creates a font info object for a screen-compatible font,
-  i.e. the font must not a printer font.
+  i.e. the font must not be a printer font.
   <li> QWidget::fontInfo() returns the font info for a
   widget's current font. The font info object is automatically
   updated if somebody sets a new widget font.
@@ -1589,6 +1621,7 @@ bool QFontInfo::strikeOut() const
 
 /*!
   Returns the fixed pitch value of the matched window system font.
+  A fixed pitch font is a font that has constant character pixel width.  
   \sa QFont::fixedPitch()
 */
 
