@@ -1,5 +1,5 @@
 /****************************************************************************
-** $Id: qgmanagr.cpp,v 2.11 1997/03/26 17:56:29 paul Exp $
+** $Id: qgmanagr.cpp,v 2.21 1997/09/29 13:40:32 hanord Exp $
 **
 ** Implementation of QGGeometry class
 **
@@ -7,21 +7,25 @@
 **
 ** Copyright (C) 1992-1997 Troll Tech AS.  All rights reserved.
 **
-** This file is part of the non-commercial distribution of Qt 1.2.
+** This file is part of the free edition of Qt 1.33.
 **
 ** See the file LICENSE included in the distribution for the usage
 ** and distribution terms, or http://www.troll.no/qt/license.html.
 **
-** (This file differs from the one in the commercial distribution of
-** Qt only by this comment.)
+** IMPORTANT NOTE: You may NOT copy this file or any part of it into
+** your own programs or libraries.
+**
+** (This file differs from the one in the commercial edition of Qt
+** only by this comment.)
 **
 *****************************************************************************/
 
 #include "qgmanagr.h"
 #include "qlist.h"
+#include "qmenubar.h"
 
 
-RCSTAG("$Id: qgmanagr.cpp,v 2.11 1997/03/26 17:56:29 paul Exp $");
+RCSTAG("$Id: qgmanagr.cpp,v 2.21 1997/09/29 13:40:32 hanord Exp $");
 
 
 
@@ -31,7 +35,7 @@ RCSTAG("$Id: qgmanagr.cpp,v 2.11 1997/03/26 17:56:29 paul Exp $");
 
   This class is intended for those who write geometry managers and
   graphical designers. <strong>It is not for the faint of
-  heart. </strong> The QBoxLayout and QGridLayout classes are
+  heart. </strong> The QHBoxLayout, QVBoxLayout and QGridLayout classes are
   available for normal application programming.
 
   Each dimension (horizontal and vertical) is handled independently. Widgets
@@ -46,7 +50,7 @@ RCSTAG("$Id: qgmanagr.cpp,v 2.11 1997/03/26 17:56:29 paul Exp $");
   the same position.
 
 
-  \sa QBoxLayout QGridLayout 
+  \sa QHBoxLayout QVBoxLayout QGridLayout 
 
 */
 
@@ -288,7 +292,9 @@ QSerChain::~QSerChain()
 	delete chain.at(i);
     }
     for ( i = 0; i < (int)branches.count(); i++ ) {
-	delete branches.at(i);
+	QBranchData *bd = branches.at(i);
+	delete bd->chain;
+	delete bd;
     }
 }
 
@@ -336,11 +342,19 @@ void QSerChain::distribute( wDict & wd, int pos, int space )
 
     fixed available = toFixed( space - minSize() );
     if ( available < 0 ) {
-	warning( "QGManager: not enough space to go around" );
+	QString msg;
+	msg.sprintf( "QGManager: not enough space for %d-item %sal chain with %d branches",
+	    chain.count(),
+	    horz( direction() ) ? "horizont" : "vertic",
+	    branches.count()
+	);
+	warning( msg );
 	available = 0;
     }
     int sf = sumStretch();
 
+#define DISTRIBUTE_AVAILABLE_AFTER_MINIMUM // Qt 1.x
+#ifdef DISTRIBUTE_AVAILABLE_AFTER_MINIMUM
     QArray<fixed> sizes( chain.count() );
     int i;
     for ( i = 0; i < (int)chain.count(); i++ )
@@ -370,6 +384,84 @@ void QSerChain::distribute( wDict & wd, int pos, int space )
 	    sizes[i] = siz;
 	}
     }
+#else
+    QArray<fixed> sizes( chain.count() );
+    QArray<bool> forced( chain.count() );
+    int i;
+    for ( i = 0; i < (int)chain.count(); i++ )
+	forced[i] = FALSE;
+    bool doAgain = TRUE;
+    bool done_pass = FALSE;
+    int numChains = chain.count();
+    fixed sp = toFixed( space );
+    while ( doAgain && numChains ) {
+	doAgain = FALSE;
+	for ( i = 0; i < (int)chain.count(); i++ ) {
+	    if ( forced[i] )
+		continue;
+	    fixed maxS = toFixed( chain.at(i)->maxSize() );
+	    fixed minS = toFixed( chain.at(i)->minSize() );
+	    fixed siz;
+	    if ( sf )
+		siz = sp * chain.at(i)->stretch() / sf;
+	    else
+		siz = sp / numChains;
+	    if ( siz <= minS ) {
+		sp -= minS;
+		sizes[i] = minS;
+		forced[i] = TRUE;
+		sf -= chain.at(i)->stretch();
+		numChains--;
+		doAgain = TRUE;
+		break;
+	    } else if ( siz >= maxS ) {
+		if ( done_pass ) {
+		    sp -= maxS;
+		    sizes[i] = maxS;
+		    forced[i] = TRUE;
+		    sf -= chain.at(i)->stretch();
+		    numChains--;
+		    doAgain = TRUE;
+		    break;
+		} else {
+		    doAgain = TRUE;
+		}
+	    }
+	    sizes[i] = siz;
+	}
+	if ( i == (int)chain.count() ) done_pass = TRUE;
+    }
+    fixed tsize = 0;
+    int nsf0 = 0;
+    for ( i = 0; i < (int)chain.count(); i++ ) {
+	tsize += sizes[i];
+	if ( chain.at(i)->stretch() == 0
+	  && sizes[i] < toFixed( chain.at(i)->maxSize() ) ) nsf0++;
+    }
+    while ( tsize < toFixed(space) && nsf0 ) {
+	fixed tsp = (toFixed(space) - tsize);
+	fixed portion = tsp / nsf0;
+	int n0 = nsf0;
+	int n = nsf0;
+	for ( i = 0; i < (int)chain.count(); i++ ) {
+	    if ( chain.at(i)->stretch() == 0
+	      && sizes[i] < toFixed( chain.at(i)->maxSize() ) )
+	    {
+		fixed extra = toFixed( chain.at(i)->maxSize() ) - sizes[i];
+		if ( --n == 0 ) {
+		    portion = tsp - (n0-1) * portion;
+		}
+		if ( extra <= portion ) {
+		    nsf0--;
+		} else {
+		    extra = portion;
+		}
+		sizes[i] += portion;
+		tsize += portion;
+	    }
+	}
+    }
+#endif
 
     int n = chain.count();
     QArray<int> places( n + 1 );
@@ -643,7 +735,6 @@ bool QGManager::eventFilter( QObject *o, QEvent *e )
     if ( e->type() == Event_Resize ) {
 	QResizeEvent *r = (QResizeEvent*)e;
 	resizeHandle( w, r->size() );
-	return TRUE;			    // eat event
     }
 
     return FALSE;			    // standard event processing
@@ -722,20 +813,13 @@ void QGManager::resizeAll()
 
 
     // Resize menubar first to get valid size for rest of widgets to
-     // arrange themselves around.
+    // arrange themselves around.
      
-     // size may not be set yet
-     int ww = QMAX( min.width(), QMIN( main->width(), max.width() ) );
-     int hh = QMAX( min.height(), QMIN( main->height(), max.height() ) );
+    // size may not be set yet
+    int ww = QMAX( min.width(), QMIN( main->width(), max.width() ) );
+    int hh;
  
-     if ( menuBar )
-     {
-	 //### workaround; don't try this at home, kids
-	 QResizeEvent e( QSize( ww, hh ), QSize( 1, 1 ) );
-	 menuBar->eventFilter( main, &e );
-     }
- 
-    int mbh = menuBar ? menuBar->height() : 0;
+    int mbh = menuBar ? ((QMenuBar *)menuBar)->heightForWidth( ww ) : 0;
 
     if ( menuBar && mbh != menuBarHeight ) {
 	int ombh = menuBarHeight;
@@ -746,7 +830,6 @@ void QGManager::resizeAll()
     }
     ww = QMAX( min.width(), QMIN( main->width(), max.width() ) );
     hh = QMAX( min.height(), QMIN( main->height(), max.height() ) );
-
     
     xC->distribute( lookupTable, border, ww - 2*border );
     yC->distribute( lookupTable, mbh + border, hh - 2*border - mbh );
