@@ -1,5 +1,5 @@
 /****************************************************************************
-** $Id: qlayout.cpp,v 2.14 1997/03/23 19:53:40 agulbra Exp $
+** $Id: qlayout.cpp,v 2.24 1997/09/01 19:57:07 paul Exp $
 **
 ** Implementation of layout classes
 **
@@ -7,20 +7,23 @@
 **
 ** Copyright (C) 1992-1997 Troll Tech AS.  All rights reserved.
 **
-** This file is part of the non-commercial distribution of Qt 1.2.
+** This file is part of the free edition of Qt 1.33.
 **
 ** See the file LICENSE included in the distribution for the usage
 ** and distribution terms, or http://www.troll.no/qt/license.html.
 **
-** (This file differs from the one in the commercial distribution of
-** Qt only by this comment.)
+** IMPORTANT NOTE: You may NOT copy this file or any part of it into
+** your own programs or libraries.
+**
+** (This file differs from the one in the commercial edition of Qt
+** only by this comment.)
 **
 *****************************************************************************/
 
 #include "qlayout.h"
 #include "qmenubar.h"
 
-RCSTAG("$Id: qlayout.cpp,v 2.14 1997/03/23 19:53:40 agulbra Exp $");
+RCSTAG("$Id: qlayout.cpp,v 2.24 1997/09/01 19:57:07 paul Exp $");
 
 
 /*!
@@ -81,7 +84,7 @@ QLayout::QLayout( QWidget *parent, int border, int autoBorder, const char *name
 
 /*!
   Returns the main widget of this layout, or 0 if this layout is
-  a sublayout which is not yet inserted.
+  a sub-layout which is not yet inserted.
 */
 
 QWidget * QLayout::mainWidget()
@@ -258,18 +261,19 @@ void QLayout::setMenuBar( QMenuBar *w )
 
 /*!
   \class QBoxLayout qlayout.h
-  \brief The QBoxLayout class specifies child widget geometry.
+  \brief The QBoxLayout class specifies child widget geometry. More... 
 
   Contents are arranged serially, either horizontal or vertical.
   The contents fill the available space.
 
-  A QBoxLayout (box for short) can contain widgets or other
-  boxes.
+  A QBoxLayout can contain widgets or other layouts.
+
+  QHBoxLayout and QVBoxLayout are convenience classes.
 
   \warning The current version does not support deletion of child widgets.
   If you need to remove widgets that are under geometry management, you have to
-  delete the layout manager first.
-*/
+  delete the layout manager first. 
+ */
 
 static inline bool horz( QGManager::Direction dir )
 {
@@ -334,8 +338,7 @@ QBoxLayout::QBoxLayout( Direction d,
 
 
 /*!
-  Deletes this box. Geometry management is terminated if
-  this is a top-level box.
+  Destroys this box. 
 */
 
 QBoxLayout::~QBoxLayout()
@@ -363,7 +366,8 @@ void QBoxLayout::addLayout( QLayout *layout, int stretch )
 {
     if ( !basicManager() ) {
 #if defined(CHECK_STATE)
-	warning( "QBoxLayout::addLayout: Box must be inserted before use" );
+	warning( "QBoxLayout::addLayout: Box must have a widget parent or be\n"
+		 "                       added in another layout before use" );
 #endif
 	return;
     }
@@ -420,12 +424,12 @@ QChain * QBoxLayout::mainHorizontalChain()
 
   \sa addStretch
 */
-//###... Should perhaps replace default space?
 void QBoxLayout::addSpacing( int size )
 {
     if ( !basicManager() ) {
 #if defined(CHECK_STATE)
-	warning( "QBoxLayout::addSpacing: Box must be inserted before use" );
+	warning("QBoxLayout::addSpacing: Box must have a widget parent or be\n"
+		"                        added in another layout before use.");
 #endif
 	return;
     }
@@ -443,7 +447,8 @@ void QBoxLayout::addStretch( int stretch )
 {
     if ( !basicManager() ) {
 #if defined(CHECK_STATE)
-	warning("QBoxLayout::addStretch: Box must be inserted before use" );
+	warning("QBoxLayout::addStretch: Box must have a widget parent or be\n"
+		 "                       added in another layout before use.");
 #endif
 	return;
     }
@@ -462,7 +467,8 @@ void QBoxLayout::addStrut( int size )
 {
     if ( !basicManager() ) {
 #if defined(CHECK_STATE)
-	warning( "QBoxLayout::addStrut: Box must be inserted before use" );
+	warning( "QBoxLayout::addStrut: Box must have a widget parent or be\n"
+		 "                      added in another layout before use." );
 #endif
 	return;
     }
@@ -518,11 +524,17 @@ void QBoxLayout::addMaxStrut( int size)
   \sa addLayout(), addSpacing()
 */
 
+#if QT_VERSION == 200
+#error "Binary compatibility."
+#endif
+
+
 void QBoxLayout::addWidget( QWidget *widget, int stretch, int align )
 {
     if ( !basicManager() ) {
 #if defined(CHECK_STATE)
-	warning( "QBoxLayout::addLayout: Box must be inserted before use" );
+	warning( "QBoxLayout::addLayout: Box must have a widget parent or be\n"
+		 "                       added in another layout before use.");
 #endif
 	return;
     }
@@ -559,6 +571,7 @@ void QBoxLayout::addWidget( QWidget *widget, int stretch, int align )
     pristine = FALSE;
 }
 
+
 /*!
   \fn QBoxLayout::Direction QBoxLayout::direction() const
 
@@ -568,20 +581,65 @@ void QBoxLayout::addWidget( QWidget *widget, int stretch, int align )
   works perpendicular to this direction.
 
   The directions are \c LeftToRight, \c RightToLeft, \c TopToBottom
-  and \c BottomToTop. For the last two, the shorter aliases \c Up and
-  \c Down are also available.
+  and \c BottomToTop. For the last two, the shorter aliases \c Down and
+  \c Up are also available.
 
   \sa addWidget(), addBox(), addSpacing()
 */
 
 
+
+
 /*!
-  \class QGridLayout qlayout.h
+  \class QHBoxLayout qlayout.h
+  \brief The QHBoxLayout class provides a horizontal layout box
 
-  \brief The QGridLayout class specifies child widget geometry.
+  The contents are arranged left to right, they will stretch to fill
+  the available space. 
+
+  \warning The current version does not support deletion of child widgets.
+  If you need to remove widgets that are under geometry management, you have to
+  delete the layout manager first.
+*/
+
+
+/*!
+  Creates a new top-level horizontal box.
+ */
+QHBoxLayout::QHBoxLayout( QWidget *parent, int border,
+		int autoBorder, const char *name )
+    :QBoxLayout( parent, LeftToRight, border, autoBorder, name )
+{
+    
+}
+
+/*!
+  Creates a new horizontal box. You have to add it to another 
+  layout before using it.
+ */
+QHBoxLayout::QHBoxLayout( int autoBorder, const char *name )
+    :QBoxLayout( LeftToRight, autoBorder, name )
+{
+}
+
+
+/*!
+  Destroys this box. 
+*/
+
+QHBoxLayout::~QHBoxLayout()
+{
+}
+
+
+
+/*!
+  \class QVBoxLayout qlayout.h
+  \brief The QVBoxLayout class provides a vertical layout box
+
+  The contents are arranged top to bottom, they will stretch to fill
+  the available space.
 
-  Contents are arranged in a grid. If you need a more flexible layout,
-  see the QBoxLayout class.
 
   \warning The current version does not support deletion of child widgets.
   If you need to remove widgets that are under geometry management, you have to
@@ -589,7 +647,57 @@ void QBoxLayout::addWidget( QWidget *widget, int stretch, int align )
 */
 
 /*!
-  Constructs a new QGridLayout with \a nRows, \a nCols columns
+  Creates a new top-level vertical box.
+ */
+QVBoxLayout::QVBoxLayout( QWidget *parent, int border,
+		int autoBorder, const char *name )
+    :QBoxLayout( parent, TopToBottom, border, autoBorder, name )
+{
+    
+}
+
+/*!
+  Creates a new vertical box. You have to add it to another 
+  layout before using it.
+ */
+QVBoxLayout::QVBoxLayout( int autoBorder, const char *name )
+    :QBoxLayout( TopToBottom, autoBorder, name )
+{
+}
+
+/*!
+  Destroys this box. 
+*/
+
+QVBoxLayout::~QVBoxLayout()
+{
+}
+
+/*!
+  \class QGridLayout qlayout.h
+
+  \brief The QGridLayout class specifies child widget geometry.
+
+  Contents are arranged in a grid. If you need a more flexible layout,
+  see the QHBoxLayout and QVBoxLayout classes.
+
+  To avoid squashed widgets, each column and each row of the grid
+  should have a a minimum size or a nonzero stretch factor. Stretch
+  factors are set with setRowStretch() and setColStretch(). Minimum sizes
+  are set by addColSpacing(), addRowSpacing() and by the minimum sizes
+  of the widgets added. 
+
+  Note that a widget which spans several rows or columns does not
+  influence the minimum size of any of the rows/columns it spans.
+
+  \warning The current version does not support deletion of child widgets.
+  If you need to remove widgets that are under geometry management, you have to
+  delete the layout manager first.  
+*/
+
+
+/*!
+  Constructs a new QGridLayout with \a nRows rows, \a nCols columns
    and main widget \a  parent.	\a parent may not be 0.
 
   \a border is the number of pixels between the edge of the widget and
@@ -696,25 +804,31 @@ void QGridLayout::addWidget( QWidget *w, int row, int col, int align )
 {
     if ( !basicManager() ) {
 #if defined(CHECK_STATE)
-	warning( "QGridLayout::addWidget: Grid must be inserted before use" );
+       warning("QGridLayout::addWidget: Grid must have a widget parent or be\n"
+	       "                        added in another layout before use." );
 #endif
 	return;
     }
+    if ( rows->size() == 0 || cols->size() == 0   ) {
+#if defined(CHECK_RANGE)
+	warning( "QGridLayout::addWidget: Zero sized grid" );
+#endif
+	return;
+    }
+
     addMultiCellWidget( w, row, row, col, col, align );
 }
 
 /*!
   Adds the widget \a w to the cell grid, spanning multiple rows/columns.
 
-  Note that multicell widgets do not define the columns/rows they
-  span.	 Each column must contain at least one widget that does not
-  span multiple columns. Likewise, each row must contain one widget
-  that does not span multiple rows. If your layout does not satisfy this,
-  consider using the QBoxLayout class instead.
+  Note that multicell widgets do not influence the minimum or maximum
+  size of columns/rows they span. Use addColSpacing() or addRowSpacing()
+  to set minimum sizes explicitly.
 
   Alignment is specified by \a align which takes the same arguments as
   QLabel::setAlignment(), alignment has no effect unless you have set
-  QWidget::maximumSize().
+  QWidget::maximumSize().  
 */
 
 void QGridLayout::addMultiCellWidget( QWidget *w, int fromRow, int toRow,
@@ -723,7 +837,14 @@ void QGridLayout::addMultiCellWidget( QWidget *w, int fromRow, int toRow,
     if ( !basicManager() ) {
 #if defined(CHECK_STATE)
 	warning( "QGridLayout::addMultiCellWidget: "
-		 "Grid must be inserted before use" );
+		 "Grid must have a widget parent or be\n"
+		 "        added in another layout before use." );
+#endif
+	return;
+    }
+    if ( rows->size() == 0 || cols->size() == 0   ) {
+#if defined(CHECK_RANGE)
+	warning( "QGridLayout::addMultiCellWidget: Zero sized grid" );
 #endif
 	return;
     }
@@ -776,7 +897,14 @@ void QGridLayout::addLayout( QLayout *layout, int row, int col)
 {
     if ( !basicManager() ) {
 #if defined(CHECK_STATE)
-	warning( "QGridLayout::addLayout: Grid must be inserted before use" );
+       warning("QGridLayout::addLayout: Grid must have a widget parent or be\n"
+	       "                        added in another layout before use." );
+#endif
+	return;
+    }
+    if ( rows->size() == 0 || cols->size() == 0   ) {
+#if defined(CHECK_RANGE)
+	warning( "QGridLayout::addLayout: Zero sized grid" );
 #endif
 	return;
     }
@@ -804,10 +932,18 @@ void QGridLayout::setRowStretch( int row, int stretch )
 {
     if ( !basicManager() ) {
 #if defined(CHECK_STATE)
-	warning( "QGridLayout::setRowStretch: Grid must be inserted before use");
+	warning( "QGridLayout::setRowStretch: Grid must have a widget parent\n"
+		 "        or be added in another layout before use.");
 #endif
 	return;
     }
+    if ( rows->size() == 0 ) {
+#if defined(CHECK_RANGE)
+	warning( "QGridLayout::setRowStretch: Zero sized grid" );
+#endif
+	return;
+    }
+
     QChain *c =	 (*rows)[ row ];
     basicManager()->setStretch( c, stretch );
 }
@@ -829,7 +965,14 @@ void QGridLayout::setColStretch( int col, int stretch )
 {
     if ( !basicManager() ) {
 #if defined(CHECK_STATE)
-	warning( "QGridLayout::setColStretch: Grid must be inserted before use");
+	warning( "QGridLayout::setColStretch: Grid must have a widget parent\n"
+		 "        or be added in another layout before use.");
+#endif
+	return;
+    }
+    if ( cols->size() == 0 ) {
+#if defined(CHECK_RANGE)
+	warning( "QGridLayout::setColStretch: Zero sized grid" );
 #endif
 	return;
     }
@@ -838,6 +981,51 @@ void QGridLayout::setColStretch( int col, int stretch )
 }
 
 
+/*!
+  Sets the minimum width of \a row to \a minsize pixels.
+ */
+void QGridLayout::addRowSpacing( int row, int minsize )
+{
+    if ( !basicManager() ) {
+#if defined(CHECK_STATE)
+	warning( "QGridLayout::setColStretch: Grid must have a widget parent\n"
+		 "        or be added in another layout before use.");
+#endif
+	return;
+    }
+    if ( rows->size() == 0 ) {
+#if defined(CHECK_RANGE)
+	warning( "QGridLayout::addRowSpacing: Zero sized grid" );
+#endif
+	return;
+    }
+
+    QChain *c =	 (*rows)[ row ];
+    basicManager()->addSpacing( c, minsize );
+}
+
+/*!
+  Sets the minimum height of \a col to \a minsize pixels.
+ */
+void QGridLayout::addColSpacing( int col, int minsize )
+{
+    if ( !basicManager() ) {
+#if defined(CHECK_STATE)
+	warning( "QGridLayout::setColStretch: Grid must have a widget parent\n"
+		 "        or be added in another layout before use.");
+#endif
+	return;
+    }
+    if ( cols->size() == 0 ) {
+#if defined(CHECK_RANGE)
+	warning( "QGridLayout::setColStretch: Zero sized grid" );
+#endif
+	return;
+    }
+    QChain *c =	 (*cols)[ col ];
+    basicManager()->addSpacing( c, minsize );
+}
+
 /*!
   \fn QChain *QGridLayout::mainVerticalChain()
   This function returns the main vertical chain.
