@@ -1,5 +1,5 @@
 /****************************************************************************
-** $Id: qpicture.cpp,v 2.0 1996/09/24 10:15:57 agulbra Exp $
+** $Id: qpicture.cpp,v 2.2 1997/08/06 15:05:41 hanord Exp $
 **
 ** Implementation of QPicture class
 **
@@ -7,13 +7,16 @@
 **
 ** Copyright (C) 1992-1997 Troll Tech AS.  All rights reserved.
 **
-** This file is part of the non-commercial distribution of Qt 1.2.
+** This file is part of the free edition of Qt 1.33.
 **
 ** See the file LICENSE included in the distribution for the usage
 ** and distribution terms, or http://www.troll.no/qt/license.html.
 **
-** (This file differs from the one in the commercial distribution of
-** Qt only by this comment.)
+** IMPORTANT NOTE: You may NOT copy this file or any part of it into
+** your own programs or libraries.
+**
+** (This file differs from the one in the commercial edition of Qt
+** only by this comment.)
 **
 *****************************************************************************/
 
@@ -24,7 +27,7 @@
 #include "qfile.h"
 #include "qdstream.h"
 
-RCSTAG("$Id: qpicture.cpp,v 2.0 1996/09/24 10:15:57 agulbra Exp $");
+RCSTAG("$Id: qpicture.cpp,v 2.2 1997/08/06 15:05:41 hanord Exp $");
 
 
 /*!
@@ -84,6 +87,39 @@ static const UINT16 mfhdr_min = 0;		// minor version #
 */
 
 
+/*!
+  \fn bool QPicture::isNull() const
+  Returns TRUE if the picture contains no data, otherwise FALSE.
+*/
+
+/*!
+  \fn uint QPicture::size() const
+  Returns the size of the picture data.
+  \sa data()
+*/
+
+/*!
+  \fn const char *QPicture::data() const
+  Returns a pointer to the picture data.  The returned pointer is null
+  if the picture contains no data.
+  \sa size(), isNull()
+*/
+
+/*!
+  Sets the picture data directly from \a data and \a size. This function
+  copies the input data.
+  \sa data(), size()
+*/
+
+void QPicture::setData( const char *data, uint size )
+{
+    QByteArray a( size );
+    memcpy( a.data(), data, size );
+    pictb.setBuffer( a );			// set byte array in buffer
+    formatOk = FALSE;				// we'll have to check
+}
+
+
 /*!
   Loads a picture from the file specified by \e fileName and returns TRUE
   if successful, otherwise FALSE.
@@ -176,9 +212,9 @@ bool QPicture::play( QPainter *painter )
 	    return FALSE;
 	}
 	formatOk = TRUE;			// picture seems to be ok
-    }
-    else
+    } else {
 	s.device()->at( 10 );			// go directly to the data
+    }
 
     UINT8  c, clen;
     UINT32 nrecords;
@@ -360,9 +396,9 @@ bool QPicture::exec( QPainter *painter, QDataStream &s, int nrecords )
 		break;
 	    case PDC_SETTABARRAY:
 		s >> i_16;
-		if ( i_16 == 0 )
+		if ( i_16 == 0 ) {
 		    painter->setTabArray( 0 );
-		else {
+		} else {
 		    int *ta = new int[i_16];
 		    CHECK_PTR( ta );
 		    for ( int i=0; i<i_16; i++ ) {
@@ -436,8 +472,7 @@ bool QPicture::cmd( int c, QPainter *, QPDevCmdParam *p )
 	s << (UINT32)trecs;			// total number of records
 	formatOk = FALSE;
 	return TRUE;
-    }
-    else if ( c == PDC_END ) {			// end; calc checksum and close
+    } else if ( c == PDC_END ) {		// end; calc checksum and close
 	trecs++;
 	s << (UINT8)c << (UINT8)0;
 	QByteArray buf = pictb.buffer();
@@ -551,8 +586,7 @@ bool QPicture::cmd( int c, QPainter *, QPDevCmdParam *p )
     if ( length < 255 ) {			// write 8-bit length
 	pictb.at(pos - 1);			// position to right index
 	s << (UINT8)length;
-    }
-    else {					// write 32-bit length
+    } else {					// write 32-bit length
 	s << (UINT32)0;				// extend the buffer
 	pictb.at(pos - 1);			// position to right index
 	s << (UINT8)255;			// indicate 32-bit length
