@@ -1,5 +1,5 @@
 /****************************************************************************
-** $Id: qptr_x11.cpp,v 2.16 1997/04/01 14:47:46 aavit Exp $
+** $Id: qptr_x11.cpp,v 2.28 1997/09/29 11:32:53 eiriken Exp $
 **
 ** Implementation of QPainter class for X11
 **
@@ -7,13 +7,16 @@
 **
 ** Copyright (C) 1992-1997 Troll Tech AS.  All rights reserved.
 **
-** This file is part of the non-commercial distribution of Qt 1.2.
+** This file is part of the free edition of Qt 1.33.
 **
 ** See the file LICENSE included in the distribution for the usage
 ** and distribution terms, or http://www.troll.no/qt/license.html.
 **
-** (This file differs from the one in the commercial distribution of
-** Qt only by this comment.)
+** IMPORTANT NOTE: You may NOT copy this file or any part of it into
+** your own programs or libraries.
+**
+** (This file differs from the one in the commercial edition of Qt
+** only by this comment.)
 **
 *****************************************************************************/
 
@@ -31,7 +34,7 @@
 #include <X11/Xutil.h>
 #include <X11/Xos.h>
 
-RCSTAG("$Id: qptr_x11.cpp,v 2.16 1997/04/01 14:47:46 aavit Exp $");
+RCSTAG("$Id: qptr_x11.cpp,v 2.28 1997/09/29 11:32:53 eiriken Exp $");
 
 
 /*****************************************************************************
@@ -403,7 +406,7 @@ static QPaintDeviceDict *pdev_dict = 0;
     QPixmap pm( myWidget->width(), myWidget->height() );
     pm.fill( myWidget->backgroundColor() );
     QPainter::redirect( myWidget, &pm );
-    myWidget->repaint();
+    myWidget->repaint( FALSE );
     QPainter::redirect( myWidget, 0 );
   \endcode
 */
@@ -782,7 +785,7 @@ static uchar *pat_tbl[] = {
   On the X Window System, paint commands are buffered and may not appear
   on the screen immediately. The flush() function flushes the buffer.
 
-  As an alternative to calling begin() and end(), you  use the QPainter
+  As an alternative to calling begin() and end(), you can use the QPainter
   constructor that takes a paint device argument. This is for short-lived
   painters, for example in \link QWidget::paintEvent() paint events\endlink.
 
@@ -791,7 +794,7 @@ static uchar *pat_tbl[] = {
   <li>The \link setFont() font\endlink is set to the default \link
   QApplication::setFont() application font\endlink, or to the \link
   QWidget::setFont() widget's font\endlink if \e pd is a widget.
-  <li>The \link setPen() pen\endlink is set QPen(black,0,SolidLine),
+  <li>The \link setPen() pen\endlink is set to QPen(black,0,SolidLine),
   or to QPen(\link QWidget::foregroundColor() widget->foreground()\endlink,
   0,SolidLine) if \e pd is a widget.
   <li>The \link setBrush() brush\endlink is set to QBrush(NoBrush).
@@ -829,7 +832,8 @@ bool QPainter::begin( const QPaintDevice *pd )
 {
     if ( isActive() ) {				// already active painting
 #if defined(CHECK_STATE)
-	warning( "QPainter::begin: Painter is already active" );
+	warning( "QPainter::begin: Painter is already active."
+		 "\n\tYou must end() the painter before a second begin()" );
 #endif
 	return FALSE;
     }
@@ -1621,6 +1625,38 @@ void QPainter::drawPoint( int x, int y )
     XDrawPoint( dpy, hd, gc, x, y );
 }
 
+/*!
+  Draws/plots an array of points using the current pen.  The
+  \a index and \a npoints arguments allow a subsequence of the
+  array to be drawn.
+*/
+void QPainter::drawPoints( const QPointArray& a, int index, int npoints )
+{
+    if ( npoints < 0 )
+	npoints = a.size() - index;
+    if ( index + npoints > (int)a.size() )
+	npoints = a.size() - index;
+    if ( !isActive() || npoints < 1 || index < 0 || cpen.style() == NoPen )
+	return;
+    QPointArray pa = a;
+    if ( testf(ExtDev|VxF|WxF) ) {
+	if ( testf(ExtDev) ) {
+	    QPDevCmdParam param[1];
+	    for (int i=0; i<npoints; i++) {
+		QPoint p( pa[index+i].x(), pa[index+i].y() );
+		param[0].point = &p;
+		if ( !pdev->cmd(PDC_DRAWPOINT,this,param))
+		    return;
+	    }
+	    if ( !hd ) return;
+	}
+	if ( txop != TxNone )
+	    pa = xForm( a );
+    }
+    XDrawPoints( dpy, hd, gc, (XPoint*)(pa.data()+index), npoints,
+		    CoordModeOrigin );
+}
+
 /*!
   Sets the current point.
   \sa lineTo(), drawLine()
@@ -1766,15 +1802,58 @@ void QPainter::drawRect( int x, int y, int w, int h )
 */
 
 void QPainter::drawWinFocusRect( int x, int y, int w, int h )
+{
+    drawWinFocusRect( x, y, w, h, TRUE, color0 );
+}
+
+/*!
+  Draws a Windows focus rectangle with upper left corner at \e (x,y) and with
+  width \e w and height \e h using a pen color that contrasts with \e bgColor.
+
+  This function draws a stippled rectangle (XOR is not used) that is used
+  to indicate keyboard focus (when the \link QApplication::style() GUI
+  style\endlink is \c WindowStyle).
+
+  The pen color used to draw the rectangle is either white or black
+  depending on the grayness of \e bgColor (see QColor::gray()).
+
+  \warning This function draws nothing if the coordinate system has been
+  \link rotate() rotated\endlink or \link shear() sheared\endlink.
+
+  \sa drawRect(), QApplication::style()
+*/
+void QPainter::drawWinFocusRect( int x, int y, int w, int h, 
+				 const QColor &bgColor )
+{
+    drawWinFocusRect( x, y, w, h, FALSE, bgColor );
+}
+
+
+/*!
+  \internal
+ */
+void QPainter::drawWinFocusRect( int x, int y, int w, int h,
+				 bool xorPaint, const QColor &bgColor )
 {
     if ( !isActive() || txop == TxRotShear )
 	return;
-    static char winfocous_line[] = { 1, 1 };
+    static char winfocus_line[] = { 1, 1 };
 
     QPen     old_pen = cpen;
     RasterOp old_rop = (RasterOp)rop;
-    setPen( white );
-    setRasterOp( XorROP );
+
+    if ( xorPaint ) {
+	if ( QColor::numBitPlanes() <= 8 )
+	    setPen( color1 );
+	else
+	    setPen( white );
+	setRasterOp( XorROP );
+    } else {
+	if ( qGray( bgColor.rgb() ) < 128 )
+	    setPen( white );
+	else
+	    setPen( black );
+    }
 
     if ( testf(ExtDev|VxF|WxF) ) {
 	if ( testf(ExtDev) ) {
@@ -1791,7 +1870,7 @@ void QPainter::drawWinFocusRect( int x, int y, int w, int h )
 	    return;
 	fix_neg_rect( &x, &y, &w, &h );
     }
-    XSetDashes( dpy, gc, 0, winfocous_line, 2 );
+    XSetDashes( dpy, gc, 0, winfocus_line, 2 );
     XSetLineAttributes( dpy, gc, 0, LineOnOffDash, CapButt, JoinMiter );
 
     XDrawRectangle( dpy, hd, gc, x, y, w-1, h-1 );
@@ -2048,6 +2127,14 @@ void QPainter::drawArc( int x, int y, int w, int h, int a, int alen )
 
 void QPainter::drawPie( int x, int y, int w, int h, int a, int alen )
 {
+    // Make sure "a" is 0..360*16, as otherwise a*4 may overflow 16 bits.
+    if ( a > (360*16) ) {
+	a = a % (360*16);
+    } else if ( a < 0 ) {
+	a = a % (360*16);
+	if ( a < 0 ) a += (360*16);
+    }
+
     if ( !isActive() )
 	return;
     if ( testf(ExtDev|VxF|WxF) ) {
@@ -2253,6 +2340,7 @@ void QPainter::drawPolyline( const QPointArray &a, int index, int npoints )
 	    }
 	    QPDevCmdParam param[1];
 	    param[0].ptarr = (QPointArray*)&pa;
+	    // ### ??? We do this even if NoPen?
 	    if ( !pdev->cmd(PDC_DRAWPOLYLINE,this,param) || !hd )
 		return;
 	}
@@ -2402,10 +2490,27 @@ void QPainter::drawPixmap( int x, int y, const QPixmap &pixmap,
 {
     if ( !isActive() || pixmap.isNull() )
 	return;
+
+    // right/bottom
     if ( sw < 0 )
 	sw = pixmap.width()  - sx;
     if ( sh < 0 )
 	sh = pixmap.height() - sy;
+
+    // Sanity-check clipping
+    if ( sx < 0 ) {
+	x -= sx;
+	sx = 0;
+    }
+    if ( sw + sx > pixmap.width() )
+	sw = pixmap.width() - sx;
+    if ( sy < 0 ) {
+	y -= sy;
+	sy = 0;
+    }
+    if ( sh + sy > pixmap.height() )
+	sh = pixmap.height() - sy;
+
     if ( testf(ExtDev|VxF|WxF) ) {
 	if ( testf(ExtDev) || txop == TxScale || txop == TxRotShear ) {
 	    if ( sx != 0 || sy != 0 ||
@@ -2739,6 +2844,19 @@ void QPainter::drawText( int x, int y, const char *str, int len )
   <li> \c GrayText grays out the text.
   </ul>
 
+  Horizontal alignment defaults to AlignLeft and vertical alignment
+  defaults to AlignTop.
+
+  If several of the horizontal or several of the vertical alignment flags   
+  are set, the resulting alignment is undefined.
+
+  If ExpandTabs is set and no \link setTabStops() tab stops \endlink or
+  \link setTabArray() tab array \endlink have been set tabs will expand to
+  the closest reasonable tab stop based on the current font. For \link
+  QFont::setFixedPitch() fixed pitch\endlink (fixed width) fonts you are
+  guaranteed that each tab stop will be at a multiple of eight of the
+  width of the characters in the font.
+
   \a brect (if non-null) is set to the actual bounding rectangle of
   the output.  \a internal is, yes, internal.
 
@@ -2846,6 +2964,12 @@ void QPainter::drawText( int x, int y, int w, int h, int tf,
     if ( decode )				// skip encoding
 	k = len;
 
+    int localTabStops = 0;	       		// tab stops
+    if ( tabstops )
+	localTabStops = tabstops;
+    else
+	localTabStops = fm.width('x')*8;       	// default to 8 times x
+
     while ( k < len ) {				// convert string to codes
 
 	if ( UCHAR(*p) > 32 ) {			// printable character
@@ -2888,8 +3012,8 @@ void QPainter::drawText( int x, int y, int w, int h, int tf,
 			    tabindex++;
 			}
 		    }
-		    if ( cw == 0 && tabstops )	// use fixed tab stops
-			cw = tabstops - tw%tabstops;
+		    if ( cw == 0 )		// use fixed tab stops
+			cw = localTabStops - tw%localTabStops;
 		    cc = TABSTOP | QMIN(tw+cw,MAXWIDTH);
 		} else {			// convert TAB to space
 		    cc = ' ';
@@ -3004,6 +3128,34 @@ void QPainter::drawText( int x, int y, int w, int h, int tf,
     else
 	xp = 0;					// left aligned
 
+#if defined(CHECK_RANGE)
+    int hAlignFlags = 0;
+    if ( (tf & AlignRight) == AlignRight )
+	hAlignFlags++;
+    if ( (tf & AlignHCenter) == AlignHCenter )
+	hAlignFlags++;
+    if ( (tf & AlignLeft ) == AlignLeft )
+	hAlignFlags++;
+
+    if ( hAlignFlags > 1 ) 
+	warning("QPainter::drawText: More than one of AlignRight, AlignLeft\n"
+		"                    and AlignHCenter set in the tf parameter."
+		);
+
+    int vAlignFlags = 0;
+    if ( (tf & AlignTop) == AlignTop )
+	vAlignFlags++;
+    if ( (tf & AlignVCenter) == AlignVCenter )
+	vAlignFlags++;
+    if ( (tf & AlignBottom ) == AlignBottom )
+	vAlignFlags++;
+
+    if ( hAlignFlags > 1 )
+	warning("QPainter::drawText: More than one of AlignTop, AlignBottom\n"
+		"                    and AlignVCenter set in the tf parameter."
+		);
+#endif // CHECK_RANGE
+
     QRect br( x+xp, y+yp, maxwidth, nlines*fheight );
     if ( brect )				// set bounding rect
 	*brect = br;
