@@ -1,5 +1,5 @@
 /****************************************************************************
-** $Id: qrgn_x11.cpp,v 2.0 1996/09/24 10:16:02 agulbra Exp $
+** $Id: qrgn_x11.cpp,v 2.4 1997/08/06 15:05:46 hanord Exp $
 **
 ** Implementation of QRegion class for X11
 **
@@ -7,13 +7,16 @@
 **
 ** Copyright (C) 1992-1997 Troll Tech AS.  All rights reserved.
 **
-** This file is part of the non-commercial distribution of Qt 1.2.
+** This file is part of the free edition of Qt 1.33.
 **
 ** See the file LICENSE included in the distribution for the usage
 ** and distribution terms, or http://www.troll.no/qt/license.html.
 **
-** (This file differs from the one in the commercial distribution of
-** Qt only by this comment.)
+** IMPORTANT NOTE: You may NOT copy this file or any part of it into
+** your own programs or libraries.
+**
+** (This file differs from the one in the commercial edition of Qt
+** only by this comment.)
 **
 *****************************************************************************/
 
@@ -25,7 +28,7 @@
 #include <X11/Xutil.h>
 #include <X11/Xos.h>
 
-RCSTAG("$Id: qrgn_x11.cpp,v 2.0 1996/09/24 10:16:02 agulbra Exp $");
+RCSTAG("$Id: qrgn_x11.cpp,v 2.4 1997/08/06 15:05:46 hanord Exp $");
 
 
 static QRegion *empty_region = 0;
@@ -38,7 +41,8 @@ static void cleanup_empty_region()
 
 
 /*!
-  Constructs an empty region.
+  Constructs an null region.
+  \sa isNull()
 */
 
 QRegion::QRegion()
@@ -53,7 +57,7 @@ QRegion::QRegion()
 }
 
 /*!
-  Internal constructor that creates an empty region.
+  Internal constructor that creates a null region.
 */
 
 QRegion::QRegion( bool )
@@ -86,14 +90,12 @@ QRegion::QRegion( const QRect &r, RegionType t )
 	xr.height = rr.height();
 	XUnionRectWithRegion( &xr, data->rgn, data->rgn );
 	id = QRGN_SETRECT;
-    }
-    else if ( t == Ellipse ) {			// elliptic region
+    } else if ( t == Ellipse ) {			// elliptic region
 	QPointArray a;
 	a.makeEllipse( rr.x(), rr.y(), rr.width(), rr.height() );
 	id = QRGN_SETELLIPSE;
 	data->rgn = XPolygonRegion( (XPoint*)a.data(), a.size(), EvenOddRule );
-    }
-    else {
+    } else {
 #if defined(CHECK_RANGE)
 	warning( "QRegion: Invalid region type" );
 #endif
@@ -116,8 +118,7 @@ QRegion::QRegion( const QPointArray &a, bool winding )
     if ( winding ) {
 	r = WindingRule;
 	c = QRGN_SETPTARRAY_WIND;
-    }
-    else {
+    } else {
 	r = EvenOddRule;
 	c = QRGN_SETPTARRAY_ALT;
     }
@@ -128,7 +129,8 @@ QRegion::QRegion( const QPointArray &a, bool winding )
 }
 
 /*!
-  Constructs a region which is a shallow copy of \e r.
+  Constructs a region which is a
+  \link shclass.html shallow copy\endlink of \e r.
 */
 
 QRegion::QRegion( const QRegion &r )
@@ -150,8 +152,9 @@ QRegion::~QRegion()
 }
 
 /*!
-  Assigns a shallow copy of \e r to this region and returns a reference to
-  the region.
+  Assigns a 
+  \link shclass.html shallow copy\endlink of \e r to this region and
+  returns a reference to the region.
 */
 
 QRegion &QRegion::operator=( const QRegion &r )
@@ -167,7 +170,8 @@ QRegion &QRegion::operator=( const QRegion &r )
 
 
 /*!
-  Returns a deep copy of the region.
+  Returns a 
+  \link shclass.html deep copy\endlink of the region.
 */
 
 QRegion QRegion::copy() const
@@ -180,7 +184,12 @@ QRegion QRegion::copy() const
 
 
 /*!
-  Returns TRUE if the region is a null region.
+  Returns TRUE if the region is a null region, otherwise FALSE.
+
+  A null region is a region that has not been initialized. The
+  documentation for isEmpty() contains an example that shows how to use
+  isNull() and isEmpty().
+
   \sa isEmpty()
 */
 
@@ -190,13 +199,31 @@ bool QRegion::isNull() const
 }
 
 /*!
-  Returns TRUE if the region is empty, or FALSE if it is
-  non-empty. \sa isNull()
+  Returns TRUE if the region is empty, or FALSE if it is non-empty.
+
+  Example:
+  \code
+    QRegion r1( 10, 10, 20, 20 );
+    QRegion r2( 40, 40, 20, 20 );
+    QRegion r3;
+    r1.isNull();		// FALSE
+    r1.isEmpty();		// FALSE
+    r3.isNull();		// TRUE
+    r3.isEmpty();		// TRUE
+    r3 = r1.intersect( r2 );	// r3 = intersection of r1 and r2
+    r3.isNull();		// FALSE
+    r3.isEmpty();		// TRUE
+    r3 = r1.unite( r2 );	// r3 = union of r1 and r2
+    r3.isNull();		// FALSE
+    r3.isEmpty();		// FALSE
+  \endcode
+
+  \sa isNull()
 */
 
 bool QRegion::isEmpty() const
 {
-    return XEmptyRegion( data->rgn );
+    return data->bop.isNull() || XEmptyRegion( data->rgn );
 }
 
 
@@ -277,7 +304,7 @@ QRegion QRegion::subtract( const QRegion &r ) const
   Returns a region which is this region XOR \e r.
 */
 
-QRegion QRegion::xor( const QRegion &r ) const
+QRegion QRegion::eor( const QRegion &r ) const
 {
     QRegion result( TRUE );
     XXorRegion( data->rgn, r.data->rgn, result.data->rgn );
