@@ -1,5 +1,5 @@
 /****************************************************************************
-** $Id: qsize.cpp,v 2.0 1996/09/24 10:16:03 agulbra Exp $
+** $Id: qsize.cpp,v 2.4 1997/08/27 13:39:55 agulbra Exp $
 **
 ** Implementation of QSize class
 **
@@ -7,13 +7,16 @@
 **
 ** Copyright (C) 1992-1997 Troll Tech AS.  All rights reserved.
 **
-** This file is part of the non-commercial distribution of Qt 1.2.
+** This file is part of the free edition of Qt 1.33.
 **
 ** See the file LICENSE included in the distribution for the usage
 ** and distribution terms, or http://www.troll.no/qt/license.html.
 **
-** (This file differs from the one in the commercial distribution of
-** Qt only by this comment.)
+** IMPORTANT NOTE: You may NOT copy this file or any part of it into
+** your own programs or libraries.
+**
+** (This file differs from the one in the commercial edition of Qt
+** only by this comment.)
 **
 *****************************************************************************/
 
@@ -21,7 +24,7 @@
 #include "qsize.h"
 #include "qdstream.h"
 
-RCSTAG("$Id: qsize.cpp,v 2.0 1996/09/24 10:16:03 agulbra Exp $");
+RCSTAG("$Id: qsize.cpp,v 2.4 1997/08/27 13:39:55 agulbra Exp $");
 
 
 /*!
@@ -44,7 +47,7 @@ RCSTAG("$Id: qsize.cpp,v 2.0 1996/09/24 10:16:03 agulbra Exp $");
 
 /*!
   \fn QSize::QSize()
-  Constructs a size with undefined width and height.
+  Constructs a size with invalid (negative) width and height.
 */
 
 /*!
@@ -59,7 +62,7 @@ RCSTAG("$Id: qsize.cpp,v 2.0 1996/09/24 10:16:03 agulbra Exp $");
 
 /*!
   \fn bool QSize::isEmpty() const
-  Returns TRUE if the width is less than 0 or the height is less than 0,
+  Returns TRUE if the width is <= 0 or the height is <= 0,
   otherwise FALSE.
 */
 
@@ -93,6 +96,17 @@ RCSTAG("$Id: qsize.cpp,v 2.0 1996/09/24 10:16:03 agulbra Exp $");
   \sa height(), setWidth()
 */
 
+/*!
+  Swaps the values of width and height.
+*/
+
+void QSize::transpose()
+{
+    QCOORD tmp = wd;
+    wd = ht;
+    ht = tmp;
+}
+
 /*!
   \fn QCOORD &QSize::rwidth()
   Returns a reference to the width.
