@@ -1,5 +1,5 @@
 /****************************************************************************
-** $Id: qbitarry.cpp,v 2.0 1996/09/24 10:16:07 agulbra Exp $
+** $Id: qbitarry.cpp,v 2.4 1997/09/29 15:45:42 eiriken Exp $
 **
 ** Implementation of QBitArray class
 **
@@ -7,20 +7,23 @@
 **
 ** Copyright (C) 1992-1997 Troll Tech AS.  All rights reserved.
 **
-** This file is part of the non-commercial distribution of Qt 1.2.
+** This file is part of the free edition of Qt 1.33.
 **
 ** See the file LICENSE included in the distribution for the usage
 ** and distribution terms, or http://www.troll.no/qt/license.html.
 **
-** (This file differs from the one in the commercial distribution of
-** Qt only by this comment.)
+** IMPORTANT NOTE: You may NOT copy this file or any part of it into
+** your own programs or libraries.
+**
+** (This file differs from the one in the commercial edition of Qt
+** only by this comment.)
 **
 *****************************************************************************/
 
 #include "qbitarry.h"
 #include "qdstream.h"
 
-RCSTAG("$Id: qbitarry.cpp,v 2.0 1996/09/24 10:16:07 agulbra Exp $");
+RCSTAG("$Id: qbitarry.cpp,v 2.4 1997/09/29 15:45:42 eiriken Exp $");
 
 
 #define SHBLOCK	 ((bitarr_data*)(sharedBlock()))
@@ -34,6 +37,32 @@ RCSTAG("$Id: qbitarry.cpp,v 2.0 1996/09/24 10:16:07 agulbra Exp $");
   It is probably a bad idea to use it in any other context.
 */
 
+/*!
+  \fn QBitVal::QBitVal (QBitArray* a, uint i) 
+
+  Constructs a reference to an element in a QBitArray.  This is
+  what QBitArray::operator[] contructs its return value with.
+*/
+
+/*!
+  \fn QBitVal::operator int() 
+
+  Returns the value referenced by the QBitVal.
+*/
+
+/*!
+  \fn QBitVal& QBitVal::operator= (const QBitVal& v) 
+
+  Sets the value referenced by the QBitVal to that referenced by another
+  QBitVal.
+*/
+
+/*!
+  \fn QBitVal& QBitVal::operator= (int v) 
+
+  Sets the value referenced by the QBitVal.
+*/
+
 
 /*!
   \class QBitArray qbitarry.h
