@@ -1,5 +1,5 @@
 /****************************************************************************
-** $Id: qbuffer.cpp,v 2.6 1997/02/09 17:11:39 hanord Exp $
+** $Id: qbuffer.cpp,v 2.12 1997/08/06 15:05:49 hanord Exp $
 **
 ** Implementation of QBuffer class
 **
@@ -7,20 +7,23 @@
 **
 ** Copyright (C) 1992-1997 Troll Tech AS.  All rights reserved.
 **
-** This file is part of the non-commercial distribution of Qt 1.2.
+** This file is part of the free edition of Qt 1.33.
 **
 ** See the file LICENSE included in the distribution for the usage
 ** and distribution terms, or http://www.troll.no/qt/license.html.
 **
-** (This file differs from the one in the commercial distribution of
-** Qt only by this comment.)
+** IMPORTANT NOTE: You may NOT copy this file or any part of it into
+** your own programs or libraries.
+**
+** (This file differs from the one in the commercial edition of Qt
+** only by this comment.)
 **
 *****************************************************************************/
 
 #include "qbuffer.h"
 #include <stdlib.h>
 
-RCSTAG("$Id: qbuffer.cpp,v 2.6 1997/02/09 17:11:39 hanord Exp $");
+RCSTAG("$Id: qbuffer.cpp,v 2.12 1997/08/06 15:05:49 hanord Exp $");
 
 
 /*!
@@ -128,6 +131,11 @@ bool QBuffer::setBuffer( QByteArray buf )
     return TRUE;
 }
 
+/*!
+  \fn QByteArray QBuffer::buffer() const
+
+  Returns the buffer most recently set by setBuffer(), or at construction.
+*/
 
 /*!
   Opens the file specified by the file name currently set, using the mode \e m.
@@ -163,6 +171,7 @@ bool QBuffer::open( int m  )
     } else {
 	index = 0;
     }
+    a_inc = 16;
     setState( IO_Open );
     setStatus( 0 );
     return TRUE;
@@ -178,6 +187,7 @@ void QBuffer::close()
     if ( isOpen() ) {
 	setFlags( IO_Direct );
 	index = 0;
+	a_inc = 16;
     }
 }
 
@@ -197,6 +207,12 @@ void QBuffer::flush()
   \sa size()
 */
 
+/*!
+  \fn uint QBuffer::size() const
+  Returns the number of bytes in the buffer.
+  \sa at()
+*/
+
 /*!
   Sets the buffer index to \e pos. Returns TRUE if successful, otherwise FALSE.
   \sa size()
@@ -245,13 +261,10 @@ int QBuffer::readBlock( char *p, uint len )
 #endif
     if ( (uint)index + len > a.size() ) {	// overflow
 	if ( (uint)index >= a.size() ) {
-#if defined(CHECK_RANGE)
-	    warning( "QBuffer::readBlock: Buffer read error" );
-#endif
 	    setStatus( IO_ReadError );
 	    return -1;
 	} else {
-	    len = a.size() - (uint)index - 1;
+	    len = a.size() - (uint)index;
 	}
     }
     memcpy( p, a.data()+index, len );
@@ -260,8 +273,9 @@ int QBuffer::readBlock( char *p, uint len )
 }
 
 /*!
-  Writes \e len bytes from \e p to the buffer and returns the number of
-  bytes actually written.
+  Writes \e len bytes from \e p into the buffer at the current index,
+  overwriting any characters there and extending the buffer if necessary.
+  Returns the number of bytes actually written.
 
   Returns -1 if a serious error occurred.
 
@@ -366,9 +380,6 @@ int QBuffer::getch()
     }
 #endif
     if ( (uint)index+1 > a.size() ) {		// overflow
-#if defined(CHECK_RANGE)
-	warning( "QBuffer::getch: Buffer read error" );
-#endif
 	setStatus( IO_ReadError );
 	return -1;
     }
@@ -376,7 +387,9 @@ int QBuffer::getch()
 }
 
 /*!
-  Writes the character \e ch to the buffer.
+  Writes the character \e ch into the buffer, overwriting
+  the character at the current index, extending the buffer
+  if necessary.
 
   Returns \e ch, or -1 if some error occurred.
 
