@@ -1,5 +1,5 @@
 /****************************************************************************
-** $Id: qdatetm.cpp,v 2.3 1997/02/03 14:20:15 hanord Exp $
+** $Id: qdatetm.cpp,v 2.7 1997/09/25 10:14:03 hanord Exp $
 **
 ** Implementation of date and time classes
 **
@@ -7,13 +7,16 @@
 **
 ** Copyright (C) 1992-1997 Troll Tech AS.  All rights reserved.
 **
-** This file is part of the non-commercial distribution of Qt 1.2.
+** This file is part of the free edition of Qt 1.33.
 **
 ** See the file LICENSE included in the distribution for the usage
 ** and distribution terms, or http://www.troll.no/qt/license.html.
 **
-** (This file differs from the one in the commercial distribution of
-** Qt only by this comment.)
+** IMPORTANT NOTE: You may NOT copy this file or any part of it into
+** your own programs or libraries.
+**
+** (This file differs from the one in the commercial edition of Qt
+** only by this comment.)
 **
 *****************************************************************************/
 
@@ -41,11 +44,11 @@
 extern "C" int gettimeofday( struct timeval *, struct timezone * );
 #endif
 
-RCSTAG("$Id: qdatetm.cpp,v 2.3 1997/02/03 14:20:15 hanord Exp $");
+RCSTAG("$Id: qdatetm.cpp,v 2.7 1997/09/25 10:14:03 hanord Exp $");
 
 
 static const uint FIRST_DAY	= 2361222;	// Julian day for 17520914
-static const int  FIRST_YEAR	= 1752;
+static const int  FIRST_YEAR	= 1752;		// ### wrong for many countries
 static const uint SECS_PER_DAY	= 86400;
 static const uint MSECS_PER_DAY = 86400000;
 static const uint SECS_PER_HOUR = 3600;
@@ -389,9 +392,9 @@ uint QDate::greg2jul( int y, int m, int d )
     uint c, ya;
     if ( y <= 99 )
 	y += 1900;
-    if ( m > 2 )
+    if ( m > 2 ) {
 	m -= 3;
-    else {
+    } else {
 	m += 9;
 	y--;
     }
@@ -422,9 +425,9 @@ void QDate::jul2greg( uint jd, int &y, int &m, int &d )
     m = (5*x - 3)/153;
     x = 5*x - 3 - 153*m;
     d = (x + 5)/5;
-    if ( m < 10 )
+    if ( m < 10 ) {
 	m += 3;
-    else {
+    } else {
 	m -= 9;
 	y++;
     }
@@ -566,7 +569,7 @@ bool QTime::setHMS( int h, int m, int s, int ms )
 QTime QTime::addSecs( int nsecs ) const
 {
     QTime t;
-    t.ds = (ds + nsecs*1000) % MSECS_PER_DAY;
+    t.ds = ((int)ds + nsecs*1000) % MSECS_PER_DAY;
     return t;
 }
 
@@ -577,7 +580,7 @@ QTime QTime::addSecs( int nsecs ) const
 
 int QTime::secsTo( const QTime &t ) const
 {
-    return (t.ds - ds)/1000;
+    return ((int)t.ds - (int)ds)/1000;
 }
 
 /*!
@@ -597,7 +600,7 @@ QTime QTime::addMSecs( int ms ) const
 
 int QTime::msecsTo( const QTime &t ) const
 {
-    return t.ds - ds;
+    return (int)t.ds - (int)ds;
 }
 
 
@@ -728,7 +731,7 @@ bool QTime::isValid( int h, int m, int s, int ms )
     QTime t;
     t.start();				// start clock
     ... // some lengthy task
-    debug( "%d\n", t.elapsed() );		// prints # msecs elapsed
+    debug( "%d\n", t.elapsed() );	// prints # msecs elapsed
   \endcode
 
   \sa restart(), elapsed()
@@ -909,7 +912,7 @@ QDateTime QDateTime::addDays( int ndays ) const
 
 QDateTime QDateTime::addSecs( int nsecs ) const
 {
-    int dd = (t.ds + nsecs*1000)/MSECS_PER_DAY;
+    int dd = ((int)t.ds + nsecs*1000)/MSECS_PER_DAY;
     return QDateTime( d.addDays(dd), t.addSecs(nsecs) );
 }
 
@@ -938,7 +941,7 @@ int QDateTime::daysTo( const QDateTime &dt ) const
 
 int QDateTime::secsTo( const QDateTime &dt ) const
 {
-    return t.secsTo( dt.t ) + d.daysTo( dt.d )*SECS_PER_DAY;
+    return t.secsTo(dt.t) + d.daysTo(dt.d)*SECS_PER_DAY;
 }
 
 
