@@ -1,5 +1,5 @@
 /****************************************************************************
-** $Id: qfile.cpp,v 2.4 1997/03/11 18:45:29 hanord Exp $
+** $Id: qfile.cpp,v 2.10 1997/08/29 10:45:21 paul Exp $
 **
 ** Implementation of QFile class
 **
@@ -7,20 +7,23 @@
 **
 ** Copyright (C) 1992-1997 Troll Tech AS.  All rights reserved.
 **
-** This file is part of the non-commercial distribution of Qt 1.2.
+** This file is part of the free edition of Qt 1.33.
 **
 ** See the file LICENSE included in the distribution for the usage
 ** and distribution terms, or http://www.troll.no/qt/license.html.
 **
-** (This file differs from the one in the commercial distribution of
-** Qt only by this comment.)
+** IMPORTANT NOTE: You may NOT copy this file or any part of it into
+** your own programs or libraries.
+**
+** (This file differs from the one in the commercial edition of Qt
+** only by this comment.)
 **
 *****************************************************************************/
 
 #include "qfile.h"
 #include "qfiledef.h"
 
-RCSTAG("$Id: qfile.cpp,v 2.4 1997/03/11 18:45:29 hanord Exp $");
+RCSTAG("$Id: qfile.cpp,v 2.10 1997/08/29 10:45:21 paul Exp $");
 
 
 /*!
@@ -31,10 +34,25 @@ RCSTAG("$Id: qfile.cpp,v 2.4 1997/03/11 18:45:29 hanord Exp $");
   \ingroup files
   \ingroup stream
 
-  QFile is an I/O device for reading and writing binary and text files.	 A
+  QFile is an I/O device for reading and writing binary and text files.	A
   QFile may be used by itself (readBlock and writeBlock) or by more
-  conveniently using QDataStream or QTextStream.  Most of its behavior is
-  inherited from QIODevice.
+  conveniently using QDataStream or QTextStream.
+
+  Here is a code fragment that uses QTextStream to read a text
+  file line by line. It prints each line with a line number.
+  \code
+    QFile f("file.txt");
+    if ( f.open(IO_ReadOnly) ) {    // file opened successfully
+	QTextStream t( &f );	    // use a text stream
+	QString s;
+	int n = 1;
+	while ( !t.eof() ) {	    // until end of file...
+	    s = t.readLine();	    // line of text excluding '\n'
+	    printf( "%3d: %s\n", n++, (const char *)s );
+	}
+	f.close();
+    }
+  \endcode
 
   The QFileInfo class holds detailed information about a file, such as
   access permissions, file dates and file types.
@@ -60,11 +78,12 @@ QFile::QFile()
 */
 
 QFile::QFile( const char *name )
+    : fn(name)
 {
     init();
-    fn = name;
 }
 
+
 /*!
   Destroys a QFile.  Calls close().
 */
@@ -74,6 +93,7 @@ QFile::~QFile()
     close();
 }
 
+
 /*!
   \internal
   Initialize internal data.
@@ -143,7 +163,7 @@ bool QFile::exists() const
 {
     if ( fn.isEmpty() )
 	return FALSE;
-    return ACCESS( fn.data(), F_OK ) == 0;
+    return ACCESS(fn.data(), F_OK) == 0;
 }
 
 /*!
@@ -174,22 +194,35 @@ bool QFile::exists( const char *fileName )
   Opens the file specified by the file name currently set, using the mode \e m.
   Returns TRUE if successful, otherwise FALSE.
 
-  The mode parameter \e m must be a combination of the following flags.
+  The mode parameter \e m must be a combination of the following flags:
   <ul>
-  <li>\c IO_Raw specified raw (unbuffered) file access.
-  <li>\c IO_ReadOnly opens a file in read-only mode.
-  <li>\c IO_WriteOnly opens a file in write-only mode.
-  <li>\c IO_ReadWrite opens a file in read/write mode.
-  <li>\c IO_Append sets the file index to the end of the file.
+  <li>\c IO_Raw specified raw (non-buffered) file access.
+  <li>\c IO_ReadOnly opens the file in read-only mode.
+  <li>\c IO_WriteOnly opens the file in write-only mode.
+  <li>\c IO_ReadWrite opens the file in read/write mode, equivalent to
+  \c (IO_ReadOnly|IO_WriteOnly).
+  <li>\c IO_Append opens the file in append mode. This mode is very useful
+  when you want to write something to a log file. The file index is set to
+  the end of the file. Note that the result is undefined if you position the
+  file index manually using at() in append mode.
   <li>\c IO_Truncate truncates the file.
   <li>\c IO_Translate enables carriage returns and linefeed translation
-  for text files under MS-DOS, Window, OS/2 and Macintosh.  Cannot be
-  combined with \c IO_Raw.
+  for text files under MS-DOS, Window, OS/2 and Macintosh.
   </ul>
 
   The raw access mode is best when I/O is block-operated using 4kB block size
-  or greater.  Buffered access works better when reading small portions of
+  or greater. Buffered access works better when reading small portions of
   data at a time.
+  
+  <strong>Important:</strong> When working with buffered files, data may
+  not be written to the file at once. Call \link flush() flush\endlink
+  to make sure the data is really written.
+  
+  \warning We have experienced problems with some C libraries when a buffered
+  file is opened for both reading and writing. If a read operation takes place
+  immediately after a write operation, the read buffer contains garbage data.
+  Worse, the same garbage is written to the file. Calling flush() before
+  readBlock() solved this problem.
 
   If the file does not exist and \c IO_WriteOnly or \c IO_ReadWrite is
   specified, it is created.
@@ -202,7 +235,7 @@ bool QFile::exists( const char *fileName )
     f2.open( IO_ReadOnly | IO_Translate );
   \endcode
 
-  \sa name(), close(), isOpen()
+  \sa name(), close(), isOpen(), flush()
 */
 
 bool QFile::open( int m )
@@ -229,22 +262,28 @@ bool QFile::open( int m )
     }
     bool ok = TRUE;
     if ( isRaw() ) {				// raw file I/O
-	int oflags = 0;
-	if ( isReadable() )
-	    oflags |= OPEN_RDONLY;
+	int oflags = OPEN_RDONLY;
+	if ( isReadable() && isWritable() )
+	    oflags = OPEN_RDWR;
+	else if ( isWritable() )
+	    oflags = OPEN_WRONLY;
 	if ( flags() & IO_Append ) {		// append to end of file?
 	    if ( flags() & IO_Truncate )
 		oflags |= (OPEN_CREAT | OPEN_TRUNC);
 	    else
 		oflags |= (OPEN_APPEND | OPEN_CREAT);
 	    setFlags( flags() | IO_WriteOnly ); // append implies write
-	} else if ( isWritable() )		// create/trunc if writable
-	    oflags |= (OPEN_CREAT | OPEN_TRUNC);//   but not append
-	if ( isWritable() )
-	    oflags |= isReadable() ? OPEN_WRONLY : OPEN_RDWR;
+	} else if ( isWritable() ) {		// create/trunc if writable
+	    if ( flags() & IO_Truncate )
+		oflags |= (OPEN_CREAT | OPEN_TRUNC);
+	    else
+		oflags |= OPEN_CREAT;
+	}
 #if defined(HAS_TEXT_FILEMODE) && !defined(_OS_MAC_)
 	if ( isTranslated() )
 	    oflags |= OPEN_TEXT;
+	else
+	    oflags |= OPEN_BINARY;
 #endif
 #if defined(HAS_ASYNC_FILEMODE)
 	if ( isAsynchronous() )
@@ -252,12 +291,13 @@ bool QFile::open( int m )
 #endif
 	fd = OPEN( (const char *)fn, oflags, 0666 );
 	if ( fd != -1 ) {			// open successful
-	    length = LSEEK( fd, 0, SEEK_END );	// get size of file
-	    if ( !(flags() & IO_Append) )	// reset file position
-		LSEEK( fd, 0, SEEK_SET );
-	}
-	else
+	    STATBUF st;
+	    FSTAT( fd, &st );
+	    length = st.st_size;
+	    index  = (flags() & IO_Append) == 0 ? 0 : length;
+	} else {
 	    ok = FALSE;
+	}
     } else {					// buffered file I/O
 	const char *perm = 0;
 	char perm2[4];
@@ -267,16 +307,17 @@ bool QFile::open( int m )
 	    perm = isReadable() ? "a+" : "a";
 	} else {
 	    if ( isReadWrite() ) {
-		if ( flags() & IO_Truncate )
+		if ( flags() & IO_Truncate ) {
 		    perm = "w+";
-		else {
+		} else {
 		    perm = "r+";
 		    try_create = TRUE;		// try to create if not exists
 		}
-	    } else if ( isReadable() )
+	    } else if ( isReadable() ) {
 		perm = "r";
-	    else if ( isWritable() )
+	    } else if ( isWritable() ) {
 		perm = "w";
+	    }
 	}
 	strcpy( perm2, perm );
 #if defined(HAS_TEXT_FILEMODE)
@@ -291,21 +332,17 @@ bool QFile::open( int m )
 	    fh = fopen( (const char *)fn, perm2 );
 	}
 	if ( fh ) {
-	    if ( flags() & IO_Append ) {	// index is at end of file
-		length = ftell( fh );
-		index = length;
-	    } else {				// calc size of file
-		fseek( fh, 0, SEEK_END );
-		length = ftell( fh );
-		fseek( fh, 0, SEEK_SET );
-		index = 0;
-	    }
-	} else
+	    STATBUF st;
+	    FSTAT( FILENO(fh), &st );
+	    length = st.st_size;
+	    index  = (flags() & IO_Append) == 0 ? 0 : length;
+	} else {
 	    ok = FALSE;
+	}
     }
-    if ( ok )
+    if ( ok ) {
 	setState( IO_Open );
-    else {
+    } else {
 	init();
 	if ( errno == EMFILE )			// no more file handles/descrs
 	    setStatus( IO_ResourceError );
@@ -315,6 +352,7 @@ bool QFile::open( int m )
     return ok;
 }
 
+
 /*!
   Opens a file in the mode \e m using an existing file handle \e f.
   Returns TRUE if successful, otherwise FALSE.
@@ -357,20 +395,21 @@ bool QFile::open( int m, FILE *f )
     if ( fh == stdin || fh == stdout || fh == stderr ) {
 	length = INT_MAX;
     } else {
+	STATBUF st;
+	FSTAT( FILENO(fh), &st );
+	length = st.st_size;
 	index = ftell( fh );
-	fseek( fh, 0, SEEK_END );
-	length = ftell( fh );
-	fseek( fh, index, SEEK_SET );
     }
     return TRUE;
 }
 
+
 /*!
   Opens a file in the mode \e m using an existing file descriptor \e f.
   Returns TRUE if successful, otherwise FALSE.
 
   When a QFile is opened using this function, close() does not actually
-  close the file, only flushes it.
+  close the file.
 
   \warning If \e f is one of 0 (stdin), 1 (stdout) or 2 (stderr), you may not
   be able to seek. size() is set to \c INT_MAX (in limits.h).
@@ -394,13 +433,15 @@ bool QFile::open( int m, int f )
     if ( fd == 0 || fd == 1 || fd == 2 ) {
 	length = INT_MAX;
     } else {
-	index = LSEEK( fd, 0, SEEK_CUR );
-	length = LSEEK( fd, 0, SEEK_END );
-	LSEEK( fd, index, SEEK_SET );
+	STATBUF st;
+	FSTAT( fd, &st );
+	length = st.st_size;
+	index  = LSEEK(fd, 0, SEEK_CUR);
     }
     return TRUE;
 }
 
+
 /*!
   Closes an open file.
 
@@ -429,6 +470,7 @@ void QFile::close()
     init();					// restore internal state
 }
 
+
 /*!
   Flushes the file buffer to the disk.
 
@@ -449,21 +491,15 @@ void QFile::flush()
 
 uint QFile::size() const
 {
-    if ( isOpen() ) {
-	return length;
-    } else {
-	QFile f( fn );
-	uint s;
-	if ( f.open(IO_ReadOnly) ) {
-	    s = f.size();
-	    f.close();
-	} else {
-	    s = 0;
-	}
-	return s;
-    }
+    STATBUF st;
+    if ( isOpen() )
+	FSTAT( fh ? FILENO(fh) : fd, &st );
+    else
+	STAT( fn, &st );
+    return st.st_size;
 }
 
+
 /*!
   \fn int QFile::at() const
   Returns the file index.
@@ -483,7 +519,10 @@ uint QFile::size() const
     f.close();
   \endcode
 
-  \sa size()
+  \warning The result is undefined if the file was \link open() opened\endlink
+  using the \c IO_Append specifier.
+
+  \sa size(), open()
 */
 
 bool QFile::at( int pos )
@@ -494,13 +533,12 @@ bool QFile::at( int pos )
 #endif
 	return FALSE;
     }
-    bool ok = TRUE;
+    bool ok;
     if ( isRaw() ) {				// raw file
-	if ( LSEEK(fd, pos, SEEK_SET) == -1 )
-	    ok = FALSE;
+	pos = LSEEK(fd, pos, SEEK_SET);
+	ok = pos != -1;
     } else {					// buffered file
-	if ( fseek(fh, pos, SEEK_SET) != 0 )
-	    ok = FALSE;
+	ok = fseek(fh, pos, SEEK_SET) == 0;
     }
     if ( ok )
 	index = pos;
@@ -511,6 +549,7 @@ bool QFile::at( int pos )
     return ok;
 }
 
+
 /*!
   Returns TRUE if the end of file has been reached, otherwise FALSE.
   \sa size()
@@ -540,6 +579,12 @@ bool QFile::atEnd() const
 
   Returns -1 if a serious error occurred.
 
+  \warning We have experienced problems with some C libraries when a buffered
+  file is opened for both reading and writing. If a read operation takes place
+  immediately after a write operation, the read buffer contains garbage data.
+  Worse, the same garbage is written to the file. Calling flush() before
+  readBlock() solved this problem.
+
   \sa writeBlock()
 */
 
@@ -565,12 +610,17 @@ int QFile::readBlock( char *p, uint len )
     return nread;
 }
 
+
 /*!
   Writes \e len bytes from \e p to the file and returns the number of
   bytes actually written.
 
   Returns -1 if a serious error occurred.
 
+  <strong>Important:</strong> When working with buffered files, data may
+  not be written to the file at once. Call \link flush() flush\endlink
+  to make sure the data is really written.
+
   \sa readBlock()
 */
 
@@ -617,8 +667,8 @@ int QFile::writeBlock( const char *p, uint len )
   Reads a line of text.
 
   Reads bytes from the file until end-of-line is reached, or up to \a
-  maxlen bytes, and returns the number of bytes read.  The terminating
-  newline is not stripped.
+  maxlen bytes, and returns the number of bytes read, or -1 in case of
+  error.  The terminating newline is not stripped.
 
   This function is efficient only for buffered files.  Avoid
   readLine() for files that have been opened with the \c IO_Raw
