@@ -1,5 +1,5 @@
 /****************************************************************************
-** $Id: qglist.cpp,v 2.0 1996/09/24 10:16:12 agulbra Exp $
+** $Id: qglist.cpp,v 2.2 1997/08/06 15:05:52 hanord Exp $
 **
 ** Implementation of QGList and QGListIterator classes
 **
@@ -7,13 +7,16 @@
 **
 ** Copyright (C) 1992-1997 Troll Tech AS.  All rights reserved.
 **
-** This file is part of the non-commercial distribution of Qt 1.2.
+** This file is part of the free edition of Qt 1.33.
 **
 ** See the file LICENSE included in the distribution for the usage
 ** and distribution terms, or http://www.troll.no/qt/license.html.
 **
-** (This file differs from the one in the commercial distribution of
-** Qt only by this comment.)
+** IMPORTANT NOTE: You may NOT copy this file or any part of it into
+** your own programs or libraries.
+**
+** (This file differs from the one in the commercial edition of Qt
+** only by this comment.)
 **
 *****************************************************************************/
 
@@ -21,7 +24,7 @@
 #include "qgvector.h"
 #include "qdstream.h"
 
-RCSTAG("$Id: qglist.cpp,v 2.0 1996/09/24 10:16:12 agulbra Exp $");
+RCSTAG("$Id: qglist.cpp,v 2.2 1997/08/06 15:05:52 hanord Exp $");
 
 
 /*!
@@ -249,15 +252,13 @@ QLNode *QGList::locate( uint index )
     if ( distance < 0 )
 	distance = -distance;
     if ( (uint)distance < index && (uint)distance < numNodes - index ) {
-	node =	curNode;
-	forward = index > (uint)curIndex;	// start from current node
-    }
-    else if ( index < numNodes - index ) {	// start from first node
+	node =	curNode;			// start from current node
+	forward = index > (uint)curIndex;
+    } else if ( index < numNodes - index ) {	// start from first node
 	node = firstNode;
 	distance = index;
 	forward = TRUE;
-    }
-    else {					// start from last node
+    } else {					// start from last node
 	node = lastNode;
 	distance = numNodes - index - 1;
 	if ( distance < 0 )
@@ -267,8 +268,7 @@ QLNode *QGList::locate( uint index )
     if ( forward ) {				// now run through nodes
 	while ( distance-- )
 	    node = node->next;
-    }
-    else {
+    } else {
 	while ( distance-- )
 	    node = node->prev;
     }
@@ -344,8 +344,7 @@ bool QGList::insertAt( uint index, GCI d )
     if ( index == 0 ) {				// insert at head of list
 	prepend( d );
 	return TRUE;
-    }
-    else if ( index == numNodes ) {		// append at tail of list
+    } else if ( index == numNodes ) {		// append at tail of list
 	append( d );
 	return TRUE;
     }
@@ -398,26 +397,24 @@ QLNode *QGList::unlink()
 	return 0;
     register QLNode *n = curNode;		// unlink this node
     if ( n == firstNode ) {			// removing first node ?
-	if ( (firstNode = n->next) )
+	if ( (firstNode = n->next) ) {
 	    firstNode->prev = 0;
-	else {
+	} else {
 	    lastNode = curNode = 0;		// list becomes empty
 	    curIndex = -1;
 	}
-    }
-    else {
+    } else {
 	if ( n == lastNode ) {			// removing last node ?
 	    lastNode = n->prev;
 	    lastNode->next = 0;
-	}
-	else {					// neither last nor first node
+	} else {				// neither last nor first node
 	    n->prev->next = n->next;
 	    n->next->prev = n->prev;
 	}
     }
-    if ( n->next )				// change current node
+    if ( n->next ) {				// change current node
 	curNode = n->next;
-    else if ( n->prev ) {
+    } else if ( n->prev ) {
 	curNode = n->prev;
 	curIndex--;
     }
@@ -645,8 +642,7 @@ int QGList::findRef( GCI d, bool fromStart )
     if ( fromStart ) {				// start from first node
 	n = firstNode;
 	index = 0;
-    }
-    else {					// start from current node
+    } else {					// start from current node
 	n = curNode;
 	index = curIndex;
     }
@@ -671,8 +667,7 @@ int QGList::find( GCI d, bool fromStart )
     if ( fromStart ) {				// start from first node
 	n = firstNode;
 	index = 0;
-    }
-    else {					// start from current node
+    } else {					// start from current node
 	n = curNode;
 	index = curIndex;
     }
