@@ -1,5 +1,5 @@
 /****************************************************************************
-** $Id: qregexp.cpp,v 2.1 1997/03/11 12:10:31 hanord Exp $
+** $Id: qregexp.cpp,v 2.4 1997/08/06 15:05:54 hanord Exp $
 **
 ** Implementation of QRegExp class
 **
@@ -7,13 +7,16 @@
 **
 ** Copyright (C) 1992-1997 Troll Tech AS.  All rights reserved.
 **
-** This file is part of the non-commercial distribution of Qt 1.2.
+** This file is part of the free edition of Qt 1.33.
 **
 ** See the file LICENSE included in the distribution for the usage
 ** and distribution terms, or http://www.troll.no/qt/license.html.
 **
-** (This file differs from the one in the commercial distribution of
-** Qt only by this comment.)
+** IMPORTANT NOTE: You may NOT copy this file or any part of it into
+** your own programs or libraries.
+**
+** (This file differs from the one in the commercial edition of Qt
+** only by this comment.)
 **
 *****************************************************************************/
 
@@ -21,7 +24,7 @@
 #include <ctype.h>
 #include <stdlib.h>
 
-RCSTAG("$Id: qregexp.cpp,v 2.1 1997/03/11 12:10:31 hanord Exp $");
+RCSTAG("$Id: qregexp.cpp,v 2.4 1997/08/06 15:05:54 hanord Exp $");
 
 
 /*!
@@ -156,6 +159,8 @@ QRegExp::~QRegExp()
 QRegExp &QRegExp::operator=( const QRegExp &r )
 {
     rxstring = (const char *)r.rxstring;
+    cs = r.cs;
+    wc = r.wc;
     compile();
     return *this;
 }
@@ -185,6 +190,13 @@ bool QRegExp::operator==( const QRegExp &r ) const
     return rxstring == r.rxstring && cs == r.cs && wc == r.wc;
 }
 
+/*!
+  \fn bool QRegExp::operator!=( const QRegExp &r ) const
+
+  Returns TRUE if this regexp is \e not equal to \e r.
+
+  \sa operator==()
+*/
 
 /*!
   \fn bool QRegExp::isEmpty() const
