@@ -1,5 +1,5 @@
 /****************************************************************************
-** $Id: qtstream.cpp,v 2.4 1997/01/13 12:59:55 hanord Exp $
+** $Id: qtstream.cpp,v 2.8 1997/08/06 15:05:55 hanord Exp $
 **
 ** Implementation of QTextStream class
 **
@@ -7,13 +7,16 @@
 **
 ** Copyright (C) 1992-1997 Troll Tech AS.  All rights reserved.
 **
-** This file is part of the non-commercial distribution of Qt 1.2.
+** This file is part of the free edition of Qt 1.33.
 **
 ** See the file LICENSE included in the distribution for the usage
 ** and distribution terms, or http://www.troll.no/qt/license.html.
 **
-** (This file differs from the one in the commercial distribution of
-** Qt only by this comment.)
+** IMPORTANT NOTE: You may NOT copy this file or any part of it into
+** your own programs or libraries.
+**
+** (This file differs from the one in the commercial edition of Qt
+** only by this comment.)
 **
 *****************************************************************************/
 
@@ -24,7 +27,7 @@
 #include <ctype.h>
 #include <stdlib.h>
 
-RCSTAG("$Id: qtstream.cpp,v 2.4 1997/01/13 12:59:55 hanord Exp $");
+RCSTAG("$Id: qtstream.cpp,v 2.8 1997/08/06 15:05:55 hanord Exp $");
 
 
 /*!
@@ -34,15 +37,36 @@ RCSTAG("$Id: qtstream.cpp,v 2.4 1997/01/13 12:59:55 hanord Exp $");
   \brief The QTextStream class provides basic functions for reading and
   writing text using a QIODevice.
 
-  The text stream class has a functional interface that is very similar to
-  that of the standard C++ iostream class.  The difference between
-  iostream and QTextStream is that our stream operates on a QIODevice.
+  \define endl
+  \define bin
+  \define oct
+  \define dec
+  \define hex
+  \define flush
+  \define ws
+  \define reset
+
+  The text stream class has a functional interface that is very
+  similar to that of the standard C++ iostream class.  The difference
+  between iostream and QTextStream is that our stream operates on a
+  QIODevice, which is easily subclassed, while iostream operates on
+  FILE * pointers, which can not be subclassed.
+
+  Qt provides several global functions similar to the ones in iostream:
+  <ul>
+  <li> \c bin sets the QTextStream to output binary numbers
+  <li> \c oct sets the QTextStream to output octal numbers
+  <li> \c dec sets the QTextStream to output decimal numbers
+  <li> \c hex sets the QTextStream to output hexadecimal numbers
+  <li> \c endl forces a line break
+  <li> \c flush forces the QIODevice to flush any buffered data
+  <li> \c ws eats any available white space (on input)
+  <li> \c reset resets the QTextStream to its default mode (see reset()).
+  </ul>
 
   The QTextStream class reads and writes ASCII text and it is not
   appropriate for dealing with binary data (but QDataStream is).
 
-  This class is not fully documented yet.
-
   \sa QDataStream
 */
 
@@ -74,6 +98,27 @@ RCSTAG("$Id: qtstream.cpp,v 2.4 1997/01/13 12:59:55 hanord Exp $");
   is executed with the argument.
 */
 
+/*! \fn QTSManip::QTSManip (QTSMFI m, int a)
+
+  Constructs a QTSManip object which will call \m (a member function
+  in QTextStream which accepts a single int) with argument \a a when
+  QTSManip::exec() is called.  Used internally in e.g. endl:
+
+  \code
+    s << "some text" << endl << "more text";
+  \endcode
+*/
+
+/*! \fn void QTSManip::exec (QTextStream& s)
+
+  Calls the member function specified in the constructor, for object
+  \a s.  Used internally in e.g. endl:
+
+  \code
+    s << "some text" << endl << "more text";
+  \endcode
+*/
+
 
 /*****************************************************************************
   QTextStream member functions
@@ -510,7 +555,7 @@ static double input_double( QTextStream *s )
 
 /*!
   Reads a signed \c short integer from the stream and returns a reference to
-  the stream.
+  the stream. See flags() for an explanation of expected input format.
 */
 
 QTextStream &QTextStream::operator>>( signed short &i )
@@ -523,7 +568,7 @@ QTextStream &QTextStream::operator>>( signed short &i )
 
 /*!
   Reads an unsigned \c short integer from the stream and returns a reference to
-  the stream.
+  the stream. See flags() for an explanation of expected input format.
 */
 
 QTextStream &QTextStream::operator>>( unsigned short &i )
@@ -536,7 +581,7 @@ QTextStream &QTextStream::operator>>( unsigned short &i )
 
 /*!
   Reads a signed \c int from the stream and returns a reference to the
-  stream.
+  stream. See flags() for an explanation of expected input format.
 */
 
 QTextStream &QTextStream::operator>>( signed int &i )
@@ -549,7 +594,7 @@ QTextStream &QTextStream::operator>>( signed int &i )
 
 /*!
   Reads an unsigned \c int from the stream and returns a reference to the
-  stream.
+  stream. See flags() for an explanation of expected input format.
 */
 
 QTextStream &QTextStream::operator>>( unsigned int &i )
@@ -562,7 +607,7 @@ QTextStream &QTextStream::operator>>( unsigned int &i )
 
 /*!
   Reads a signed \c long int from the stream and returns a reference to the
-  stream.
+  stream. See flags() for an explanation of expected input format.
 */
 
 QTextStream &QTextStream::operator>>( signed long &i )
@@ -575,7 +620,7 @@ QTextStream &QTextStream::operator>>( signed long &i )
 
 /*!
   Reads an unsigned \c long int from the stream and returns a reference to the
-  stream.
+  stream. See flags() for an explanation of expected input format.
 */
 
 QTextStream &QTextStream::operator>>( unsigned long &i )
@@ -588,6 +633,7 @@ QTextStream &QTextStream::operator>>( unsigned long &i )
 
 /*!
   Reads a \c float from the stream and returns a reference to the stream.
+  See flags() for an explanation of expected input format.
 */
 
 QTextStream &QTextStream::operator>>( float &f )
@@ -600,6 +646,7 @@ QTextStream &QTextStream::operator>>( float &f )
 
 /*!
   Reads a \c double from the stream and returns a reference to the stream.
+  See flags() for an explanation of expected input format.
 */
 
 QTextStream &QTextStream::operator>>( double &f )
@@ -1021,6 +1068,30 @@ QTextStream &QTextStream::writeRawBytes( const char *s, uint len )
 /*!
   \fn int QTextStream::flags() const
   Returns the current stream flags. The default value is 0.
+
+  The meaning of the flags are:
+  <ul>
+    <li> \e skipws - Not currently used - whitespace always skipped
+    <li> \e left - Numeric fields are left-aligned
+    <li> \e right - Not currently used (by default numerics are right aligned)
+    <li> \e internal - Put any padding spaces between +/- and value
+    <li> \e bin - Output \e and input only in binary
+    <li> \e oct - Output \e and input only in octal
+    <li> \e dec - Output \e and input only in decimal
+    <li> \e hex - Output \e and input only in hexadecimal
+    <li> \e showbase - Annotate numeric outputs with 0b, 0, or 0x if in
+		\e bin, \e oct, or \e hex format
+    <li> \e showpoint - Not currently used
+    <li> \e uppercase - Use 0B and 0X rather than 0b and 0x
+    <li> \e showpos - Show + for positive numeric values
+    <li> \e scientific - Use scientific notation for floating point values
+    <li> \e fixed - Use fixed-point notation for floating point values
+  </ul>
+
+  Note that unless \e bin, \e oct, \e dec, or \e hex is set, the input base is
+    octal if the value starts with 0, hexadecimal if it starts with 0x, binary
+    if the value starts with 0b, and decimal otherwise.
+
   \sa setf(), unsetf()
 */
 
@@ -1029,7 +1100,7 @@ QTextStream &QTextStream::writeRawBytes( const char *s, uint len )
   Sets the stream flags to \e f.
   Returns the previous stream flags.
 
-  \sa setf(), unsetf()
+  \sa setf(), unsetf(), flags()
 */
 
 /*!
