@@ -1,5 +1,5 @@
 /****************************************************************************
-** $Id: qbutton.cpp,v 2.10 1997/03/25 16:26:05 hanord Exp $
+** $Id: qbutton.cpp,v 2.40.2.1 1998/01/13 17:38:16 agulbra Exp $
 **
 ** Implementation of QButton widget class
 **
@@ -7,13 +7,16 @@
 **
 ** Copyright (C) 1992-1997 Troll Tech AS.  All rights reserved.
 **
-** This file is part of the non-commercial distribution of Qt 1.2.
+** This file is part of the free edition of Qt 1.33.
 **
 ** See the file LICENSE included in the distribution for the usage
 ** and distribution terms, or http://www.troll.no/qt/license.html.
 **
-** (This file differs from the one in the commercial distribution of
-** Qt only by this comment.)
+** IMPORTANT NOTE: You may NOT copy this file or any part of it into
+** your own programs or libraries.
+**
+** (This file differs from the one in the commercial edition of Qt
+** only by this comment.)
 **
 *****************************************************************************/
 
@@ -23,10 +26,86 @@
 #include "qpainter.h"
 #include "qkeycode.h"
 #include "qtimer.h"
+#include "qaccel.h"
+#include "qpmcache.h"
+#include <ctype.h>
 
-RCSTAG("$Id: qbutton.cpp,v 2.10 1997/03/25 16:26:05 hanord Exp $");
+RCSTAG("$Id: qbutton.cpp,v 2.40.2.1 1998/01/13 17:38:16 agulbra Exp $");
+
+
+static const int autoRepeatDelay  = 300;
+static const int autoRepeatPeriod = 100;
+
+static const int drawingPixWidth  = 300;
+static const int drawingPixHeight = 100;
+
+
+/*
+  Returns a pixmap of dimension (drawingPixWidth x drawingPixHeight). The
+  pixmap is used by paintEvent for flicker-free drawing.
+ */
+
+static QPixmap *getDrawingPixmap()
+{
+    QPixmap *pm = QPixmapCache::find( "$qt-button-drawpix" );
+    if ( !pm ) {
+	pm = new QPixmap( drawingPixWidth, drawingPixHeight );
+	CHECK_PTR( pm );
+	QPixmapCache::insert( "$qt-button-drawpix", pm );
+    }
+    return pm;
+}
+
+
+struct QButtonData
+{
+    QButtonData() : group(0) {}
+    QButtonGroup *group;
+    QTimer	  timer;
+};
+
+void QButton::ensureData()
+{
+    if ( !d ) {
+	d = new QButtonData;
+	CHECK_PTR( d );
+	connect(&d->timer, SIGNAL(timeout()), this, SLOT(autoRepeatTimeout()));
+    }
+}
+
+QButtonGroup *QButton::group() const
+{
+    return d ? d->group : 0;
+}
+
+void QButton::setGroup( QButtonGroup* g )
+{
+    ensureData();
+    d->group = g;
+}
+
+QTimer *QButton::timer()
+{
+    ensureData();
+    return &d->timer;
+}
+
+
+/*
+  Internal function that returns the shortcut character in a string.
+  Returns zero if no shortcut character was found.
+  Example:
+    shortcutChar("E&xit") returns 'x'.
+*/
+
+static int shortcutChar( const char *str )
+{
+    const char *p = str ? strchr(str, '&') : 0;
+    while ( p && *p && p[1] == '&' )
+	p = strchr( p+2, '&' );
+    return (p && *p && p[1] && p[1] != '&') ? p[1] : 0;
+}
 
-static const int autoRepeatPeriod = 200;
 
 /*!
   \class QButton qbutton.h
@@ -39,27 +118,84 @@ static const int autoRepeatPeriod = 200;
   The QButton class implements an abstract button, and lets subclasses
   specify how to reply to user actions and how to draw the button.
 
-  The QButton class has three signals. The pressed() signal is emitted
-  when the left mouse button is pressed while the cursor is inside the
-  button. After being pressed, the button will be down until the left
-  mouse button is again released, which causes a released() signal. If the
-  left mouse button is released when the cursor is inside the button, the
-  clicked() signal will be emitted.
+  QButton provides both push and toggle buttons.  The QRadioButton and
+  QCheckBox classes provide only toggle buttons, QPushButton provides
+  both toggle and push buttons.
 
-  There are two types of buttons; standard buttons and toggle buttons. A
-  standard button can either be pressed down or released. The QPushButton
-  class is an example of a standard button. A toggle button has an
-  additional flag that is toggled each time the button is clicked. The
-  QRadioButton and QCheckBox classes are examples of toggle buttons.
+  Any button can have either a text or pixmap label.  setText() sets
+  the button to be a text button and setPixmap() sets it to be a
+  pixmap button.  The text/pixmap is manipulated as necessary to
+  create "disabled" appearance when the button is \link
+  QWidget::setEnabled() disabled\endlink.
 
-  The button label can be a \link setText() text\endlink or a \link
-  setPixmap() pixmap\endlink.  It is up to widget implementation to
-  display the text or the pixmap.  All the button classess provided
-  with Qt can show both texts and pixmaps.
+  QButton provides most of the states used for buttons:
+  <ul>
+  <li>isDown() determines whether the button is \e pressed down.
+  <li>isOn() determines whether the button is \e on.
+  Only toggle buttons can be switched on and off  (see below).
+  <li>isEnabled() determines whether the button can be pressed by the
+  user.
+  <li>setAutoRepeat() determines whether the button will auto-repeat
+  if the user holds it down.
+  <li>setToggleButton() determines whether the button is a toggle
+  button or not.
+  </ul>
 
-  Another convention regarding QButtons is \link setAutoResize()
-  auto-resizing\endlink.  Enabling auto-resizing makes a button resize
-  itself whenever the contents change.
+  The difference between isDown() and isOn() is as follows:
+  When the user clicks a toggle button to toggle it on, the button is
+  first \e pressed, then released into \e on state.  When the user
+  clicks it again (to toggle it off) the button moves first to the \e
+  pressed state, then to the \e off state (isOn() and isDown() are
+  both FALSE).
+
+  Default buttons (as used in many dialogs) are provided by
+  QPushButton::setDefault() and QPushButton::setAutoDefault().
+
+  QButton provides four signals:
+  <ul>
+  <li>pressed() is emitted when the left mouse button is pressed while
+  the mouse cursor is inside the button.
+  <li>released() is emitted when the left mouse button is released.
+  <li>clicked() is emitted when the button is first pressed then
+  released, or when the accelerator key is typed, or when animateClick()
+  is called.
+  <li>toggled() is emitted when the state of a toggle button changes.
+  </ul>
+
+  If the button is a text button with "&" in its text, QButton creates an
+  automatic accelerator key.  This code creates a push button labelled "Rock
+  & Roll" (where the c is underscored).  The button gets an automatic
+  accelerator key, Alt-C:
+
+  \code
+    QPushButton *p = new QPushButton( "Ro&ck && Roll", this );
+  \endcode
+
+  In this example, when the user presses Alt-C the button will
+  \link animateClick() animate a click\endlink.
+
+  You can also set a custom accelerator using the setAccel() function.
+  This is very useful for pixmap buttons since they have no automatic
+  accelerator.
+
+  \code
+    QPushButton *p;
+    p->setPixmap( QPixmap("print.gif") );
+    p->setAccel( ALT+Key_F7 );
+  \endcode
+
+  All of the buttons provided by Qt (\l QPushButton, \l QCheckBox and \l
+  QRadioButton) support both text and pixmap labels.
+
+  To subclass QButton, you have to reimplement at least drawButton()
+  (to draw the button's outskirts) and drawButtonLabel() (to draw its
+  text or pixmap).  It is generally advisable to reimplement
+  sizeHint() as well, and sometimes hitButton() (to determine whether
+  a button press is within the button).
+
+  To reduce flickering the QButton::paintEvent() sets up a pixmap that the
+  drawButton() function draws in. You should not reimplement paintEvent()
+  for a subclass of QButton unless you want to take over all drawing.
 
   \sa QButtonGroup
 */
@@ -82,13 +218,12 @@ QButton::QButton( QWidget *parent, const char *name )
     buttonOn   = FALSE;				// button is off
     mlbDown    = FALSE;				// mouse left button up
     autoresize = FALSE;				// not auto resizing
-    isTiming   = FALSE;				// not in keyboard mode
+    animation  = FALSE;				// no pending animateClick
     repeat     = FALSE;				// not in autorepeat mode
+    d	       = 0;
     if ( parent && parent->inherits("QButtonGroup") ) {
-	group = (QButtonGroup*)parent;
-	group->insert( this );			// insert into button group
-    } else {
-	group = 0;
+	setGroup((QButtonGroup*)parent);
+	group()->insert( this );		// insert into button group
     }
     setFocusPolicy( TabFocus );
 }
@@ -99,9 +234,10 @@ QButton::QButton( QWidget *parent, const char *name )
 
 QButton::~QButton()
 {
-    if ( group )				// remove from button group
-	group->remove( this );
+    if ( group() )				// remove from button group
+	group()->remove( this );
     delete bpixmap;
+    delete d;
 }
 
 
@@ -158,6 +294,8 @@ void QButton::setText( const char *text )
 {
     if ( btext == text )
 	return;
+    int oldAccelChar = shortcutChar(btext);
+    int newAccelChar = shortcutChar(text);
     btext = text;
     if ( bpixmap ) {
 	delete bpixmap;
@@ -165,8 +303,11 @@ void QButton::setText( const char *text )
     }
     if ( autoresize )
 	adjustSize();
-    else
-	update();
+    if ( oldAccelChar && !newAccelChar && !accel() )
+	setAccel( 0 );
+    if ( newAccelChar )
+	setAccel( ALT+toupper(newAccelChar) );
+    repaint( FALSE );
 }
 
 
@@ -179,13 +320,15 @@ void QButton::setText( const char *text )
   Sets the button pixmap to \a pixmap and redraws the contents.
 
   If \a pixmap is monochrome (i.e. it is a QBitmap or its \link
-  QPixmap::depth() depth\endlink is 1) and it does not have a mask, this
-  function sets the pixmap to be its own mask. The purpose is to draw
-  transparent bitmaps, which is important for e.g. toggle buttons.
+  QPixmap::depth() depth\endlink is 1) and it does not have a mask,
+  this function sets the pixmap to be its own mask. The purpose of
+  this is to draw transparent bitmaps, which is important for
+  e.g. toggle buttons.
 
-  The button resizes itself if auto-resizing is enabled.
+  The button resizes itself if auto-resizing is enabled, and always
+  disables any accelerator.
 
-  \sa pixmap(), setText(), setAutoResize(), QPixmap::mask()
+  \sa pixmap(), setText(), setAutoResize(), setAccel(), QPixmap::mask()
 */
 
 void QButton::setPixmap( const QPixmap &pixmap )
@@ -203,12 +346,70 @@ void QButton::setPixmap( const QPixmap &pixmap )
     *bpixmap = pixmap;
     if ( bpixmap->depth() == 1 && !bpixmap->mask() )
 	bpixmap->setMask( *((QBitmap *)bpixmap) );
+    int oldAccelChar = shortcutChar(btext);
     if ( !btext.isNull() )
 	btext.resize( 0 );
     if ( autoresize && !sameSize )
 	adjustSize();
-    else
-	repaint( FALSE );
+    if ( oldAccelChar )
+	setAccel( 0 );
+    repaint( FALSE );
+}
+
+
+/*!
+  Returns the accelerator key currently set for the button, or 0
+  if no accelerator key has been set.
+  \sa setAccel()
+*/
+
+int QButton::accel() const
+{
+    QAccel *a = CHILD((QObject*)this,QAccel,"buttonAccel");
+    return a ? a->key(0) : 0;
+}
+
+/*!
+  Specifies an accelerator \a key for the button, or removes the
+  accelerator if \a key is 0.
+
+  Setting a button text containing a shortcut character (for
+  example the 'x' in E&xit) automatically defines an ALT+letter
+  accelerator for the button.
+  You only need to call this function in order to specify a custom
+  accelerator.
+
+  Example:
+  \code
+    QPushButton *b1 = new QPushButton;
+    b1->setText( "&OK" );		// sets accel ALT+'O'
+
+    QPushButton *b2 = new QPushButton;
+    b2->setPixmap( printIcon );		// pixmap instead of text
+    b2->setAccel( CTRL+'P' );		// custom accel
+  \endcode
+
+  \sa accel(), setText(), QAccel
+*/
+
+void QButton::setAccel( int key )
+{
+    if ( key == -1 ) {				// guess from the text
+	int c = shortcutChar( text() );
+	key = c ? ALT+toupper(c) : 0;
+    }
+
+    QAccel *a = CHILD(this,QAccel,"buttonAccel");
+    if ( key == 0 ) {				// delete accel
+	delete a;
+    } else {
+	if ( a )
+	    a->clear();
+	else
+	    a = new QAccel( this, "buttonAccel" );
+	a->connectItem( a->insertItem(key,0),
+			this, SLOT(animateClick()) );
+    }
 }
 
 
@@ -243,6 +444,17 @@ void QButton::setAutoResize( bool enable )
 }
 
 
+/*!
+  \fn bool QButton::autoRepeat() const
+
+  Returns TRUE if the button is auto-repeating, else FALSE.
+
+  The default is FALSE.
+
+  \sa setAutoRepeat()
+*/
+
+
 /*!
   Turns on auto-repeat for the button if \a enable is TRUE, or
   turns it off if \a enable is FALSE.
@@ -260,7 +472,36 @@ void QButton::setAutoRepeat( bool enable )
 {
     repeat = (uint)enable;
     if ( repeat && mlbDown )
-	QTimer::singleShot( autoRepeatPeriod, this, SLOT(autoRepeatSlot()) );
+	timer()->start( autoRepeatDelay, TRUE );
+}
+
+
+/*!
+  Performs an animated click: The button is pressed and a short while
+  later released.
+
+  pressed(), released(), clicked() and toggled() signals are emitted as
+  appropriate.
+
+  This function does nothing if the button is \link setEnabled()
+  disabled\endlink or if it is the enabled toggle button in an
+  exclusive button group (this means, in practice, a \link
+  QRadioButton radio button \endlink in a button group).
+
+  \sa setAccel()
+*/
+
+void QButton::animateClick()
+{
+    if ( !isEnabled() || animation )
+	return;
+    if ( isOn() && group() && group()->isExclusive() )
+	return;
+    animation  = TRUE;
+    buttonDown = TRUE;
+    repaint( FALSE );
+    emit pressed();
+    QTimer::singleShot( 100, this, SLOT(animateTimeout()) );
 }
 
 
@@ -286,6 +527,7 @@ void QButton::setAutoRepeat( bool enable )
 
 void QButton::setDown( bool enable )
 {
+    mlbDown = FALSE;				// the safe setting
     if ( (bool)buttonDown != enable ) {
 	buttonDown = enable;
 	repaint( FALSE );
@@ -379,6 +621,21 @@ void QButton::drawButtonLabel( QPainter * )
 }
 
 
+/*!
+  Handles keyboard events for the button.
+
+  Space is the only key that has any effect; it calls animateClick().
+*/
+
+void QButton::keyPressEvent( QKeyEvent *e )
+{
+    if ( e->key() == Key_Space )
+	animateClick();
+    else
+	e->ignore();
+}
+
+
 /*!
   Handles mouse press events for the button.
   \sa mouseReleaseEvent()
@@ -386,7 +643,7 @@ void QButton::drawButtonLabel( QPainter * )
 
 void QButton::mousePressEvent( QMouseEvent *e )
 {
-    if ( e->button() != LeftButton || mlbDown )
+    if ( e->button() != LeftButton )
 	return;
     bool hit = hitButton( e->pos() );
     if ( hit ) {				// mouse press on button
@@ -395,8 +652,7 @@ void QButton::mousePressEvent( QMouseEvent *e )
 	repaint( FALSE );
 	emit pressed();
 	if ( repeat )
-	    QTimer::singleShot( autoRepeatPeriod,
-				this, SLOT(autoRepeatSlot()) );
+	    timer()->start( autoRepeatDelay, TRUE );
     }
 }
 
@@ -407,8 +663,10 @@ void QButton::mousePressEvent( QMouseEvent *e )
 
 void QButton::mouseReleaseEvent( QMouseEvent *e)
 {
-    if ( e->button() != LeftButton || !mlbDown )
+    if ( e->button() != LeftButton )
 	return;
+    if ( d )
+	timer()->stop();
     mlbDown = FALSE;				// left mouse button up
     bool hit = hitButton( e->pos() );
     buttonDown = FALSE;
@@ -451,20 +709,47 @@ void QButton::mouseMoveEvent( QMouseEvent *e )
     }
 }
 
+
 /*!
   Handles paint events for the button.
 
   Opens the painter on the button and calls drawButton().
 */
 
-void QButton::paintEvent( QPaintEvent * )
+void QButton::paintEvent( QPaintEvent *event )
 {
     QPainter paint;
-    paint.begin( this );
-    drawButton( &paint );
-    paint.end();
-}
 
+    if ( event && width() <= drawingPixWidth && height() <= drawingPixHeight ) {
+	QPixmap *pm = getDrawingPixmap();
+	ASSERT( pm );
+
+	pm->fill( this, 0, 0 );
+	paint.begin( pm );
+	paint.setFont( font() );
+	drawButton( &paint );
+	paint.end();
+	
+	bitBlt( this, event->rect().topLeft(), pm, event->rect() );
+    } else {
+	paint.begin( this );
+
+	// This optimization is worth it, since we often call repaint()
+	// to draw exactly the whole button.
+	if ( event && !event->rect().contains(rect()) )
+	    paint.setClipRect( event->rect() );
+
+	if ( event )
+	    erase( event->rect() );
+	else
+	    erase();
+
+	drawButton( &paint );
+	paint.end();
+    }
+
+
+}
 
 /*!
   Handles focus in events for the button.
@@ -488,51 +773,32 @@ void QButton::focusOutEvent( QFocusEvent * )
 
 
 /*!
-  Handles keyboard events for the button.
-
-  Space is the only key that has any effect; it emulates a
-  mousePressEvent() followed by a mouseReleaseEvent().
+  Internal slot used for auto repeat.
 */
 
-void QButton::keyPressEvent( QKeyEvent *e ) 
+void QButton::autoRepeatTimeout()
 {
-    if ( e->key() == Key_Space ) {
-	QTimer *timer;
-	if ( isTiming ) {
-	    timer = CHILD(this,QTimer,"timer");
-	    if ( timer ) {
-		timer->stop();
-		timer->start( autoRepeatPeriod, TRUE );
-	    }
-	    return;
-	} else {
-	    isTiming = TRUE;
-	    timer = new QTimer( this, "timer" );
-	    CHECK_PTR( timer );
-	    connect( timer, SIGNAL(timeout()), SLOT(timerSlot()) );
-	    timer->start( autoRepeatPeriod, TRUE );
+    if ( mlbDown && isEnabled() && autoRepeat() ) {
+	if ( buttonDown ) {
+	    emit released();
+	    emit clicked();
+	    emit pressed();
 	}
-	buttonDown = TRUE;
-	repaint( FALSE );
-	emit pressed();
-    } else {
-	e->ignore();
+	timer()->start( autoRepeatPeriod, TRUE );
     }
 }
 
 
 /*!
-  Acts like a mouseReleaseEvent during keyboard interaction.
+  Internal slot used for the second stage of animateClick().
 */
 
-void QButton::timerSlot()
+void QButton::animateTimeout()
 {
-    if ( !isTiming )
+    if ( !animation )
 	return;
-    isTiming   = FALSE;
+    animation  = FALSE;
     buttonDown = FALSE;
-    QTimer *timer = CHILD(this,QTimer,"timer");
-    delete timer;
     if ( toggleBt )
 	buttonOn = !buttonOn;
     repaint( FALSE );
@@ -544,13 +810,12 @@ void QButton::timerSlot()
 
 
 /*!
-  Emits a clicked() signal if autoRepeat() is on.
+  Reimplemented for implementational reasons.
 */
 
-void QButton::autoRepeatSlot()
+void QButton::enabledChange( bool e )
 {
-    if ( !mlbDown )
-	return;
-    QTimer::singleShot( autoRepeatPeriod, this, SLOT(autoRepeatSlot()) );
-    emit clicked();
+    if ( !e )
+	setDown( FALSE );
+    QWidget::enabledChange( e );
 }
