@@ -1,5 +1,5 @@
 /**********************************************************************
-** $Id: qcombo.cpp,v 2.26 1997/03/26 22:21:54 hanord Exp $
+** $Id: qcombo.cpp,v 2.64 1997/09/26 11:25:25 eiriken Exp $
 **
 ** Implementation of QComboBox widget class
 **
@@ -7,13 +7,16 @@
 **
 ** Copyright (C) 1992-1997 Troll Tech AS.  All rights reserved.
 **
-** This file is part of the non-commercial distribution of Qt 1.2.
+** This file is part of the free edition of Qt 1.33.
 **
 ** See the file LICENSE included in the distribution for the usage
 ** and distribution terms, or http://www.troll.no/qt/license.html.
 **
-** (This file differs from the one in the commercial distribution of
-** Qt only by this comment.)
+** IMPORTANT NOTE: You may NOT copy this file or any part of it into
+** your own programs or libraries.
+**
+** (This file differs from the one in the commercial edition of Qt
+** only by this comment.)
 **
 *****************************************************************************/
 
@@ -31,7 +34,7 @@
 #include "qlined.h"
 #include <limits.h>
 
-RCSTAG("$Id: qcombo.cpp,v 2.26 1997/03/26 22:21:54 hanord Exp $");
+RCSTAG("$Id: qcombo.cpp,v 2.64 1997/09/26 11:25:25 eiriken Exp $");
 
 
 /*!
@@ -40,6 +43,8 @@ RCSTAG("$Id: qcombo.cpp,v 2.26 1997/03/26 22:21:54 hanord Exp $");
 
   \ingroup realwidgets
 
+  \define QComboBox::Policy
+
   A combo box may be defined as a selection widget which displays the
   current selection, and which can pop up a list of possible
   selections.  Some combo boxes also allow the user to select
@@ -89,8 +94,6 @@ RCSTAG("$Id: qcombo.cpp,v 2.26 1997/03/26 22:21:54 hanord Exp $");
   argument and one with an \c int argument.  If the user highlights or
   activates a pixmap, only the \c int signals are emitted.
 
-  QComboBox has no public slots.
-
   Read-write combo boxes offer four policies for dealing with typed
   input: <ul> <li> \c NoInsertion means to simply emit the activated()
   signal, <li> \c AtBottom means to insert the string at the bottom of
@@ -103,10 +106,16 @@ RCSTAG("$Id: qcombo.cpp,v 2.26 1997/03/26 22:21:54 hanord Exp $");
   default insertion policy is \c AtBottom, you can change it using
   setInsertionPolicy().
 
+  It is possible to constrain the input to an editable combo box using
+  QValidator; see setValidator().  By default, all input is accepted.
+
   A combo box has a default focusPolicy() of \c TabFocus, i.e. it will
   not grab focus if clicked.  This differs from both Windows and Motif.
 
-
+  <img src=qcombo1-m.gif>(Motif 1, read-only)<br clear=all>
+  <img src=qcombo2-m.gif>(Motif 2, read-write)<br clear=all>
+  <img src=qcombo3-m.gif>(Motif 2, read-only)<br clear=all>
+  <img src=qcombo1-w.gif>(Windows style)
 */
 
 
@@ -244,7 +253,7 @@ QComboBox::QComboBox( QWidget *parent, const char *name )
     d->current               = 0;
     d->maxCount              = INT_MAX;
     d->sizeLimit	     = 10;
-    d->p = AtBottom;
+    d->p                    =  AtBottom;
     d->autoresize            = FALSE;
     d->poppedUp              = FALSE;
     d->arrowDown             = FALSE;
@@ -290,18 +299,21 @@ QComboBox::QComboBox( bool rw, QWidget *parent, const char *name )
     d->discardNextMousePress = FALSE;
     d->shortClick = FALSE;
 
-    if ( style() == MotifStyle )
-	setFocusPolicy( TabFocus );
+    setFocusPolicy( StrongFocus );
 
     if ( rw ) {
 	d->ed = new QLineEdit( this, "this is not /bin/ed" );
-	if ( style() == WindowsStyle )
+	if ( style() == WindowsStyle ) {
 	    d->ed->setGeometry( 2, 2, width() - 2 - 2 - 16, height() - 2 - 2 );
-	else
+	    d->ed->setFrame( FALSE );
+	} else {
 	    d->ed->setGeometry( 3, 3, width() - 3 - 3 - 21, height() - 3 - 3 );
+	}
 	d->ed->installEventFilter( this );
-	d->ed->setFrame( FALSE );
-    
+	setFocusProxy( d->ed );
+
+	setBackgroundMode( NoBackground );
+
 	connect( d->ed, SIGNAL(returnPressed()), SLOT(returnPressed()) );
     } else {
 	d->ed = 0;
@@ -331,6 +343,55 @@ QComboBox::~QComboBox()
 }
 
 
+
+/*!  Reimplemented for implementational reasons.
+
+  Note that QComboBox always turns into a new-style Motif combo box
+  when it is changed from Windows to Motif style (even if it was an
+  old-style combo box before).
+*/
+
+void QComboBox::setStyle( GUIStyle s )
+{
+    if ( s != style() ) {
+	QWidget::setStyle( s );
+	if ( !d->usingListBox ) {
+	    QPopupMenu * p = d->popup;
+	    d->listBox = new QListBox( 0, 0, WType_Popup );
+	    d->listBox->setAutoScrollBar( FALSE );
+	    d->listBox->setBottomScrollBar( FALSE );
+	    d->listBox->setAutoBottomScrollBar( FALSE );
+	    d->listBox->setFrameStyle( QFrame::Box | QFrame::Plain );
+	    d->listBox->setLineWidth( 1 );
+	    d->listBox->resize( 100, 10 );
+	    d->usingListBox      = TRUE;
+	    connect( d->listBox, SIGNAL(selected(int)),
+		     SLOT(internalActivate(int)) );
+	    connect( d->listBox, SIGNAL(highlighted(int)),
+		     SLOT(internalHighlight(int)));
+	    if ( p ) {
+		int n;
+		for( n=p->count()-1; n>=0; n-- ) {
+		    if ( p->text( n ) ) 
+			d->listBox->insertItem( p->text( n ), 0 );
+		    else if ( p->pixmap( n ) )
+			d->listBox->insertItem( *(p->pixmap( n )), 0 );
+		}
+		delete p;
+	    }
+	}
+
+    }
+    if ( d->ed ) {
+	d->ed->setStyle( s );
+	d->ed->setFrame( s == MotifStyle );
+    }
+    if ( d->listBox )
+	d->listBox->setStyle( s );
+}
+
+
+
 /*!
   Returns the number of items in the combo box.
 */
@@ -360,25 +421,22 @@ void QComboBox::insertStrList( const QStrList *list, int index )
     const char *tmp;
     if ( index < 0 )
 	index = count();
-    bool updcur = d->current == 0 && index == 0;
     while ( (tmp=it.current()) ) {
 	++it;
 	if ( d->usingListBox )
-	    d->listBox->insertItem( tmp, index++ );
+	    d->listBox->insertItem( tmp, index );
 	else
-	    d->popup->insertItem( tmp, index++ );
+	    d->popup->insertItem( tmp, index );
+	if ( index++ == d->current ) {
+	    if ( d->ed )
+		d->ed->setText( text( d->current ) );
+	    else
+		repaint();
+	    currentChanged();
+	}
     }
     if ( index != count() )
 	reIndex();
-    if ( index <= d->current ) {
-	if ( d->ed )
-	    d->ed->setText( text( d->current ) );
-	else
-	    repaint();
-    }
-	
-    if ( updcur )
-	currentChanged();
 }
 
 /*!
@@ -405,25 +463,23 @@ void QComboBox::insertStrList( const char **strings, int numStrings, int index)
     }
     if ( index < 0 )
 	index = count();
-    bool updcur = d->current == 0 && index == 0;
     int i = 0;
     while ( (numStrings<0 && strings[i]!=0) || i<numStrings ) {
 	if ( d->usingListBox )
-	    d->listBox->insertItem( strings[i], index++ );
+	    d->listBox->insertItem( strings[i], index );
 	else
-	    d->popup->insertItem( strings[i], index++ );
+	    d->popup->insertItem( strings[i], index );
 	i++;
+	if ( index++ == d->current ) {
+	    if ( d->ed )
+		d->ed->setText( text( d->current ) );
+	    else
+		repaint();
+	    currentChanged();
+	}
     }
     if ( index != count() )
 	reIndex();
-    if ( index <= d->current ) {
-	if ( d->ed )
-	    d->ed->setText( text( d->current ) );
-	else
-	    repaint();
-    }
-    if ( updcur )
-	currentChanged();
 }
 
 
@@ -627,6 +683,14 @@ void QComboBox::setCurrentItem( int index )
     d->current = index;
     if ( d->ed )
 	d->ed->setText( text( index ) );
+    if ( d->poppedUp ) {
+	if ( d->usingListBox && d->listBox )
+	    d->listBox->setCurrentItem( index );
+	else if ( d->popup )
+	    // the popup will soon send an override, but for the
+	    // moment this is correct
+	    internalHighlight( index );
+    }
     currentChanged();
 }
 
@@ -669,19 +733,19 @@ void QComboBox::setAutoResize( bool enable )
 */
 QSize QComboBox::sizeHint() const
 {
-    int dist, buttonH, buttonW;
-    getMetrics( &dist, &buttonW, &buttonH );
-
     int i, w, h;
-    int maxW = 0;
-    int maxH = 0;
     const char *tmp;
-    for( i = 0 ; i < count() ; i++ ) {
+    QFontMetrics fm = fontMetrics();
+
+    int extraW = 20;
+    int maxW = 0;
+    int maxH = QMAX( fm.height(), style()==WindowsStyle ? 16 : 18 );
+    
+    for( i = 0; i < count(); i++ ) {
 	tmp = text( i );
 	if ( tmp ) {
-	    QFontMetrics fm = fontMetrics();
 	    w = fm.width( tmp );
-	    h = fm.lineSpacing() + 1;
+	    h = 0;
 	} else {
 	    const QPixmap *pix = pixmap( i );
 	    if ( pix ) {
@@ -697,7 +761,7 @@ QSize QComboBox::sizeHint() const
 	if ( h > maxH )
 	    maxH = h;
     }
-    return QSize( 4 + 4 + maxW + 2*dist + buttonW, maxH + 5 + 5 );
+    return QSize( 4 + 4 + maxW + extraW, maxH + 5 + 5 );
 }
 
 
@@ -719,10 +783,8 @@ void QComboBox::internalActivate( int index )
 	d->popup->removeEventFilter( this );
     d->poppedUp = FALSE;
 
-    const char *t = text( index );
+    QString t( text( index ) );
     emit activated( index );
-    if ( !t )
-	return;					// shouldn't happen
     if ( d->ed )
 	d->ed->setText( t );
     emit activated( t );
@@ -763,9 +825,7 @@ void QComboBox::internalClickTimeout()
 void QComboBox::setBackgroundColor( const QColor &color )
 {
     QWidget::setBackgroundColor( color );
-    if ( d->usingListBox )
-	d->listBox->setBackgroundColor( color );
-    else
+    if ( !d->usingListBox )
 	d->popup->setBackgroundColor( color );
 }
 
@@ -799,6 +859,8 @@ void QComboBox::setFont( const QFont &font )
 	d->listBox->setFont( font );
     else
 	d->popup->setFont( font );
+    if (d->ed)
+	d->ed->setFont( font );
     if ( d->autoresize )
 	adjustSize();
 }
@@ -823,27 +885,31 @@ void QComboBox::resizeEvent( QResizeEvent * )
   Handles paint events for the combo box.
 */
 
-void QComboBox::paintEvent( QPaintEvent * )
+void QComboBox::paintEvent( QPaintEvent *event )
 {
-    QPainter p;
+    QPainter p( this );
+    if ( event )
+	p.setClipRect( event->rect() );
     QColorGroup g  = colorGroup();
 
-    p.begin( this );
-
     if ( width() < 5 || height() < 5 ) {
-	qDrawShadePanel( &p, rect(), g, FALSE, 2 );
-        p.end();
+	QBrush fill( g.background() );
+	qDrawShadePanel( &p, rect(), g, FALSE, 2, &fill );
 	return;
     }
 
-    if ( !d->usingListBox ) {		// motif 1.x style
+    bool has_focus = hasFocus()
+	|| (d && d->usingListBox && d->ed && d->ed->hasFocus());
+
+    if ( !d->usingListBox ) {			// motif 1.x style
 	int dist, buttonH, buttonW;
+	QBrush fill( g.background() );
 
 	getMetrics( &dist, &buttonW, &buttonH );
 	int xPos = width() - dist - buttonW - 1;
+	qDrawShadePanel( &p, rect(), g, FALSE, 2, &fill );
 	qDrawShadePanel( &p, xPos, (height() - buttonH)/2,
 			 buttonW, buttonH, g, FALSE, 2 );
-	QFontMetrics fm = p.fontMetrics();
 	QRect clip( 4, 2, xPos - 2 - 4, height() - 4 );
 	const char *str = d->popup->text( d->current );
 	if ( str ) {
@@ -856,62 +922,77 @@ void QComboBox::paintEvent( QPaintEvent * )
 		p.setClipping( FALSE );
 	    }
 	}
-	qDrawShadePanel( &p, rect(), g, FALSE, 2 );
 
-	if ( hasFocus() )
+	if ( has_focus )
 	    p.drawRect( xPos - 5, 4, width() - xPos + 1 , height() - 8 );
 
-    } else if ( style() == MotifStyle ) { // motif 2.0 style
-	int x1, y1;
-	QPointArray l;
+    } else if ( style() == MotifStyle ) {	// motif 2.0 style
+	int awh, ax, ay, sh, sy;
+	QBrush fill( g.background() );
 
-	x1 = width() - 2 - 5 - 15;
+	if ( height() < 6 ) {
+	    awh = height();
+	    ay = 0;
+	} else if ( height() < 18 ) {
+	    awh = height() - 6;
+	    ay = 0;
+	} else {
+	    awh = height()*4/10;
+	    ay = awh/2;
+	}
 
-	y1 = height()/2 - 9;
+	sh = (awh+3)/4;
+	sy = height() - ay - sh;
+	if ( sh < 3 ) {
+	    sy = sy+sh-3;
+	    sh = 3;
+	}
+	if ( sy - ay - awh > 3 ) {
+	    sy -= ( sy-ay-awh-3 )/2;
+	    ay += ( sy-ay-awh-3 )/2;
+	}
+	awh = awh;
 
-	// light matter
-	l.setPoints( 20,
-		     x1,y1+17, x1,y1+17-2, 
-		     x1,y1+17-2, x1+15, y1+17-2,
-		     x1+1,y1, x1+14,y1,
-		     x1+1,y1+1, x1+12,y1+1,
-		     x1+2,y1+2, x1+2,y1+3,
-		     x1+3,y1+2, x1+3,y1+5,
-		     x1+4,y1+4, x1+4,y1+7,
-		     x1+5,y1+6, x1+5,y1+9,
-		     x1+6,y1+8, x1+6,y1+11,
-		     x1+7,y1+10, x1+7,y1+11 );
-	p.setPen( colorGroup().light() );
-	p.drawLineSegments( l );
+	if ( d->ed )
+	    ax = d->ed->geometry().right() + 4;
+	else
+	    ax = width() - 3 - 21;
 
-	// dark matter
-	l.setPoints( 18,
-		     x1+8,y1+10, x1+8,y1+11,
-		     x1+9,y1+8, x1+9,y1+11,
-		     x1+10,y1+6, x1+10,y1+9,
-		     x1+11,y1+4, x1+11,y1+7,
-		     x1+12,y1+2, x1+12,y1+5,
-		     x1+13,y1+1, x1+13,y1+3,
-		     x1+14,y1+1, x1+14,y1+1,
-		     x1+1,y1+17, x1+15,y1+17,
-		     x1+15,y1+16, x1+15,y1+16 );
-	p.setPen( colorGroup().dark() );
-	p.drawLineSegments( l );
+	if ( ax + awh + 2 < width() )
+	    ax += ( width() - 2 - ax - awh ) / 2;
 
-	QFontMetrics fm = p.fontMetrics();
-	QRect clip( 4, 2, x1 - 2 - 4, height() - 4 );
+	qDrawShadePanel( &p, rect(), g, FALSE, 2, &fill );
+
+	qDrawArrow( &p, DownArrow, MotifStyle, FALSE,
+		    ax, ay, awh, awh, g );
+
+	p.setPen( g.light() );
+	p.drawLine( ax, sy, ax+awh-1, sy );
+	p.drawLine( ax, sy, ax, sy+sh-1 );
+	p.setPen( g.dark() );
+	p.drawLine( ax+1, sy+sh-1, ax+awh-1, sy+sh-1 );
+	p.drawLine( ax+awh-1, sy+1, ax+awh-1, sy+sh-1 );
+
+	QRect clip( 3, 3, width() - 3 - 3 - 21, height() - 3 - 3 );
 	const char *str = d->listBox->text( d->current );
-	p.setPen( colorGroup().foreground() );
-	p.drawText( clip, AlignCenter | SingleLine, str );
-	qDrawShadePanel( &p, rect(), g, FALSE, 2 );
+	if ( str ) {
+	    p.setPen( g.foreground() );
+	    p.drawText( clip, AlignCenter | SingleLine, str );
+	} else {
+	    const QPixmap *pix = d->listBox->pixmap( d->current );
+	    if ( pix ) {
+		p.setClipRect( clip );
+		p.drawPixmap( 4, (height()-pix->height())/2, *pix );
+		p.setClipping( FALSE );
+	    }
+	}
 
-	if ( hasFocus() )
-	    p.drawRect( x1 - 2, y1 - 2, 20, 22 );
+	if ( has_focus )
+	    p.drawRect( ax - 2, ay - 2, awh+4, sy+sh+4-ay );
 
-    } else {				// windows 95 style
-	QColor	  bg  = isEnabled() ? g.base() : g.background();
-	QFontMetrics  fm  = fontMetrics();
-	const char   *str = d->listBox->text( d->current );
+    } else {					// windows 95 style
+	QColor bg = isEnabled() ? g.base() : g.background();
+	const char *str = d->listBox->text( d->current );
 
 	QBrush fill( bg );
 	qDrawWinPanel( &p, 0, 0, width(), height(), g, TRUE, &fill );
@@ -922,23 +1003,37 @@ void QComboBox::paintEvent( QPaintEvent * )
 		    arrowR.x() + 2, arrowR.y() + 2,
 		    arrowR.width() - 4, arrowR.height() - 4, g );
 
-	QRect clipR( 5, 4, width()  - 5 - 4 - arrowR.width(), 
+	QRect textR( 5, 4, width()  - 5 - 4 - arrowR.width(), 
 		     height() - 4 - 4 );
-	p.setClipRect( clipR );
-	if ( str ) {	
-	    QFontMetrics fm = fontMetrics();
-	    p.drawText( clipR, AlignLeft | AlignVCenter | SingleLine, str);
+
+	if ( has_focus ) {
+	    QBrush fill( QApplication::winStyleHighlightColor() );
+	    p.fillRect( textR.x()-1, textR.y(),
+		textR.width(), textR.height(), fill );
+	    p.drawWinFocusRect( textR.x()-2, textR.y()-1,
+		textR.width()+2, textR.height()+2, backgroundColor() );
+	}
+
+	p.setClipRect( textR );
+
+	if ( str ) {
+	    p.setPen(
+		has_focus
+		? white
+		: g.text()
+	    );
+	    p.drawText( textR, AlignLeft | AlignVCenter | SingleLine, str);
 	} else {
 	    const QPixmap *pix = d->listBox->pixmap( d->current );
-	    if ( pix ) {
+	    if ( pix )
 		p.drawPixmap( 4, (height()-pix->height())/2, *pix );
-	    }
 	}
 	p.setClipping( FALSE );
     }
     p.end();
 }
 
+
 /*!
   \internal
   Returns the button arrow rectangle for windows style combo boxes.
@@ -948,6 +1043,7 @@ QRect QComboBox::arrowRect() const
     return QRect( width() - 2 - 16, 2, 16, height() - 4 );
 }
 
+
 /*!
   Handles mouse press events for the combo box.
 */
@@ -967,21 +1063,11 @@ void QComboBox::mousePressEvent( QMouseEvent *e )
 	    repaint( FALSE );
 	}
     } else if ( d->usingListBox ) {
-	if ( e->pos().x() >= width() - 23 ) {
-	    popup();
-	    QTimer::singleShot( 200, this, SLOT(internalClickTimeout()));
-	    d->shortClick = TRUE;
-	}
+	popup();
+	QTimer::singleShot( 200, this, SLOT(internalClickTimeout()));
+	d->shortClick = TRUE;
     } else {
 	popup();
-	QMouseEvent me1( Event_MouseButtonPress,
-			 d->popup->mapFromGlobal(mapToGlobal( QPoint(0,0) ) ),
-			 e->button(), e->state() );
-	QApplication::sendEvent( d->popup, &me1 );
-	QMouseEvent me2( Event_MouseMove,
-			 d->popup->mapFromGlobal(mapToGlobal( e->pos() ) ),
-			 e->button(), e->state() );
-	QApplication::sendEvent( d->popup, &me2 );
 	QTimer::singleShot( 200, this, SLOT(internalClickTimeout()));
 	d->shortClick = TRUE;
     }
@@ -1024,58 +1110,53 @@ void QComboBox::mouseDoubleClickEvent( QMouseEvent *e )
 void QComboBox::keyPressEvent( QKeyEvent *e )
 {
     int c;
-    switch ( e->key() ) {
-    case Key_Return:
-    case Key_Enter:
-	if ( style() == MotifStyle ) {
-	    e->ignore();
-	    break;
-	}
-	// fall through for motif
-	e->accept();
-	popup();
-	break;
-    case Key_Space:
-	if ( d->usingListBox ) {
-	    e->accept();
-	    popup();
-	}
-	break;
-    case Key_Left:
-	if ( style() != WindowsStyle ) {
-	    e->ignore();
-	    break;
-	}
-	// fall through for windows
-    case Key_Up:
+
+    if ( d->usingListBox &&
+	 (e->key() == Key_Up ||
+	  (style() == MotifStyle && e->key() == Key_Left && !d->ed)) ) {
 	c = currentItem();
 	if ( c > 0 )
 	    setCurrentItem( c-1 );
 	else
 	    setCurrentItem( count()-1 );
 	e->accept();
-	break;
-    case Key_Right:
-	if ( style() == MotifStyle ) {
-	    e->ignore();
-	    break;
-	}
-	// fall through for windows
-    case Key_Down:
-c	 = currentItem();
+    } else if ( d->usingListBox &&
+		(e->key() == Key_Down ||
+		 (style() == MotifStyle && e->key() == Key_Right && !d->ed))){
+	c = currentItem();
 	if ( ++c < count() )
 	    setCurrentItem( c );
 	else
 	    setCurrentItem( 0 );
 	e->accept();
-	break;
-    default:
+    } else if ( style() == MotifStyle &&
+		!d->ed && e->key() == Key_Space ) {
+	e->accept();
+	d->popup->setActiveItem( d->current );
+	popup();
+	return;
+    } else {
 	e->ignore();
-	break;
+	return;
     }
+
+    c = currentItem();
+    emit highlighted( c );
+    if ( text( c ) )
+	emit activated( text( c ) );
+    emit activated( c );
 }
 
 
+/*!
+  Handles focus in events.
+*/
+
+void QComboBox::focusInEvent( QFocusEvent * )
+{
+    repaint( FALSE );
+}
+
 /*!
   \internal
    Calculates the listbox height needed to contain all items, or as
@@ -1191,28 +1272,53 @@ bool QComboBox::eventFilter( QObject *object, QEvent *event )
 {
     if ( !event )
 	return TRUE;
-    else if ( object == (QObject *)(d->ed) ) {
-	if ( event->type() == Event_KeyPress &&
-		    style() == MotifStyle ) {
+    else if ( object == d->ed ) {
+	if ( event->type() == Event_KeyPress ) {
+	    QKeyEvent * e = (QKeyEvent *) event;
 	    int c;
-	    switch( ((QKeyEvent *)event)->key() ) {
-	    case Key_Up:
+	    if ( d->usingListBox &&
+		 (e->key() == Key_Up ||
+		  (style() == MotifStyle && e->key() == Key_Left && !d->ed)) ){
 		c = currentItem();
 		if ( c > 0 )
 		    setCurrentItem( c-1 );
 		else
 		    setCurrentItem( count()-1 );
-		return TRUE;
-	    case Key_Down:
+		e->accept();
+	    } else if ( d->usingListBox &&
+			(e->key() == Key_Down ||
+			 (style() == MotifStyle && e->key() == Key_Right &&
+			  !d->ed)) ) {
 		c = currentItem();
 		if ( ++c < count() )
 		    setCurrentItem( c );
 		else
 		    setCurrentItem( 0 );
-		return TRUE;
-	    default:
-		break;
+		e->accept();
+	    } else if ( style() == MotifStyle &&
+			!d->usingListBox &&
+			e->key() == Key_Space ) {
+		e->accept();
+		popup();
+		return FALSE;
+	    } else {
+		return FALSE;
 	    }
+
+	    c = currentItem();
+	    emit highlighted( c );
+	    if ( text( c ) )
+		emit activated( text( c ) );
+	    emit activated( c );
+	    return TRUE;
+	} else if ( (event->type() == Event_FocusIn ||
+		     event->type() == Event_FocusOut ) ) {
+	    // to get the focus indication right
+	    update();
+	} else if ( event->type() == Event_MouseButtonPress &&
+		    !hasFocus() && !d->ed->hasFocus() &&
+		    (focusPolicy() & ClickFocus) ) {
+	    setFocus();
 	}
     } else if ( d->usingListBox && object == d->listBox ) {
 	QMouseEvent *e = (QMouseEvent*)event;
@@ -1272,6 +1378,11 @@ bool QComboBox::eventFilter( QObject *object, QEvent *event )
 		return TRUE;
 	    }
 	    break;
+	case Event_KeyPress:
+	    if ( ((QKeyEvent *)event)->key() == Key_Escape ) {
+		popDownListBox();
+		return TRUE;
+	    }
 	default:
 	    break;
 	}
@@ -1391,7 +1502,7 @@ QComboBox::Policy QComboBox::insertionPolicy() const
   \sa policy()
 */
 
-void QComboBox::setInsertionPolicy( QComboBox::Policy policy )
+void QComboBox::setInsertionPolicy( Policy policy )
 {
     d->p = policy;
 }
@@ -1404,39 +1515,37 @@ void QComboBox::setInsertionPolicy( QComboBox::Policy policy )
 
 void QComboBox::returnPressed()
 {
-    const char * s = d->ed->text();
-    if ( s ) {
-	int c = 0;
-	switch ( insertionPolicy() ) {
-	case AtCurrent:
-	    if ( qstrcmp( s, text( currentItem() ) ) )
-		changeItem( s, currentItem() );
-	    emit activated( currentItem() );
-	    emit activated( s );
-	    return;
-	case NoInsertion:
-	    emit activated( s );
-	    return;
-	case AtTop:
-	    c = 0;
-	    break;
-	case AtBottom:
-	    c = count() - 1;
-	    break;
-	case BeforeCurrent:
-	    c = currentItem();
-	    break;
-	case AfterCurrent:
-	    c = currentItem() + 1;
-	    break;
-	}
-	if ( count() == d->maxCount )
-	    removeItem( count() - 1 );
-	insertItem( s, c );
-	setCurrentItem( c );
-	emit activated( c );
+    QString s( d->ed->text() );
+    int c = 0;
+    switch ( insertionPolicy() ) {
+    case AtCurrent:
+	if ( qstrcmp( s, text( currentItem() ) ) )
+	    changeItem( s, currentItem() );
+	emit activated( currentItem() );
 	emit activated( s );
+	return;
+    case NoInsertion:
+	emit activated( s );
+	return;
+    case AtTop:
+	c = 0;
+	break;
+    case AtBottom:
+	c = count();
+	break;
+    case BeforeCurrent:
+	c = currentItem();
+	break;
+    case AfterCurrent:
+	c = currentItem() + 1;
+	break;
     }
+    if ( count() == d->maxCount )
+	removeItem( count() - 1 );
+    insertItem( s, c );
+    setCurrentItem( c );
+    emit activated( c );
+    emit activated( s );
 }
 
 
@@ -1449,3 +1558,39 @@ void QComboBox::setEnabled( bool enable )
 	d->ed->setEnabled( enable );
     QWidget::setEnabled( enable );
 }
+
+
+
+/*!  Sets this combo box to be editable only as allowed by \a v.
+
+  This function does nothing if the combo is not editable.
+
+  \sa validator() clearValidator() QValidator
+*/
+
+void QComboBox::setValidator( QValidator * v )
+{
+    if ( d && d->ed )
+	d->ed->setValidator( v );
+}
+
+
+/*!  Returns the validator which constrains editing for this combo
+  box if there is any, or else 0.
+
+  \sa setValidator() clearValidator() QValidator
+*/
+
+QValidator * QComboBox::validator() const
+{
+    return d && d->ed ? d->ed->validator() : 0;
+}
+
+
+/*!  This slot is equivalent to setValidator( 0 ). */
+
+void QComboBox::clearValidator()
+{
+    if ( d && d->ed )
+	d->ed->setValidator( 0 );
+}
