@@ -1,5 +1,5 @@
 /****************************************************************************
-** $Id: qframe.cpp,v 2.0 1996/09/24 10:16:17 agulbra Exp $
+** $Id: qframe.cpp,v 2.7 1997/09/15 16:29:53 warwick Exp $
 **
 ** Implementation of QFrame widget class
 **
@@ -7,13 +7,16 @@
 **
 ** Copyright (C) 1992-1997 Troll Tech AS.  All rights reserved.
 **
-** This file is part of the non-commercial distribution of Qt 1.2.
+** This file is part of the free edition of Qt 1.33.
 **
 ** See the file LICENSE included in the distribution for the usage
 ** and distribution terms, or http://www.troll.no/qt/license.html.
 **
-** (This file differs from the one in the commercial distribution of
-** Qt only by this comment.)
+** IMPORTANT NOTE: You may NOT copy this file or any part of it into
+** your own programs or libraries.
+**
+** (This file differs from the one in the commercial edition of Qt
+** only by this comment.)
 **
 *****************************************************************************/
 
@@ -22,7 +25,7 @@
 #include "qdrawutl.h"
 #include "qframe.h"
 
-RCSTAG("$Id: qframe.cpp,v 2.0 1996/09/24 10:16:17 agulbra Exp $");
+RCSTAG("$Id: qframe.cpp,v 2.7 1997/09/15 16:29:53 warwick Exp $");
 
 
 /*!
@@ -146,7 +149,8 @@ QFrame::QFrame( QWidget *parent, const char *name, WFlags f,
 
   The frame shapes are:
   <ul>
-  <li> \c NoFrame draws nothing.
+  <li> \c NoFrame draws nothing. Naturally, you should not specify a shadow 
+  style if you use this.
   <li> \c Box draws a rectangular box.	The contents appear to be
   level with the surrounding screen, but the border itself may be
   raised or sunken.
@@ -382,6 +386,18 @@ QRect QFrame::contentsRect() const
     return r;
 }
 
+QSize QFrame::sizeHint() const
+{
+    switch (fstyle & MShape) {
+      case HLine:
+	return QSize(-1,3);
+      case VLine:
+	return QSize(3,-1);
+      default:
+	return QWidget::sizeHint();
+    }
+}
+
 
 /*!
   Handles paint events for the frame.
@@ -392,13 +408,21 @@ QRect QFrame::contentsRect() const
   drawContents().
 */
 
-void QFrame::paintEvent( QPaintEvent * )
+void QFrame::paintEvent( QPaintEvent *event )
 {
-    QPainter paint;
-    paint.begin( this );
-    drawFrame( &paint );
+    QPainter paint( this );
+    if ( event ) {
+	if ( !contentsRect().contains( event->rect() ) ) {
+	    paint.save();
+	    paint.setClipRect( event->rect() );
+	    drawFrame( &paint );
+	    if ( !event->rect().intersects( contentsRect() ) )
+		return;
+	    paint.restore();
+	}
+	paint.setClipRect( event->rect().intersect( contentsRect() ) );
+    }
     drawContents( &paint );
-    paint.end();
 }
 
 
