@@ -1,5 +1,5 @@
 /****************************************************************************
-** $Id: qmenudta.cpp,v 2.3 1997/03/18 10:08:46 hanord Exp $
+** $Id: qmenudta.cpp,v 2.8 1997/09/12 15:32:54 warwick Exp $
 **
 ** Implementation of QMenuData class
 **
@@ -7,13 +7,16 @@
 **
 ** Copyright (C) 1992-1997 Troll Tech AS.  All rights reserved.
 **
-** This file is part of the non-commercial distribution of Qt 1.2.
+** This file is part of the free edition of Qt 1.33.
 **
 ** See the file LICENSE included in the distribution for the usage
 ** and distribution terms, or http://www.troll.no/qt/license.html.
 **
-** (This file differs from the one in the commercial distribution of
-** Qt only by this comment.)
+** IMPORTANT NOTE: You may NOT copy this file or any part of it into
+** your own programs or libraries.
+**
+** (This file differs from the one in the commercial edition of Qt
+** only by this comment.)
 **
 *****************************************************************************/
 
@@ -22,7 +25,7 @@
 #include "qpopmenu.h"
 #include "qapp.h"
 
-RCSTAG("$Id: qmenudta.cpp,v 2.3 1997/03/18 10:08:46 hanord Exp $");
+RCSTAG("$Id: qmenudta.cpp,v 2.8 1997/09/12 15:32:54 warwick Exp $");
 
 
 /*!
@@ -216,23 +219,36 @@ int QMenuData::insertAny( const char *text, const QPixmap *pixmap,
     return mi->ident;
 }
 
-void QMenuData::removePopup( QPopupMenu *popup )
+/*!
+  Internal function that finds the menu item where \a popup is located,
+  storing its index at \a index if \a index is not NULL.
+*/
+QMenuItem *QMenuData::findPopup( QPopupMenu *popup, int *index )
 {
-    int index = 0;
-    register QMenuItem *mi = mitems->first();
+    int i = 0;
+    QMenuItem *mi = mitems->first();
     while ( mi ) {
 	if ( mi->popup_menu == popup )		// found popup
 	    break;
-	index++;
+	i++;
 	mi = mitems->next();
     }
+    if ( index && mi )
+	*index = i;
+    return mi;
+}
+
+void QMenuData::removePopup( QPopupMenu *popup )
+{
+    int index = 0;
+    QMenuItem *mi = findPopup( popup, &index );
     if ( mi )
 	removeItemAt( index );
 }
 
 
 /*!
-  Sets the dirty flag of all menu items to \e dirty.
+  Sets the dirty flag of all menu items to \a dirty.
 */
 
 void QMenuData::setAllDirty( bool dirty )
@@ -246,7 +262,7 @@ void QMenuData::setAllDirty( bool dirty )
 
 
 /*!
-  Inserts a menu item with a text and optional accelerator key, and
+  Inserts a menu item with a text and an optional accelerator key, and
   connects it to an object/slot.
 
   Returns a unique menu item identifier (negative integer \<= -2).
@@ -282,7 +298,7 @@ int QMenuData::insertItem( const char *text,
 }
 
 /*!
-  Inserts a menu item with a pixmap and optional accelerator key, and
+  Inserts a menu item with a pixmap and an optional accelerator key, and
   connects it to an object/slot.
 
   Returns a unique menu item identifier (negative integer \<= -2).
@@ -302,18 +318,40 @@ int QMenuData::insertItem( const QPixmap &pixmap,
     return id;
 }
 
+/*!
+  Inserts a menu item with a pixmap, a text and an optional accelerator
+  key, and connects it to an object/slot. The pixmap will be displayed to
+  the left of the text in the item.
+
+  Returns a unique menu item identifier (negative integer \<= -2).
+
+  \sa removeItem(), changeItem(), setAccel(), connectItem(), QAccel,
+  qkeycode.h
+*/
+
+int QMenuData::insertItem( const QPixmap &pixmap, const char *text,
+			   const QObject *receiver, const char *member,
+			   int accel )
+{
+    int id = insertAny( text, &pixmap, 0, -2, -1 );
+    connectItem( id, receiver, member );
+    if ( accel )
+	setAccel( accel, id );
+    return id;
+}
+
 /*!
   Inserts a menu item with a text.  Returns the menu item identifier.
 
-  The menu item is assigned the identifier \e id or an automatically
-  generated identifier.	 It works as follows: If \e id \>= 0, this
-  identifier is assigned.  If \e id == -1 (default), the identifier is
-  set equal to the menu item's real index (see below).	If \e id is
+  The menu item is assigned the identifier \a id or an automatically
+  generated identifier.	 It works as follows: If \a id \>= 0, this
+  identifier is assigned.  If \a id == -1 (default), the identifier is
+  set equal to the menu item's real index (see below).	If \a id is
   any other negative integer, for instance -2, a unique identifier
   (negative integer \<= -2) is generated.
 
-  The \e index specifies the position in the menu.  The menu item is
-  appended at the end of the list if \e index is negative.
+  The \a index specifies the position in the menu.  The menu item is
+  appended at the end of the list if \a index is negative.
 
   \sa removeItem(), changeItem(), setAccel(), connectItem()
 */
@@ -327,15 +365,20 @@ int QMenuData::insertItem( const char *text, int id, int index )
   Inserts a menu item with a text and a sub menu.
   Returns the menu item identifier.
 
-  The menu item is assigned the identifier \e id or an automatically
-  generated identifier.	 It works as follows: If \e id \>= 0, this
-  identifier is assigned.  If \e id == -1 (default), the identifier is
-  set equal to the menu item's real index (see below).	If \e id is
+  The \a popup must be deleted by the programmer.  It is not deleted when
+  this menu item is removed or when the menu is deleted.  Note that all
+  popups are automatically deleted when the application terminates. (The
+  QApplication destructor destroys all remaining widgets.)
+
+  The menu item is assigned the identifier \a id or an automatically
+  generated identifier.	 It works as follows: If \a id \>= 0, this
+  identifier is assigned.  If \a id == -1 (default), the identifier is
+  set equal to the menu item's real index (see below).	If \a id is
   any other negative integer, for instance -2, a unique identifier
   (negative integer \<= -2) is generated.
 
-  The \e index specifies the position in the menu.  The menu item is
-  appended at the end of the list if \e index is negative.
+  The \a index specifies the position in the menu.  The menu item is
+  appended at the end of the list if \a index is negative.
 
   \sa removeItem(), changeItem(), setAccel(), connectItem()
 */
@@ -349,15 +392,15 @@ int QMenuData::insertItem( const char *text, QPopupMenu *popup,
 /*!
   Inserts a menu item with a pixmap.  Returns the menu item identifier.
 
-  The menu item is assigned the identifier \e id or an automatically
-  generated identifier.	 It works as follows: If \e id \>= 0, this
-  identifier is assigned.  If \e id == -1 (default), the identifier is
-  set equal to the menu item's real index (see below).	If \e id is
+  The menu item is assigned the identifier \a id or an automatically
+  generated identifier.	 It works as follows: If \a id \>= 0, this
+  identifier is assigned.  If \a id == -1 (default), the identifier is
+  set equal to the menu item's real index (see below).	If \a id is
   any other negative integer, for instance -2, a unique identifier
   (negative integer \<= -2) is generated.
 
-  The \e index specifies the position in the menu.  The menu item is
-  appended at the end of the list if \e index is negative.
+  The \a index specifies the position in the menu.  The menu item is
+  appended at the end of the list if \a index is negative.
 
   \sa removeItem(), changeItem(), setAccel(), connectItem()
 */
@@ -371,15 +414,20 @@ int QMenuData::insertItem( const QPixmap &pixmap, int id, int index )
   Inserts a menu item with a pixmap and a sub menu.
   Returns the menu item identifier.
 
-  The menu item is assigned the identifier \e id or an automatically
-  generated identifier.	 It works as follows: If \e id \>= 0, this
-  identifier is assigned.  If \e id == -1 (default), the identifier is
-  set equal to the menu item's real index (see below).	If \e id is
+  The \a popup must be deleted by the programmer.  It is not deleted when
+  this menu item is removed or when the menu is deleted.  Note that all
+  popups are automatically deleted when the application terminates. (The
+  QApplication destructor destroys all remaining widgets.)
+
+  The menu item is assigned the identifier \a id or an automatically
+  generated identifier.	 It works as follows: If \a id \>= 0, this
+  identifier is assigned.  If \a id == -1 (default), the identifier is
+  set equal to the menu item's real index (see below).	If \a id is
   any other negative integer, for instance -2, a unique identifier
   (negative integer \<= -2) is generated.
 
-  The \e index specifies the position in the menu.  The menu item is
-  appended at the end of the list if \e index is negative.
+  The \a index specifies the position in the menu.  The menu item is
+  appended at the end of the list if \a index is negative.
 
   \sa removeItem(), changeItem(), setAccel(), connectItem()
 */
@@ -391,8 +439,59 @@ int QMenuData::insertItem( const QPixmap &pixmap, QPopupMenu *popup,
 }
 
 /*!
-  Inserts a separator at position \e index.
-  The separator becomes the last menu item if \e index is negative.
+  Inserts a menu item with a pixmap and a text.  Returns the menu item
+  identifier.
+
+  The menu item is assigned the identifier \a id or an automatically
+  generated identifier.	 It works as follows: If \a id \>= 0, this
+  identifier is assigned.  If \a id == -1 (default), the identifier is
+  set equal to the menu item's real index (see below).	If \a id is
+  any other negative integer, for instance -2, a unique identifier
+  (negative integer \<= -2) is generated.
+
+  The \a index specifies the position in the menu.  The menu item is
+  appended at the end of the list if \a index is negative.
+
+  \sa removeItem(), changeItem(), setAccel(), connectItem()
+*/
+
+int QMenuData::insertItem( const QPixmap &pixmap, const char *text, 
+			   int id, int index )
+{
+    return insertAny( text, &pixmap, 0, id, index );
+}
+
+/*!
+  Inserts a menu item with a pixmap, a text and a sub menu.
+  Returns the menu item identifier.
+
+  The \a popup must be deleted by the programmer.  It is not deleted when
+  this menu item is removed or when the menu is deleted.  Note that all
+  popups are automatically deleted when the application terminates. (The
+  QApplication destructor destroys all remaining widgets.)
+
+  The menu item is assigned the identifier \a id or an automatically
+  generated identifier.	 It works as follows: If \a id \>= 0, this
+  identifier is assigned.  If \a id == -1 (default), the identifier is
+  set equal to the menu item's real index (see below).	If \a id is
+  any other negative integer, for instance -2, a unique identifier
+  (negative integer \<= -2) is generated.
+
+  The \a index specifies the position in the menu.  The menu item is
+  appended at the end of the list if \a index is negative.
+
+  \sa removeItem(), changeItem(), setAccel(), connectItem()
+*/
+
+int QMenuData::insertItem( const QPixmap &pixmap, const char *text,
+			   QPopupMenu *popup, int id, int index )
+{
+    return insertAny( text, &pixmap, popup, id, index );
+}
+
+/*!
+  Inserts a separator at position \a index.
+  The separator becomes the last menu item if \a index is negative.
 
   In a popup menu, a separator is rendered as a horizontal line.  In a
   Motif menubar, a separator is spacing, so the rest of the items
@@ -408,12 +507,12 @@ void QMenuData::insertSeparator( int index )
 
 /*!
   \fn void QMenuData::removeItem( int id )
-  Removes the menu item which has the identifier \e id.
+  Removes the menu item which has the identifier \a id.
   \sa removeItemAt(), clear()
 */
 
 /*!
-  Removes the menu item at position \e index.
+  Removes the menu item at position \a index.
   \sa removeItem(), clear()
 */
 
@@ -458,7 +557,7 @@ void QMenuData::clear()
 
 
 /*!
-  Returns the accelerator key that has been defined for the menu item \e id,
+  Returns the accelerator key that has been defined for the menu item \a id,
   or 0 if it has no accelerator key.
   \sa setAccel(), QAccel, qkeycode.h
 */
@@ -470,7 +569,7 @@ int QMenuData::accel( int id ) const
 }
 
 /*!
-  Defines an accelerator key for the menu item \e id.
+  Defines an accelerator key for the menu item \a id.
 
   An accelerator key consists of a key code and a combination of the modifiers
   \c SHIFT, \c CTRL and \c ALT (OR'ed or added).
@@ -510,7 +609,7 @@ void QMenuData::setAccel( int key, int id )
 
 
 /*!
-  Returns the text that has been set for menu item \e id, or 0 if no text
+  Returns the text that has been set for menu item \a id, or 0 if no text
   has been set.
   \sa changeItem(), pixmap()
 */
@@ -522,7 +621,7 @@ const char *QMenuData::text( int id ) const
 }
 
 /*!
-  Returns the pixmap that has been set for menu item \e id, or 0 if no pixmap
+  Returns the pixmap that has been set for menu item \a id, or 0 if no pixmap
   has been set.
   \sa changeItem(), text()
 */
@@ -534,7 +633,7 @@ QPixmap *QMenuData::pixmap( int id ) const
 }
 
 /*!
-  Changes the text of the menu item \e id.
+  Changes the text of the menu item \a id.
   \sa text()
 */
 
@@ -554,7 +653,7 @@ void QMenuData::changeItem( const char *text, int id )
 }
 
 /*!
-  Changes the pixmap of the menu item \e id.
+  Changes the pixmap of the menu item \a id.
   \sa pixmap()
 */
 
@@ -562,12 +661,13 @@ void QMenuData::changeItem( const QPixmap &pixmap, int id )
 {
     QMenuItem *mi = findItem( id );
     if ( mi ) {					// item found
-	if ( !mi->text_data.isNull() )		// delete text
-	    mi->text_data.resize( 0 );
 	register QPixmap *i = mi->pixmap_data;
 	bool fast_refresh = i != 0 &&
 	    i->width() == pixmap.width() &&
-	    i->height() == pixmap.height();
+	    i->height() == pixmap.height() &&
+	    !mi->text();
+	if ( !mi->text_data.isNull() )		// delete text
+	    mi->text_data.resize( 0 );
 	delete mi->pixmap_data;
 	mi->pixmap_data = new QPixmap( pixmap );
 	if ( fast_refresh )			// fast update
@@ -579,7 +679,35 @@ void QMenuData::changeItem( const QPixmap &pixmap, int id )
 
 
 /*!
-  Returns TRUE if the item with identifier \e id is enabled or FALSE if
+  Changes the pixmap and text of the menu item \a id.
+  \sa pixmap()
+*/
+
+void QMenuData::changeItem( const QPixmap &pixmap, const char *text, int id )
+{
+    QMenuItem *mi = findItem( id );
+    if ( mi ) {					// item found
+	QPixmap *i = mi->pixmap_data;
+	bool fast_refresh = i != 0 &&
+	    i->width() == pixmap.width() &&
+	    i->height() == pixmap.height();
+	if ( mi->text_data != text ) {
+	    fast_refresh = FALSE;
+	    mi->text_data = text;
+	}
+	delete mi->pixmap_data;
+	mi->pixmap_data = new QPixmap( pixmap );
+	if ( fast_refresh ) {			// fast update
+	    updateItem( id );
+	} else {
+	    menuContentsChanged();
+	}
+    }
+}
+
+
+/*!
+  Returns TRUE if the item with identifier \a id is enabled or FALSE if
   it is disabled.
   \sa setItemEnabled()
 */
@@ -591,8 +719,8 @@ bool QMenuData::isItemEnabled( int id ) const
 }
 
 /*!
-  Enables the menu item with identifier \e id if \e enable is TRUE, or
-  disables the item if \e enable is FALSE.
+  Enables the menu item with identifier \a id if \a enable is TRUE, or
+  disables the item if \a enable is FALSE.
   \sa isItemEnabled()
 */
 
@@ -620,7 +748,7 @@ bool QMenuData::isItemChecked( int id ) const
 }
 
 /*!
-  Checks a menu item if \e check is TRUE, or unchecks it if \e check is
+  Checks a menu item if \a check is TRUE, or unchecks it if \a check is
   FALSE.
   \sa isItemChecked()
 */
@@ -636,7 +764,7 @@ void QMenuData::setItemChecked( int id, bool check )
 
 
 /*!
-  Returns a pointer to the menu item with identifier \e id, or 0 if
+  Returns a pointer to the menu item with identifier \a id, or 0 if
   there is no item with such an identifier.
   \sa indexOf()
 */
@@ -665,7 +793,7 @@ QMenuItem *QMenuData::findItem( int id ) const
 }
 
 /*!
-  Returns the index of the menu item with identifier \e id, or -1 if
+  Returns the index of the menu item with identifier \a id, or -1 if
   there is no item with such an identifier.
   \sa idAt(), findItem()
 */
@@ -687,8 +815,8 @@ int QMenuData::indexOf( int id ) const
 }
 
 /*!
-  Returns the identifier of the menu item at position \e index in the internal
-  list, or -1 if \e index is out of range.
+  Returns the identifier of the menu item at position \a index in the internal
+  list, or -1 if \a index is out of range.
   \sa setId(), indexOf()
 */
 
@@ -699,7 +827,7 @@ int QMenuData::idAt( int index ) const
 }
 
 /*!
-  Sets the menu identifier of the item at \e index to \e id.
+  Sets the menu identifier of the item at \a index to \a id.
 
   If index is out of range the operation is ignored.
 
