@@ -1,5 +1,5 @@
 /**********************************************************************
-** $Id: qmlined.cpp,v 2.50 1997/03/23 18:35:17 agulbra Exp $
+** $Id: qmlined.cpp,v 2.75.2.5 1997/12/19 16:33:47 warwick Exp $
 **
 ** Definition of QMultiLineEdit widget class
 **
@@ -7,28 +7,31 @@
 **
 ** Copyright (C) 1992-1997 Troll Tech AS.  All rights reserved.
 **
-** This file is part of the non-commercial distribution of Qt 1.2.
+** This file is part of the free edition of Qt 1.33.
 **
 ** See the file LICENSE included in the distribution for the usage
 ** and distribution terms, or http://www.troll.no/qt/license.html.
 **
-** (This file differs from the one in the commercial distribution of
-** Qt only by this comment.)
+** IMPORTANT NOTE: You may NOT copy this file or any part of it into
+** your own programs or libraries.
+**
+** (This file differs from the one in the commercial edition of Qt
+** only by this comment.)
 **
 *****************************************************************************/
 
 #include "qmlined.h"
-
 #include "qpainter.h"
 #include "qscrbar.h"
 #include "qkeycode.h"
 #include "qclipbrd.h"
 #include "qpixmap.h"
 #include "qapp.h"
-
 #include <ctype.h>
 
-/*! \class QMultiLineEdit qmlined.h
+
+/*!
+  \class QMultiLineEdit qmlined.h
 
   \brief The QMultiLineEdit widget is a simple editor for inputting text.
 
@@ -43,6 +46,8 @@
 
   The default key bindings are described in keyPressEvent(); they cannot
   be customized except by inheriting the class.
+
+  <img src=qmlined-m.gif> <img src=qmlined-w.gif>
  */
 
 static const int BORDER = 3;
@@ -65,6 +70,8 @@ static int textWidthWithTabs( const QFontMetrics &fm,const char *s,int nChars )
 
     int         dist = 0;
     const char *tmp  = s;
+    if ( !tmp )
+	return 0;
     int tabDist = tabStopDist(fm);
     while ( *tmp && tmp - s < nChars ) {
 	if ( *tmp == '\t') {
@@ -143,20 +150,13 @@ QMultiLineEdit::QMultiLineEdit( QWidget *parent , const char *name )
 		   Tbl_smoothVScrolling |
 		   Tbl_clipCellPainting
 		   );
-    switch ( style() ) {
-	case WindowsStyle:
-	case MotifStyle:
-	    setFrameStyle( QFrame::WinPanel | QFrame::Sunken );
-	    setBackgroundColor( colorGroup().base() );
-	    break;
-	default:
-	    setFrameStyle( QFrame::Panel | QFrame::Plain );
-	    setLineWidth( 1 );
-    }
+    setFrameStyle( QFrame::WinPanel | QFrame::Sunken );
+    setBackgroundMode( PaletteBase );
     setFocusPolicy( StrongFocus );
     setCursor( ibeamCursor );
     ((QScrollBar*)verticalScrollBar())->setCursor( arrowCursor );
     ((QScrollBar*)horizontalScrollBar())->setCursor( arrowCursor );
+    dummy = FALSE;
     insertLine( "", -1 );
     readOnly 	   = FALSE;
     cursorOn	   = FALSE;
@@ -175,9 +175,11 @@ QMultiLineEdit::QMultiLineEdit( QWidget *parent , const char *name )
     scrollTimer    = 0;
 }
 
-/*! \fn int QMultiLineEdit::numLines()
+/*! \fn int QMultiLineEdit::numLines() const
 
-  Returns the number of lines in the editor.
+  Returns the number of lines in the editor. The count includes any
+  empty lines at top and bottom, so for an empty editor this method
+  will return 1.
 */
 
 /*! \fn bool QMultiLineEdit::atEnd() const
@@ -268,7 +270,7 @@ void QMultiLineEdit::setReadOnly( bool on )
 
 QMultiLineEdit::~QMultiLineEdit()
 {
-    
+    delete contents;
 }
 
 const int nBuffers = 3;
@@ -301,7 +303,8 @@ static QPixmap *getCacheBuffer( QSize sz )
     }
     if ( ++freeNext == 3 )
 	freeNext = 0;
-    buffer[freeNext]->resize( sz );
+    delete buffer[freeNext];
+    buffer[freeNext] = new QPixmap( sz );
     return buffer[freeNext];
 }
 
@@ -329,28 +332,13 @@ void QMultiLineEdit::paintCell( QPainter *painter, int row, int )
 
     p.setTabStops( tabStopDist(fm) );
 
-    int yPos = fm.leading()/2 - 1;
+    int yPos = 0;
     int markX1, markX2;				// in x-coordinate pixels
     markX1 = markX2 = 0;			// avoid gcc warning
     if ( markIsOn ) {
 	int markBeginX, markBeginY;
 	int markEndX, markEndY;
-	if ( markAnchorY < markDragY ) {
-	    markBeginX = markAnchorX;
-	    markBeginY = markAnchorY;
-	    markEndX   = markDragX;
-	    markEndY   = markDragY;
-	} else {
-	    markBeginX = markDragX;
-	    markBeginY = markDragY;
-	    markEndX   = markAnchorX;
-	    markEndY   = markAnchorY;
-	}
-	if ( markAnchorY == markDragY && markBeginX > markEndX ) {
-	    int tmp    = markBeginX;
-	    markBeginX = markEndX;
-	    markEndX   = tmp;
-	}
+	getMarkedRegion( &markBeginY, &markBeginX, &markEndY, &markEndX );
 	if ( row >= markBeginY && row <= markEndY ) {
 	    if ( row == markBeginY ) {
 		markX1 = markBeginX;
@@ -385,9 +373,15 @@ void QMultiLineEdit::paintCell( QPainter *painter, int row, int )
 	p.setClipping( TRUE );
 	p.setClipRect( fillxpos1 - updateR.left(), 0, 
 		       fillxpos2 - fillxpos1, cellHeight(row) );
-	p.fillRect( fillxpos1, 0, fillxpos2 - fillxpos1, 
-		    cellHeight(row), g.text() );
-	p.setPen( g.base() );
+	if ( style() == WindowsStyle ) {
+	    p.fillRect( fillxpos1, 0, fillxpos2 - fillxpos1, cellHeight(row), 
+			QApplication::winStyleHighlightColor() );
+	    p.setPen( white );	    
+	} else {
+	    p.fillRect( fillxpos1, 0, fillxpos2 - fillxpos1, cellHeight(row),
+			g.text() );
+	    p.setPen( g.base() );
+	}
 	p.drawText( BORDER, yPos, xpos2 - BORDER, cellHeight( row ),
 		    ExpandTabs, s->data() );
 	p.setClipping( FALSE );
@@ -395,8 +389,7 @@ void QMultiLineEdit::paintCell( QPainter *painter, int row, int )
 
     if ( row == cursorY && cursorOn && !readOnly ) {
 	int cursorPos = QMIN( (int)s->length(), cursorX );
-	int cXPos   = BORDER +
-			textWidthWithTabs( fm, *s, cursorPos ) - 1;
+	int cXPos   = BORDER + textWidthWithTabs( fm, *s, cursorPos ) - 1;
 	int cYPos   = 0;
 	if ( hasFocus() ) {
 	    p.drawLine( cXPos - 2, cYPos,
@@ -476,7 +469,7 @@ void QMultiLineEdit::timerEvent( QTimerEvent *t )
     if ( hasFocus() && t->timerId() == blinkTimer ) {
 	cursorOn = !cursorOn;
 	updateCell( cursorY, 0, FALSE );
-    } else if ( t->timerId() == scrollTimer ) {
+    } else if ( dragScrolling && t->timerId() == scrollTimer ) {
 	QPoint p = mapFromGlobal( QCursor::pos() );
 	if ( p.y() < 0 )
 	    cursorUp( TRUE );
@@ -489,6 +482,31 @@ void QMultiLineEdit::timerEvent( QTimerEvent *t )
     }
 }
 
+/*!
+  If there is marked text, sets \a line1, \a col1, \a line2 and \a col2
+  to the start and end of the marked region and returns TRUE. Returns
+  FALSE if there is no marked text.
+ */
+bool QMultiLineEdit::getMarkedRegion( int *line1, int *col1, 
+				      int *line2, int *col2 ) const
+{
+    if ( !markIsOn )
+	return FALSE;
+    if ( markAnchorY < markDragY ||
+	 markAnchorY == markDragY && markAnchorX < markDragX) {
+	*line1 = markAnchorY;
+	*col1 = markAnchorX;
+	*line2 = markDragY;
+	*col2 = markDragX;
+    } else {
+	*line1 = markDragY;
+	*col1 = markDragX;
+	*line2 = markAnchorY;
+	*col2 = markAnchorX;
+    }
+    return TRUE;
+}
+
 
 /*!
   Returns TRUE if there is marked text.
@@ -506,49 +524,63 @@ bool QMultiLineEdit::hasMarkedText() const
 
 QString QMultiLineEdit::markedText() const
 {
-    if ( !markIsOn )
+    int markBeginX, markBeginY;
+    int markEndX, markEndY;
+    if ( !getMarkedRegion( &markBeginY, &markBeginX, &markEndY, &markEndX ) )
 	return QString();
-    if ( markAnchorY == markDragY ) { //just one line
-	int minMark = markDragX < markAnchorX ? markDragX : markAnchorX;
-	int maxMark = markDragX > markAnchorX ? markDragX : markAnchorX;
-	QString *s  = getString( markAnchorY );
+    if ( markBeginY == markEndY ) { //just one line
+	QString *s  = getString( markBeginY );
 	ASSERT(s);
-	return s->mid( minMark, maxMark - minMark );
+	return s->mid( markBeginX, markEndX - markBeginX );
     } else { //multiline
-	int markBeginX, markBeginY;
-	int markEndX, markEndY;
-	if ( markAnchorY < markDragY ) {
-	    markBeginX = markAnchorX;
-	    markBeginY = markAnchorY;
-	    markEndX   = markDragX;
-	    markEndY   = markDragY;
-	} else {
-	    markBeginX = markDragX;
-	    markBeginY = markDragY;
-	    markEndX   = markAnchorX;
-	    markEndY   = markAnchorY;
-	}
-	
 	ASSERT( markBeginY >= 0);
 	ASSERT( markEndY < (int)contents->count() );
 	
-	QString tmp;
 	QString *firstS, *lastS;
 	firstS = getString( markBeginY );
 	lastS  = getString( markEndY );
 	ASSERT( firstS != lastS );
 
-	tmp = firstS->mid( markBeginX, firstS->length() - markBeginX  );
+	int len = firstS->length() - markBeginX + 1;
+	int i;
 
-	for( int i = markBeginY + 1 ; i < markEndY ; i++ ) {
-	    tmp += "\n";
-	    tmp += *(contents->at( i ));
+	for( i = markBeginY + 1 ; i < markEndY ; i++ ) {
+	    len += lineLength( i ) + 1;
 	}
-	tmp += "\n";
+	len += markEndX + 1;
 
-	if ( markEndX != 0 ) {
-	    tmp += lastS->left( markEndX  );
+	QString tmp( len + 1 );
+	int idx = 0;
+
+	if ( firstS && firstS->data() ) {
+	    const char *p = firstS->data() + markBeginX;
+	    while (( tmp[idx++] = *p++ ))
+		;
+	    tmp[idx-1] = '\n';
+	} else {
+	    tmp[idx++] = '\n';
 	}
+
+	for( i = markBeginY + 1; i < markEndY ; i++ ) {
+	    const char *p;
+	    p = (contents->at( i ))->data();
+	    if ( p ) {
+		while (( tmp[idx++] = *p++ ))
+		    ;
+		tmp[idx-1] = '\n';
+	    } else {
+		tmp[idx++] = '\n';
+	    }
+	}
+	if ( lastS ) {
+	    int i = 0;
+	    while ( i < markEndX )
+		tmp[idx++] = (*lastS)[i++];
+	    tmp[idx] = 0;
+	} else {
+	    tmp[idx++] = 0;
+	}
+
 	return tmp;
     }
 }
@@ -584,11 +616,25 @@ QString QMultiLineEdit::text() const
     if ( contents->count() == 0 )
 	return QString( "" );
 
-    QString tmp = contents->at( 0 )->copy();
-    for( int i = 1 ; i < (int)contents->count() ; i++ ) {
-	tmp += "\n";
-	tmp += *(contents->at( i ));
+    int len = 0;
+    int i;
+    for( i = 0 ; i < (int)contents->count() ; i++ ) {
+	len += lineLength( i ) + 1;
     }
+
+    QString tmp( len + 1 );
+    int idx = 0;
+    for( i = 0 ; i < (int)contents->count() ; i++ ) {
+	const char *p = (contents->at( i ))->data();
+	if ( p ) {
+	    while (( tmp[idx++] = *p++ ))
+		;
+	    tmp[idx-1] = '\n';
+	} else {
+	    tmp[idx++] = '\n';
+	}
+    }
+    tmp[idx-1] = 0;
     return tmp;
 }
 
@@ -602,7 +648,7 @@ void QMultiLineEdit::selectAll()
     markAnchorX    = 0;
     markAnchorY    = 0;
     markDragY = numLines() - 1;
-    markDragX = lineLength( cursorY );
+    markDragX = lineLength( markDragY );
     markIsOn = ( markDragX != markAnchorX ||  markDragY != markAnchorY );
     repaint( TRUE );
 }
@@ -721,7 +767,9 @@ void QMultiLineEdit::keyPressEvent( QKeyEvent *e )
 	    e->ignore();
 	return;
     }
-    if ( e->ascii() >= 32 && e->key() != Key_Delete ) {
+    if ( e->ascii() >= 32 &&
+	 e->key() != Key_Delete &&
+	 e->key() != Key_Backspace ) {
 	insertChar( e->ascii() );
 	if ( textDirty )
 	    emit textChanged();
@@ -833,6 +881,9 @@ void QMultiLineEdit::pageDown( bool mark )
     bool oldAuto = autoUpdate();
     if ( mark )
 	setAutoUpdate( FALSE );
+
+    if ( partiallyInvisible( cursorY ) )
+	cursorY = topCell();
     int delta = cursorY - topCell();
     int pageSize = viewHeight() / cellHeight();
     int newTopCell = QMIN( topCell() + pageSize, numLines() - 1 - pageSize );
@@ -883,6 +934,8 @@ void QMultiLineEdit::pageUp( bool mark )
     bool oldAuto = autoUpdate();
     if ( mark )
 	setAutoUpdate( FALSE );
+    if ( partiallyInvisible( cursorY ) )
+	cursorY = topCell();
     int delta = cursorY - topCell();
     int pageSize = viewHeight() / cellHeight();
     bool partial = delta == pageSize && viewHeight() != pageSize * cellHeight();
@@ -923,8 +976,31 @@ void QMultiLineEdit::pageUp( bool mark )
 	turnMarkOff();
 }
 
+/*
+  Scans txt, returning one line in s, returns the address of the next
+  line, 0 if end of text.
 
-
+ */
+static const char *getOneLine( const char *txt, QString **s )
+{
+    if ( !txt ) {
+	*s = new QString;
+	return 0;
+    }
+    int len = 0;
+    const char *p = txt;
+    while ( *p && *p != '\n' ) {
+	p++;
+	len++;
+    }
+    *s = new QString( len + 1 );
+    memmove( (*s)->data(), txt, len );
+    (**s)[len] = 0;
+    if (*p)
+	return p+1;
+    else
+	return 0;
+}
 
 
 /*!
@@ -944,61 +1020,47 @@ void QMultiLineEdit::insertAt( const char *txt, int line, int col )
 
     QString *oldLine = getString( line );
     ASSERT( oldLine );
-
-    QString t;
-    if ( txt )
-	t.setRawData( txt, strlen(txt) + 1 );
-    else
-	t = "";
-
-    int from = 0;
-
     bool cursorAfter = atEnd() || cursorY > line 
-		       || cursorY == line && cursorX >= col;
+	  || cursorY == line && cursorX >= col;
     bool onLineAfter = cursorY == line && cursorX >= col;
 
-    int to = t.find( '\n' );
-    if ( to < 0 ) { //just one line
-	oldLine->insert( col, t );
+    QString *textLine;
+    const char *p = getOneLine( txt, &textLine );
+
+    if ( !p ) { //single line
+	oldLine->insert( col, *textLine );
 	int w = textWidth( oldLine );
 	setWidth( QMAX( cellWidth(), w ) );
 	if ( onLineAfter )
-	    cursorX += t.length();
-    } else { //multiline
-	bool aupd = autoUpdate();
-	setAutoUpdate( FALSE );
+	    cursorX += textLine->length();
+    } else {
+	int w = cellWidth();
 	QString newString = oldLine->mid( col, oldLine->length() );
 	oldLine->remove( col, oldLine->length() );
 	if ( onLineAfter )
 	    cursorX -= oldLine->length();
-	*oldLine += t.left( to );
+	*oldLine += *textLine;
+	w = QMAX( textWidth( oldLine ), w );
 	line++;
 	cursorY++;
-	if ( line >= numLines() ) {
-	    insertLine( "", numLines() );
-	}
-	from = to + 1;
-	while ( (to = t.find( '\n', from )) > 0 ) {
-	    insertLine( t.mid( from, to - from ), line++ );
-	    from = to + 1;
+	while (( p = getOneLine( p, &textLine ) )) {
+	    ASSERT ( textLine );
+	    contents->insert( line++, textLine );
+	    w = QMAX( textWidth( textLine ), w );
 	    if ( cursorAfter )
 		cursorY++;
 	}
-	int lastLen = t.length() - from;
+	int lastLen = textLine->length();
 	if ( onLineAfter )
 	    cursorX += lastLen;
-	newString.prepend( t.right( lastLen ) );
+	newString.prepend( *textLine );
+	w = QMAX( textWidth( textLine ), w );
 	insertLine( newString, line );
-	updateCellWidth();
-	setAutoUpdate( aupd );
-	if ( aupd )
-	    repaint( FALSE );
+	setWidth( w );
     }
+    if ( autoUpdate() )
+	repaint( FALSE );
     textDirty = TRUE;
-    ASSERT( numLines() != 0 );
-    makeVisible();
-    if ( txt )
-	t.resetRawData( txt, strlen(txt) + 1 );
 }
 
 
@@ -1017,54 +1079,28 @@ void QMultiLineEdit::insertLine( const char *txt, int line )
 	//debug ("insertLine: removing dummy, %d", count() );
 	dummy = FALSE;
     }
+    if ( line < 0 || line >= numLines() )
+	line = numLines();
+    QString *textLine;
+    int w = cellWidth();
+    const char *p = txt;
+    do {
+	p = getOneLine( p, &textLine );
+	ASSERT ( textLine );
+	contents->insert( line++, textLine );
+	w = QMAX( textWidth( textLine ), w );
+    } while ( p );
 
-    QString t;
-    if ( txt )
-	t.setRawData( txt, strlen(txt) + 1 );
-    else
-	t = "";
+    setWidth( w );
+    setNumRows( contents->count() );
+    if ( autoUpdate() ) //### && visibleChanges
+	repaint( FALSE );
 
-    int to = t.find( '\n' );
-    if ( to < 0 ) { //just one line
-	QString *txtCopy = new QString( txt );
-	if ( line < 0 || !contents->insert( line, txtCopy ) )
-	    contents->append( txtCopy );
-	bool updt = autoUpdate() && rowIsVisible( line );
-	int w = textWidth( txtCopy );
-	setWidth( QMAX( cellWidth(), w ) );
-	setNumRows( contents->count() );
-
-	if ( updt )
-	    repaint( FALSE );
-    } else { //multiline
-	int from = 0;
-	bool hadAutoUpdate = autoUpdate();
-	setAutoUpdate( FALSE );
-	if ( line < 0 || line >= numLines() )
-	    line = numLines();
-	while ( to >= 0 ) {
-	    insertLine( t.mid( from, to - from ), line++ );
-	    from = to + 1;
-	    to = t.find( '\n', from );
-	}
-	int lastLen = t.length() - from;
-	insertLine( t.right( lastLen ), line );
-	setNumRows( contents->count() );
-	updateCellWidth();
-	if ( hadAutoUpdate ) {
-	    setAutoUpdate( TRUE );
-	    repaint(FALSE);
-	}
-    }
-    textDirty = TRUE;
     ASSERT( numLines() != 0 );
     makeVisible();
-    if ( txt )
-	t.resetRawData( txt, strlen(txt) + 1 );
+    textDirty = TRUE;
 }
 
-
-
 /*!
   Deletes the line at line number \a line. If \a
   line is less than zero, or larger than the number of lines, 
@@ -1360,34 +1396,20 @@ void QMultiLineEdit::backspace()
 
 void QMultiLineEdit::del()
 {
-    if ( hasMarkedText() ) {
+    int markBeginX, markBeginY;
+    int markEndX, markEndY;
+    if ( getMarkedRegion( &markBeginY, &markBeginX, &markEndY, &markEndX ) ) {
 	textDirty = TRUE;
 	setAutoUpdate( FALSE );
-	if ( markAnchorY == markDragY ) { //just one line
-	    int minMark = markDragX < markAnchorX ? markDragX : markAnchorX;
-	    int maxMark = markDragX > markAnchorX ? markDragX : markAnchorX;
-	    QString *s  = getString( markAnchorY );
+	if ( markBeginY == markEndY ) { //just one line
+	    QString *s  = getString( markBeginY );
 	    ASSERT(s);
-	    s->remove( minMark, maxMark - minMark );
-	    cursorX  = minMark;
-	    cursorY  = markAnchorY;
+	    s->remove( markBeginX, markEndX - markBeginX );
+	    cursorX  = markBeginX;
+	    cursorY  = markBeginY;
 	    markIsOn    = FALSE;
 	    updateCellWidth();
 	} else { //multiline
-	    int markBeginX, markBeginY;
-	    int markEndX, markEndY;
-	    if ( markAnchorY < markDragY ) {
-		markBeginX = markAnchorX;
-		markBeginY = markAnchorY;
-		markEndX   = markDragX;
-		markEndY   = markDragY;
-	    } else {
-		markBeginX = markDragX;
-		markBeginY = markDragY;
-		markEndX   = markAnchorX;
-		markEndY   = markAnchorY;
-	    }
-
 	    ASSERT( markBeginY >= 0);
 	    ASSERT( markEndY < (int)contents->count() );
 
@@ -1567,10 +1589,29 @@ void QMultiLineEdit::mouseMoveEvent( QMouseEvent *e )
     }
     newY = QMIN( (int)contents->count() - 1, newY );
     QFontMetrics fm( font() );
-    int newX = xPosToCursorPos( *getString( newY ), fm,
+    QString *s = getString( newY );
+    int newX = xPosToCursorPos( *s, fm,
 				e->pos().x() - BORDER + xOffset(),
 				cellWidth() - 2 * BORDER );
 
+    if ( wordMark ) {
+	int i = newX;
+	int lim = s->length();
+	int startclass = i < 0 || i >= lim ? -1 : charClass(s->at(i));
+	if ( markAnchorY < markDragY || markAnchorY == markDragY
+	     && markAnchorX < markDragX ) {
+	    // going right
+	    while ( i < lim && charClass(s->at(i)) == startclass )
+		i++;
+	} else {
+	    // going left
+	    while ( i >= 0 && charClass(s->at(i)) == startclass )
+		i--;
+	    i++;
+	}
+	newX = i;
+    }
+
     if ( markDragX == newX && markDragY == newY )
 	return;
     int oldY = markDragY;
@@ -1607,6 +1648,7 @@ void QMultiLineEdit::mouseDoubleClickEvent( QMouseEvent *m )
     if ( m->button() ==  LeftButton ) {
 	dragMarking    = TRUE;
 	markWord( cursorX, cursorY );
+	wordMark = TRUE;
 	updateCell( cursorY, 0, FALSE );
     }
 }
@@ -1684,6 +1726,7 @@ int QMultiLineEdit::mapFromView( int xPos, int line )
 int QMultiLineEdit::mapToView( int xIndex, int line )
 {
     QString *s = getString( line );
+    ASSERT( s );
     xIndex = QMIN( (int)s->length(), xIndex );
     QFontMetrics fm( font() );
     return BORDER + textWidthWithTabs( fm, *s, xIndex ) - 1;
@@ -1697,7 +1740,6 @@ int QMultiLineEdit::mapToView( int xIndex, int line )
 void QMultiLineEdit::updateCellWidth()
 {
     QString *s = contents->first();
-    QFontMetrics fm( font() );
     int maxW = 0;
     int w;
     while ( s ) {
@@ -1801,17 +1843,19 @@ void QMultiLineEdit::markWord( int posx, int posy )
 {
     QString *s = contents->at( posy );
     ASSERT( s );
+    int lim = s->length();
     int i = posx - 1;
 
-    while ( i >= 0 && isprint(s->at(i)) && !isspace(s->at(i)) )
+    int startclass = i < 0 || i >= lim ? -1 : charClass( s->at(i) );
+
+    while ( i >= 0 && charClass(s->at(i)) == startclass )
 	i--;
     i++;
     markAnchorY = posy;
     markAnchorX = i;
 
-    int lim = s->length();
     i = posx;
-    while ( i < lim && isprint(s->at(i)) && !isspace(s->at(i)) )
+    while ( i < lim && charClass(s->at(i)) == startclass )
 	i++;
     markDragX = i;
     markDragY = posy;
@@ -1819,6 +1863,19 @@ void QMultiLineEdit::markWord( int posx, int posy )
     copyText();
 }
 
+/*!
+  This may become a protected virtual member in a future Qt.
+  This implementation is an example of a useful classification
+  that aides selection of common units like filenames and URLs.
+*/
+int QMultiLineEdit::charClass( char ch )
+{
+    if ( !isprint(ch) || isspace(ch) ) return 1;
+    else if ( isalnum(ch) || ch=='-' || ch=='+' || ch==':'
+	    || ch=='.' || ch=='/' || ch=='\\'
+	    || ch=='@' || ch=='$' || ch=='~' ) return 2;
+    else return 3;
+}
 
 /*!
   Copies the marked text to the clipboard.
@@ -1890,10 +1947,16 @@ void QMultiLineEdit::setWidth( int w )
   The parameters are adjusted to lie within the legal range.
 
   If \a mark is FALSE, the selection is cleared. otherwise it is extended
+
+  \sa cursorPosition()
 */
 
 void QMultiLineEdit::setCursorPosition( int line, int col, bool mark )
 {
+    if ( mark && !hasMarkedText() ) {
+	markAnchorX    = cursorX;
+	markAnchorY    = cursorY;
+    }
     int oldY = cursorY;
     cursorY = QMAX( QMIN( line, numLines() - 1), 0 );
     cursorX = QMAX( QMIN( col,  lineLength( cursorY )), 0 );
@@ -1911,6 +1974,24 @@ void QMultiLineEdit::setCursorPosition( int line, int col, bool mark )
 /*!
   Sets \a line to the current line and \a col to the current character 
   position within that line.
+
+  \sa setCursorPosition()
+*/
+
+void QMultiLineEdit::cursorPosition( int *line, int *col ) const
+{
+    if ( line )
+	*line = cursorY;
+    if ( col )
+	*col = cursorX;
+}
+
+
+/*!
+  Sets \a line to the current line and \a col to the current character 
+  position within that line.
+
+  OBSOLETE - Use cursorPosition() instead.
 */
 
 void QMultiLineEdit::getCursorPosition( int *line, int *col )
@@ -1940,7 +2021,7 @@ bool QMultiLineEdit::autoUpdate() const
 
   If \e enable is TRUE (this is the default) then the editor updates
   itself automatically whenever it has changed in some way (generally,
-  when text has vbeen inserted or deleted).
+  when text has been inserted or deleted).
 
   If \e enable is FALSE, the view does NOT repaint itself, or update
   its internal state variables itself when it is changed.  This can be
@@ -1962,3 +2043,24 @@ void QMultiLineEdit::setAutoUpdate( bool enable )
 {
     QTableView::setAutoUpdate( enable );
 }
+
+
+
+/*!
+  Returns the top center point where the cursor is drawn
+*/
+
+QPoint QMultiLineEdit::cursorPoint() const
+{
+    QPoint cp( 0, 0 );
+
+    QFontMetrics fm( font() );
+    int col, row;
+    col = row = 0;
+    cursorPosition( &row, &col );
+    const char* line = textLine( row );
+    ASSERT( line );
+    cp.setX( BORDER + textWidthWithTabs( fm, line, col ) - 1 );
+    cp.setY( (row * cellHeight()) + viewRect().y() );
+    return cp;
+}
