@@ -1,5 +1,5 @@
 /****************************************************************************
-** $Id: qpopmenu.cpp,v 2.11 1997/03/26 22:21:54 hanord Exp $
+** $Id: qpopmenu.cpp,v 2.47.2.2 1997/11/19 05:14:29 agulbra Exp $
 **
 ** Implementation of QPopupMenu class
 **
@@ -7,13 +7,16 @@
 **
 ** Copyright (C) 1992-1997 Troll Tech AS.  All rights reserved.
 **
-** This file is part of the non-commercial distribution of Qt 1.2.
+** This file is part of the free edition of Qt 1.33.
 **
 ** See the file LICENSE included in the distribution for the usage
 ** and distribution terms, or http://www.troll.no/qt/license.html.
 **
-** (This file differs from the one in the commercial distribution of
-** Qt only by this comment.)
+** IMPORTANT NOTE: You may NOT copy this file or any part of it into
+** your own programs or libraries.
+**
+** (This file differs from the one in the commercial edition of Qt
+** only by this comment.)
 **
 *****************************************************************************/
 
@@ -23,11 +26,12 @@
 #include "qaccel.h"
 #include "qpainter.h"
 #include "qdrawutl.h"
-#include "qscrbar.h"				// qDrawArrow
 #include "qapp.h"
+#include "qbitmap.h"
+#include "qpmcache.h"
 #include <ctype.h>
 
-RCSTAG("$Id: qpopmenu.cpp,v 2.11 1997/03/26 22:21:54 hanord Exp $");
+RCSTAG("$Id: qpopmenu.cpp,v 2.47.2.2 1997/11/19 05:14:29 agulbra Exp $");
 
 
 // Motif style parameters
@@ -60,6 +64,69 @@ static const int motifCheckMarkHMargin	= 2;	// horiz. margins of check mark
 */
 
 
+
+
+// used for internal communication - to be replaced with a class
+// members in 2.0
+static QPopupMenu * syncMenu;
+static int syncMenuId;
+
+
+//### How to provide new member variables while keeping binary compatibility:
+#if QT_VERSION == 200
+#error "Remove QPopupMenu dict!"
+#endif
+
+#include <qptrdict.h>
+
+
+struct QPopupMenuExtra {
+    bool hasDoubleItem;
+    int maxPMWidth;
+};
+
+
+static QPtrDict<QPopupMenuExtra> *qpm_extraStuff = 0;
+
+static void cleanupPopupMenu()
+{
+    delete qpm_extraStuff;
+    qpm_extraStuff = 0;
+}
+
+
+static QPopupMenuExtra * makePMDict( QPopupMenu *that )
+{
+    if ( !qpm_extraStuff ) {
+	qpm_extraStuff = new QPtrDict<QPopupMenuExtra>;
+	CHECK_PTR( qpm_extraStuff );
+	qpm_extraStuff->setAutoDelete( TRUE );
+	qAddPostRoutine( cleanupPopupMenu );
+    }
+
+    QPopupMenuExtra *x = (QPopupMenuExtra *)qpm_extraStuff->find( that );
+    if ( !x ) {
+	x = new QPopupMenuExtra;
+	x->hasDoubleItem = FALSE;
+	x->maxPMWidth = 0;
+	qpm_extraStuff->insert( that, x );
+    }
+
+    return x;
+}
+
+
+static QPopupMenuExtra * lookInPMDict( const QPopupMenu *that )
+{
+    QPopupMenuExtra *x = 0;
+    if ( qpm_extraStuff )
+	x = (QPopupMenuExtra *)qpm_extraStuff->find( (void *)that );
+    return x;
+}
+
+
+
+
 /*!
   \class QPopupMenu qpopmenu.h
   \brief The QPopupMenu class provides a popup menu widget.
@@ -72,6 +139,10 @@ static const int motifCheckMarkHMargin	= 2;	// horiz. margins of check mark
 
   menu/menu.cpp is a typical example of QMenuBar and QPopupMenu use.
 
+  \important insertItem clear text pixmap
+
+  <img src=qpopmenu-m.gif> <img src=qpopmenu-w.gif>
+
   \sa QMenuBar
 */
 
@@ -87,19 +158,22 @@ static void getSizeOfBitmap( int gs, int *w, int *h )
 }
 
 
-static int getWidthOfCheckCol( int gs )
+static int getWidthOfCheckCol( QPopupMenu *that, int gs )
 {
-    int w = motifItemFrame + 2 * motifCheckMarkHMargin;
-    if ( gs == WindowsStyle )
-	w += 7;
-    else
-	w += 9;
+    QPopupMenuExtra *x= lookInPMDict( that );
+    int pmw = x ? x->maxPMWidth : 0;
+    int cmw = 7;   // check mark width
+    int w = cmw > pmw ? cmw : pmw;
+    if ( gs == MotifStyle )
+	w += 2;
+    w += motifItemFrame + 2 * motifCheckMarkHMargin;
     return w;
 }
 
 // Checkmark drawing -- temporarily here...
 static void qDrawCheckMark( QPainter *p, int x, int y, int w, int h,
-			    const QColorGroup &g, GUIStyle gs, bool act )
+			    const QColorGroup &g, GUIStyle gs,
+			    bool act, bool dis )
 {
     int markW, markH;
     getSizeOfBitmap( gs, &markW, &markH );
@@ -123,7 +197,20 @@ static void qDrawCheckMark( QPainter *p, int x, int y, int w, int h,
 	    a.setPoint( 2*i+1, xx, yy+2 );
 	    xx++; yy--;
 	}
-	p->setPen( act ? white : black );
+	if ( dis && !act ) {
+	    uint pnt;
+	    p->setPen( white );
+	    QPoint offset(1,1);
+	    for ( pnt = 0; pnt < a.size(); pnt++ )
+		a[pnt] += offset;
+	    p->drawLineSegments( a );
+	    for ( pnt = 0; pnt < a.size(); pnt++ )
+		a[pnt] -= offset;
+	}
+#if 0
+	p->setPen( act && !dis ? white : g.text() );
+#endif
+	p->setPen( g.text() );
 	p->drawLineSegments( a );
     }
     else {
@@ -157,11 +244,14 @@ static QString accel_str( int k )
     }
     k &= ~(SHIFT | CTRL | ALT);
     QString p;
-    if ( k >= Key_F1 && k <= Key_F24 )
+    if ( (k & ASCII_ACCEL) == ASCII_ACCEL ) {
+	k &= ~ASCII_ACCEL;
+	p.sprintf( "%c", (k & 0xff) );
+    } else if ( k >= Key_F1 && k <= Key_F24 ) {
 	p.sprintf( "F%d", k - Key_F1 + 1 );
-    else if ( k >= Key_Space && k <= Key_AsciiTilde )
+    } else if ( k >= Key_Space && k <= Key_AsciiTilde ) {
 	p.sprintf( "%c", k );
-    else {
+    } else {
 	switch ( k ) {
 	    case Key_Escape:
 		p = "Esc";
@@ -276,12 +366,12 @@ QPopupMenu::QPopupMenu( QWidget *parent, const char *name )
 	case WindowsStyle:
 	    setFrameStyle( QFrame::WinPanel | QFrame::Raised );
 	    setMouseTracking( TRUE );
-	    setCheckable( TRUE );		
+	    setCheckableFlag( TRUE );		
 	    break;
 	case MotifStyle:
 	    setFrameStyle( QFrame::Panel | QFrame::Raised );
 	    setLineWidth( motifPopupFrame );
-	    setCheckable( FALSE );		
+	    setCheckableFlag( FALSE );		
 	    break;
 	default:
 	    setFrameStyle( QFrame::Panel | QFrame::Plain );
@@ -295,6 +385,13 @@ QPopupMenu::QPopupMenu( QWidget *parent, const char *name )
 
 QPopupMenu::~QPopupMenu()
 {
+    if ( syncMenu == this ) {
+	qApp->exit_loop();
+	syncMenu = 0;
+    }
+	
+    if ( qpm_extraStuff )
+	qpm_extraStuff->remove( this );
     delete autoaccel;
     if ( parentMenu )
 	parentMenu->removePopup( this );	// remove from parent menu
@@ -318,6 +415,23 @@ void QPopupMenu::updateItem( int id )		// update popup menu item
 */
 
 void QPopupMenu::setCheckable( bool enable )
+{
+    bool oldState = isCheckable();
+    bool newState = (style() == WindowsStyle) || enable;
+    if ( oldState != newState ) {
+	setCheckableFlag( newState );
+	if ( !newState ) {
+	    // turning off isCheckable; must look for pixmaps
+	    updateSize();
+	} 
+    }
+}
+
+
+/*!
+  Does the internal magic necessary to set the checkable flag.
+ */
+void QPopupMenu::setCheckableFlag( bool enable )
 {
     bool oldState = isCheckable();
     bool newState = (style() == WindowsStyle) || enable;
@@ -325,8 +439,7 @@ void QPopupMenu::setCheckable( bool enable )
 	if ( newState ) {
 	    setNumCols( 2 );
 	    tabCheck |= 0x80000000;
-	}
-	else {
+	} else {
 	    setNumCols( 1 );
 	    tabCheck &= 0x7FFFFFFF;
 	}
@@ -335,6 +448,9 @@ void QPopupMenu::setCheckable( bool enable )
     }
 }
 
+
+
+
 /*!
   Returns whether display of check marks by the menu items is enabled.
 
@@ -403,8 +519,9 @@ void QPopupMenu::frameChanged()
 
 
 /*!
-  Opens the popup menu so that the item number \e indexAtPoint
-  will be at the specified global position \e pos.
+  Opens the popup menu so that the item number \a indexAtPoint will be
+  at the specified \e global position \a pos.  To translate a widget's
+  local coordinates into global coordinates, use QWidget::mapToGlobal().
 */
 
 void QPopupMenu::popup( const QPoint &pos, int indexAtPoint )
@@ -434,6 +551,43 @@ void QPopupMenu::popup( const QPoint &pos, int indexAtPoint )
     show();
 }
 
+/*!
+  \fn void QPopupMenu::activated( int id )
+
+  This signal is emitted when a menu item is selected; \a id is the id
+  of the selected item.
+
+  Normally, you will connect each menu item to a single slot using
+  QMenuData::insertItem(), but sometimes you will want to connect
+  several items to a single slot (most often if the user selects from
+  an array).  This signal is handy in such cases.
+
+  \sa highlighted(), QMenuData::insertItem()
+*/
+
+/*!
+  \fn void QPopupMenu::highlighted( int id )
+
+  This signal is emitted when a menu item is highlighted; \a id is the
+  id of the highlighted item.
+
+  Normally, you will connect each menu item to a single slot using
+  QMenuData::insertItem(), but sometimes you will want to connect
+  several items to a single slot (most often if the user selects from
+  an array).  This signal is handy in such cases.
+
+  \sa activated(), QMenuData::insertItem()
+*/
+
+/*! \fn void QPopupMenu::highlightedRedirect( int id )
+  \internal
+  Used internally to connect submenus to their parents.
+*/
+
+/*! \fn void QPopupMenu::activatedRedirect( int id )
+  \internal
+  Used internally to connect submenus to their parents.
+*/
 
 void QPopupMenu::subActivated( int id )
 {
@@ -592,8 +746,9 @@ int QPopupMenu::itemPos( int index )		// get y coord for item
 
 void QPopupMenu::updateSize()
 {
-    int height	  = 0;
-    int max_width = 10;
+    int height	     = 0;
+    int max_width    = 10;
+    int max_pm_width = 0;
     GUIStyle gs	  = style();
     QFontMetrics fm = fontMetrics();
 #if 0
@@ -604,17 +759,29 @@ void QPopupMenu::updateSize()
     bool hasSubMenu = FALSE;
     int cellh = fm.height() + 2*motifItemVMargin + 2*motifItemFrame;
     int tab_width = 0;
+    bool hasTextAndPixmapItem = FALSE;
     while ( (mi=it.current()) ) {
 	int w = 0;
+	int itemHeight = 0;
 	if ( mi->popup() )
 	    hasSubMenu = TRUE;
 	if ( mi->isSeparator() )
-	    height += motifSepHeight;
+	    itemHeight = motifSepHeight;
 	else if ( mi->pixmap() ) {
-	    height += mi->pixmap()->height() + 2*motifItemFrame;
-	    w = mi->pixmap()->width();
-	} else if ( mi->text() ) {
-	    height += cellh;
+	    itemHeight = mi->pixmap()->height() + 2*motifItemFrame;
+	    if ( mi->text() ) {
+		if ( gs == MotifStyle )
+		    itemHeight += 2;		// Room for check rectangle
+		hasTextAndPixmapItem = TRUE; // has text, w will be set below
+		if ( mi->pixmap()->width() > max_pm_width )
+		    max_pm_width = mi->pixmap()->width();
+	    } else {
+		w = mi->pixmap()->width();	// pixmap only
+	    }
+	}
+	if ( mi->text() && !mi->isSeparator() ) {
+	    if ( itemHeight < cellh )
+		itemHeight = cellh;
 	    const char *s = mi->text();
 	    const char *t;
 	    if ( (t=strchr(s, '\t')) ) {	// string contains tab
@@ -623,17 +790,32 @@ void QPopupMenu::updateSize()
 		if ( tw > tab_width )
 		    tab_width = tw;
 	    } else {
-		w = fm.width( s );
+		w += fm.width( s );
 	    }
 	}
+	height += itemHeight;
 #if defined(CHECK_NULL)
-	else
+	if ( !mi->text() && !mi->pixmap() && !mi->isSeparator() )
 	    warning( "QPopupMenu: Popup has invalid menu item" );
 #endif
 	if ( max_width < w )
 	    max_width = w;
 	++it;
     }
+    if ( hasTextAndPixmapItem ) {
+	QPopupMenuExtra *x = makePMDict( this );
+	x->hasDoubleItem = TRUE;
+	x->maxPMWidth = max_pm_width;
+    } else {
+	QPopupMenuExtra *x = lookInPMDict( this );
+	if ( x ) {
+	    x->hasDoubleItem = FALSE;
+	    x->maxPMWidth    = 0;
+	}
+    }
+    if ( gs == MotifStyle ) {
+	setCheckableFlag( isCheckable() || hasTextAndPixmapItem );
+    }
     int extra_width = 0;
     if ( tab_width ) {
 	extra_width = tab_width + motifTabSpacing;
@@ -645,7 +827,7 @@ void QPopupMenu::updateSize()
     max_width  += 2*motifItemHMargin;
 
     if ( isCheckable() )
-	max_width += getWidthOfCheckCol( gs ) + motifItemFrame;
+	max_width += getWidthOfCheckCol( this, gs ) + motifItemFrame;
     else
 	max_width += 2*motifItemFrame;
 
@@ -767,8 +949,30 @@ void QPopupMenu::hide()
     hidePopups();
     killTimers();
     QWidget::hide();
+    if ( syncMenu == this && qApp ) {
+	syncMenu = 0;
+	qApp->exit_loop();
+    }
 }
 
+#if 0
+/*!
+  Reimplements QWidget::setEnabled() for internal purposes.
+*/
+
+void QPopupMenu::setEnabled( bool enable )
+{
+    if ( enable == isEnabled() )
+	return;
+    if ( parentMenu ) {
+	QMenuItem *mi = parentMenu->findPopup( this );
+	if ( mi ) {
+	    parentMenu->setItemEnabled( mi->id(), enable );
+	}
+    }
+    QWidget::setEnabled( enable );
+}
+#endif
 
 /*****************************************************************************
   Implementation of virtual QTableView functions
@@ -777,12 +981,20 @@ void QPopupMenu::hide()
 int QPopupMenu::cellHeight( int row )
 {
     QMenuItem *mi = mitems->at( row );
-    int h;
-    if ( mi->isSeparator() )			// separator height
+    int h = 0;
+    if ( mi->isSeparator() ) {			// separator height
 	h = motifSepHeight;
-    else if ( mi->pixmap() )			// pixmap height
+    } else if ( mi->pixmap() ) {		// pixmap height
 	h = mi->pixmap()->height() + 2*motifItemFrame;
-    else {					// text height
+	if ( mi->text() ) {			// pixmap and text
+	    if ( style() == MotifStyle )
+		h += 2;				// Room for check rectangle
+	    QFontMetrics fm = fontMetrics();
+	    int h2 = fm.height() + 2*motifItemVMargin + 2*motifItemFrame;
+	    if ( h2 > h )
+		h = h2;
+	}
+    } else {					// text height
 	QFontMetrics fm = fontMetrics();
 	h = fm.height() + 2*motifItemVMargin + 2*motifItemFrame;
     }
@@ -795,9 +1007,9 @@ int QPopupMenu::cellWidth( int col )
 {
     if ( isCheckable() ) {
 	if ( col == 0 )
-	    return getWidthOfCheckCol(style());
+	    return getWidthOfCheckCol(this,style());
 	else
-	    return width() - ( 2*frameWidth() + getWidthOfCheckCol(style()) );
+	    return width() - (2*frameWidth()+getWidthOfCheckCol(this,style()));
     }	    
     else
 	return width() - 2*frameWidth();	
@@ -813,6 +1025,9 @@ void QPopupMenu::paintCell( QPainter *p, int row, int col )
     GUIStyle gs	  = style();
     bool act	  = row == actItem;
     bool dis	  = (selfItem && !selfItem->isEnabled()) || !mi->isEnabled();
+    QColorGroup itemg = dis ? palette().disabled()
+			: act ? palette().active()
+			: palette().normal();
 
     if ( !mi->isDirty() )
 	return;
@@ -827,14 +1042,87 @@ void QPopupMenu::paintCell( QPainter *p, int row, int col )
 	    p->drawLine( 0, 1, rw, 1 );
 	    return;
 	}
+
+	int cm = gs == MotifStyle ? 2 : 0;	// checkable margin
+
+	if ( mi->isChecked() ) {
+	    if ( gs == WindowsStyle && act && !dis ) {
+		QBrush b( g.mid() );
+		qDrawShadePanel( p, cm, cm, cellw-2*cm, cellh-2*cm,
+				 g, TRUE, 1, &b );
+	    } else if ( gs == WindowsStyle || 
+			mi->pixmap() && mi->text() ) {
+		QBrush b( g.background() );
+		qDrawShadePanel( p, cm, cm, cellw-2*cm, cellh-2*cm,
+				 g, TRUE, 1, &b );
+	    }
+	} else if ( !act ) {
+	    qDrawPlainRect( p, cm, cm, cellw-2*cm, cellh-2*cm, 
+			    g.background(), 1, 0 );
+	}		
+
+	if ( mi->text() && mi->pixmap() ) {		// draw pixmap
+	    QPixmap *pixmap = mi->pixmap();
+	    int pixw = pixmap->width();
+	    int pixh = pixmap->height();
+	    if ( gs == MotifStyle ) {
+		if ( act && !dis )			// active item frame
+		    qDrawShadePanel( p, 0, 0, rw, cellh, g, FALSE,
+				     motifItemFrame );
+		else				// incognito frame
+		    qDrawPlainRect( p, 0, 0, rw, cellh, g.background(),
+				    motifItemFrame );
+	    } else {
+		if ( act && !dis ) {
+		    if ( !mi->isChecked() )
+			qDrawShadePanel( p, 0, 0, cellw, cellh, g, FALSE, 1 );
+		}
+	    }
+	    QRect cr( cm, cm, cellw-2*cm, cellh-2*cm );
+	    QRect pmr( 0, 0, pixw, pixh );
+	    pmr.moveCenter( cr.center() );
+	    if ( style() == WindowsStyle && dis ) {
+		QString k;
+		k.sprintf( "$qt-drawitem-%x", pixmap->serialNumber() );
+		QPixmap * mask = QPixmapCache::find(k);
+		if ( !mask ) {
+		    if ( pixmap->mask() )
+			mask = new QPixmap( *pixmap->mask() );
+		    else
+			mask = new QPixmap( pixmap->createHeuristicMask() );
+		    mask->setMask( *((QBitmap*)mask) );
+		    QPixmapCache::insert( k, mask );
+		}
+		p->setPen( itemg.light() );
+		p->drawPixmap( pmr.left()+1, pmr.top()+1, *mask );
+		p->setPen( itemg.text() );
+		p->drawPixmap( pmr.topLeft(), *mask );
+	    } else {
+		p->setPen( itemg.text() );
+		p->drawPixmap( pmr.topLeft(), *pixmap );
+	    }
+	    if ( gs == WindowsStyle ) {
+		p->fillRect( cellw + 1, 0, rw - cellw - 1, cellh, 
+			     act ? QApplication::winStyleHighlightColor()
+			     : g.background());
+	    }
+	    return;
+	}
         
 	int pw = motifItemFrame;
 	if ( gs != MotifStyle )
 	    pw = 1;
 	if ( gs == WindowsStyle ) {
-	    p->fillRect( 0, 0, rw, cellh, act ? darkBlue : g.background() );
+	    if ( mi->isChecked() )
+		p->fillRect( cellw + 1, 0, rw - cellw - 1, cellh, 
+			     act ? QApplication::winStyleHighlightColor()
+			     : g.background() );
+	    else
+		p->fillRect( 0, 0, rw, cellh, 
+			     act ? QApplication::winStyleHighlightColor()
+			     : g.background() );
 	} else if ( gs == MotifStyle ) {
-	    if ( act )				// active item frame
+	    if ( act && !dis )			// active item frame
 		qDrawShadePanel( p, 0, 0, rw, cellh, g, FALSE, pw );
 	    else				// incognito frame
 		qDrawPlainRect( p, 0, 0, rw, cellh, g.background(), pw );
@@ -845,7 +1133,7 @@ void QPopupMenu::paintCell( QPainter *p, int row, int col )
 	    int mh = cellh - 2*motifItemFrame;
 	    if ( mi->isChecked() ) {
 		qDrawCheckMark( p, motifItemFrame + motifCheckMarkHMargin,
-				motifItemFrame, mw, mh, g, gs, act );
+				motifItemFrame, mw, mh, itemg, gs, act, dis );
 	    }
 	    return;
 	}
@@ -858,26 +1146,19 @@ void QPopupMenu::paintCell( QPainter *p, int row, int col )
 
     QColor discol;
     if ( dis ) {
-	discol = palette().disabled().text();
+	discol = itemg.text();
 	p->setPen( discol );
     }
 
     int x = motifItemHMargin + ( isCheckable() ? 0 : motifItemFrame);
-    if ( mi->pixmap() ) {			// draw pixmap
-	QPixmap *pixmap = mi->pixmap();
-	if ( pixmap->depth() == 1 )
-	    p->setBackgroundMode( OpaqueMode );
-	p->drawPixmap( x, motifItemFrame, *pixmap );
-	if ( pixmap->depth() == 1 )
-	    p->setBackgroundMode( TransparentMode );
-    } else if ( mi->text() ) {			// draw text
+    if ( mi->text() ) {			// draw text
 	const char *s = mi->text();
 	const char *t = strchr( s, '\t' );
 	int m = motifItemVMargin;
 	const int text_flags = AlignVCenter|ShowPrefix | DontClip | SingleLine;
 	if ( t ) {				// draw text before tab
 	    if ( gs == WindowsStyle && dis && !act ) {
-		p->setPen( white );
+		p->setPen( g.light() );
 		p->drawText( x+1, m+1, cellw, cellh-2*m, text_flags,
 			     s, (int)((long)t-(long)s) );
 		p->setPen( discol );
@@ -888,23 +1169,35 @@ void QPopupMenu::paintCell( QPainter *p, int row, int col )
 	    x = tabMark();
 	}
 	if ( gs == WindowsStyle && dis && !act ) {
-	    p->setPen( white );
+	    p->setPen( g.light() );
 	    p->drawText( x+1, m+1, cellw, cellh-2*m, text_flags, s );
 	    p->setPen( discol );
 	}
 	p->drawText( x, m, cellw, cellh-2*m, text_flags, s );
-    }
+    } else if ( mi->pixmap() ) {			// draw pixmap
+	QPixmap *pixmap = mi->pixmap();
+	if ( pixmap->depth() == 1 )
+	    p->setBackgroundMode( OpaqueMode );
+	p->drawPixmap( x, motifItemFrame, *pixmap );
+	if ( pixmap->depth() == 1 )
+	    p->setBackgroundMode( TransparentMode );
+    } 
     if ( mi->popup() ) {			// draw sub menu arrow
 	int dim = (cellh-2*motifItemFrame) / 2;
 	if ( gs == WindowsStyle && row == actItem ) {
 	    if ( !dis )
 		discol = white;
-	    g = QColorGroup( discol, darkBlue, white, white, white,
+	    g = QColorGroup( discol, QApplication::winStyleHighlightColor(),
+			     white, white,
+			     dis ? discol : white,
 			     discol, white );
 	}
-	qDrawArrow( p, RightArrow, gs, row == actItem,
+	qDrawArrow( p, RightArrow, gs,
+		    row == actItem && gs == MotifStyle && mi->isEnabled(),
 		    cellw - motifArrowHMargin - dim,  cellh/2-dim/2,
-		    dim, dim, g );
+		    dim, dim, g,
+		    gs == MotifStyle && mi->isEnabled() ||
+		    gs == WindowsStyle && (mi->isEnabled() || row == actItem));
     }
     mi->setDirty( FALSE );
 }
@@ -983,7 +1276,10 @@ void QPopupMenu::mousePressEvent( QMouseEvent *e )
 
 void QPopupMenu::mouseReleaseEvent( QMouseEvent *e )
 {
-    mouseBtDn = FALSE;				// mouse button up
+    if ( !mouseBtDn && !parentMenu && actItem < 0 )
+	return;
+
+    mouseBtDn = FALSE;
     int item = itemAtPos( e->pos() );
     if ( item == -1 ) {
 	if ( !rect().contains( e->pos() ) && tryMenuBar(e) )
@@ -1030,6 +1326,10 @@ void QPopupMenu::mouseMoveEvent( QMouseEvent *e )
 	if ( !rect().contains( e->pos() ) && !tryMenuBar( e ) )
 	    hidePopups();
     } else {					// mouse on valid item
+	// but did not register mouse press
+	if ( (e->state() & MouseButtonMask) && !mouseBtDn )
+	    mouseBtDn = TRUE; // so mouseReleaseEvent will pop down
+
 	register QMenuItem *mi = mitems->at( item );
 	QPopupMenu *popup = mi->popup();
 	if ( actItem == item ) {
@@ -1061,9 +1361,6 @@ void QPopupMenu::mouseMoveEvent( QMouseEvent *e )
 
 void QPopupMenu::keyPressEvent( QKeyEvent *e )
 {
-    if ( mouseBtDn )				// cannot handle key event
-	return;
-
     QMenuItem  *mi = 0;
     QPopupMenu *popup;
     int d = 0;
@@ -1078,11 +1375,44 @@ void QPopupMenu::keyPressEvent( QKeyEvent *e )
 	d = 1;
 	break;
 
+    case Key_Alt:
+	if ( style() == WindowsStyle ) {
+	    hideAllPopups();
+	    byeMenuBar();
+	}
+	break;
+
     case Key_Escape:
 	hideAllPopups();
 	byeMenuBar();
 	break;
 
+    case Key_Left:
+	if ( parentMenu && parentMenu->isPopupMenu ) {
+	    ((QPopupMenu *)parentMenu)->hidePopups();
+	    killTimers();
+	    startTimer( 20 );
+	} else {
+	    ok_key = FALSE;
+	}
+	break;
+
+    case Key_Right:
+	if ( actItem >= 0 && (popup=mitems->at( actItem )->popup()) ) {
+	    hidePopups();
+	    killTimers();
+	    startTimer( 20 );
+	    popup->setFirstItemActive();
+	} else {
+	    ok_key = FALSE;
+	}
+	break;
+
+    case Key_Space:
+	if ( style() != MotifStyle )
+	    break;
+	// for motif, fall through
+
     case Key_Return:
     case Key_Enter:
 	if ( actItem < 0 )
@@ -1110,11 +1440,12 @@ void QPopupMenu::keyPressEvent( QKeyEvent *e )
 
     }
 #if 1
-    if ( !ok_key && !e->state() && e->key() >= Key_A && e->key() <= Key_Z ) {
-	char c = 'A' + e->key() - Key_A;
+    if ( !ok_key && !e->state() && e->key() >= Key_0 && e->key() <= Key_Z ) {
+	char c = '0' + e->key() - Key_0;
 
 	QMenuItemListIt it(*mitems);
 	register QMenuItem *m;
+	int indx = 0;
 	while ( (m=it.current()) ) {
 	    ++it;
 	    QString s = m->text();
@@ -1129,14 +1460,24 @@ void QPopupMenu::keyPressEvent( QKeyEvent *e )
 		    }
 		}
 	    }
+	    indx++;
 	}
 	if ( mi ) {
-	    hideAllPopups();
-	    byeMenuBar();
-	    if ( mi->isEnabled() ) {
-		if ( mi->signal() )
-		    mi->signal()->activate();
-		actSig( mi->id() );
+	    popup = mi->popup();
+	    if ( popup ) {
+		setActiveItem( indx );
+		hidePopups();
+		killTimers();
+		startTimer( 20 );
+		popup->setFirstItemActive();
+	    } else {
+		hideAllPopups();
+		byeMenuBar();
+		if ( mi->isEnabled() ) {
+		    if ( mi->signal() )
+			mi->signal()->activate();
+		    actSig( mi->id() );
+		}
 	    }
 	}
     }
@@ -1149,7 +1490,9 @@ void QPopupMenu::keyPressEvent( QKeyEvent *e )
 	    ((QMenuBar*)top)->tryKeyEvent( this, e );
     }
 
-    if ( d && actItem >= 0 ) {			// highlight next/prev
+    if ( d && actItem < 0 ) {
+	setFirstItemActive();
+    } else if ( d ) {				// highlight next/prev
 	register int i = actItem;
 	int c = mitems->count();
 	int n = c;
@@ -1160,7 +1503,8 @@ void QPopupMenu::keyPressEvent( QKeyEvent *e )
 	    else if ( i < 0 )
 		i = c - 1;
 	    mi = mitems->at( i );
-	    if ( mi->isEnabled() && !mi->isSeparator() )
+	    if ( !mi->isSeparator() 
+		 && ( style() != MotifStyle || mi->isEnabled() ) )
 		break;
 	}
 	if ( i != actItem ) {
@@ -1181,17 +1525,20 @@ void QPopupMenu::keyPressEvent( QKeyEvent *e )
 
 void QPopupMenu::timerEvent( QTimerEvent *e )
 {
-    killTimer( e->timerId() );			// single-shot timer
+    killTimer( e->timerId() );			// ### single-shot timer
     if ( actItem < 0 )
 	return;
-    QPopupMenu *popup = mitems->at(actItem)->popup();
-    if ( popup ) {				// it is a popup
-	QPoint pos( width() - motifArrowHMargin,
-		    frameWidth() + motifArrowVMargin );
-	for ( int i=0; i<actItem; i++ )
-	    pos.ry() += (QCOORD)cellHeight( i );
-	popupActive = actItem;
-	popup->popup( mapToGlobal(pos) );
+    QMenuItem *mi = mitems->at(actItem);
+    if ( mi->isEnabled() ) {
+	QPopupMenu *popup = mi->popup();
+	if ( popup ) {				// it is a popup
+	    QPoint pos( width() - motifArrowHMargin,
+			frameWidth() + motifArrowVMargin );
+	    for ( int i=0; i<actItem; i++ )
+		pos.ry() += (QCOORD)cellHeight( i );
+	    popupActive = actItem;
+	    popup->popup( mapToGlobal(pos) );
+	}
     }
 }
 
@@ -1202,3 +1549,54 @@ void QPopupMenu::updateRow( int row )
     if ( isCheckable() )
 	updateCell( row, 1, FALSE );
 }
+
+
+/*!  Execute this popup synchronously.
+
+  The return code is the ID of the selected item, or -1 if no item is
+  selected (normally because the user presses Escape).
+
+  Note that all signals are emitted as usual.  If you connect a menu
+  item to a slot and call the menu's exec(), you get the result both
+  via the signal-slot connection and in the return value of exec().
+*/
+
+int QPopupMenu::exec()
+{
+    if ( !qApp )
+	return -1;
+
+    syncMenu = this;
+    syncMenuId = -1;
+    connect( this, SIGNAL(activated(int)),
+	     this, SLOT(modalActivation(int)) );
+    show();
+    qApp->enter_loop();
+    disconnect( this, SIGNAL(activated(int)),
+		this, SLOT(modalActivation(int)) );
+    syncMenu = 0;
+
+    return syncMenuId;
+}
+
+
+/*!  Internal slot used for exec(). */
+
+void QPopupMenu::modalActivation( int id )
+{
+    syncMenuId = id;
+}
+
+
+/*!  Sets the currently active item to \a i and repaints as necessary.
+*/
+
+void QPopupMenu::setActiveItem( int i )
+{
+    int lastActItem = actItem;
+    actItem = i;
+    if ( lastActItem >= 0 )
+	updateRow( lastActItem );
+    if ( i >= 0 && i != lastActItem )
+	updateRow( i );
+}
