@@ -1,5 +1,5 @@
 /****************************************************************************
-** $Id: qpushbt.cpp,v 2.14 1997/03/25 16:26:05 hanord Exp $
+** $Id: qpushbt.cpp,v 2.27 1997/09/29 11:34:51 eiriken Exp $
 **
 ** Implementation of QPushButton class
 **
@@ -7,13 +7,16 @@
 **
 ** Copyright (C) 1992-1997 Troll Tech AS.  All rights reserved.
 **
-** This file is part of the non-commercial distribution of Qt 1.2.
+** This file is part of the free edition of Qt 1.33.
 **
 ** See the file LICENSE included in the distribution for the usage
 ** and distribution terms, or http://www.troll.no/qt/license.html.
 **
-** (This file differs from the one in the commercial distribution of
-** Qt only by this comment.)
+** IMPORTANT NOTE: You may NOT copy this file or any part of it into
+** your own programs or libraries.
+**
+** (This file differs from the one in the commercial edition of Qt
+** only by this comment.)
 **
 *****************************************************************************/
 
@@ -26,7 +29,7 @@
 #include "qpmcache.h"
 #include "qbitmap.h"
 
-RCSTAG("$Id: qpushbt.cpp,v 2.14 1997/03/25 16:26:05 hanord Exp $");
+RCSTAG("$Id: qpushbt.cpp,v 2.27 1997/09/29 11:34:51 eiriken Exp $");
 
 
 /*!
@@ -41,6 +44,7 @@ RCSTAG("$Id: qpushbt.cpp,v 2.14 1997/03/25 16:26:05 hanord Exp $");
   A push button has \c TabFocus as a default focusPolicy(), i.e. it can 
   get keyboard focus by tabbing but not by clicking.
 
+  <img src=qpushbt-m.gif> <img src=qpushbt-w.gif>
 */
 
 const int extraMotifWidth = 10;
@@ -204,9 +208,7 @@ void QPushButton::setDefault( bool enable )
 {
     if ( (defButton && enable) || !(defButton || enable) )
 	return;					// no change
-    QWidget *p = this;
-    while ( p && p->parentWidget() )		// get the top level parent
-	p = p->parentWidget();
+    QWidget *p = topLevelWidget();
     if ( !p->inherits("QDialog") )		// not a dialog
 	return;
     defButton = enable;
@@ -234,8 +236,11 @@ QSize QPushButton::sizeHint() const
 	w = pm->width()	 + 6;
 	h = pm->height() + 6;
     } else {
+	QString s( text() );
+	if ( s.isEmpty() )
+	    s = "XXXX";
 	QFontMetrics fm = fontMetrics();
-	QRect br = fm.boundingRect( text() );
+	QRect br = fm.boundingRect( s );
 	w = br.width()	+ 6;
 	h = fm.height() + 6;
 	w += h/8 + 10;
@@ -316,69 +321,19 @@ void QPushButton::drawButton( QPainter *paint )
     register QPainter *p = paint;
     GUIStyle	gs = style();
     QColorGroup g  = colorGroup();
-    bool	updated = isDown() != (bool)lastDown ||
-			  lastDef != defButton ||
-			  isEnabled() != (bool)lastEnabled;
-    QColor	fillcol = g.background();
+    bool	updated = ( isDown() != (bool)lastDown ||
+			    lastDef != defButton ||
+			    isEnabled() != (bool)lastEnabled);
+
     int		x1, y1, x2, y2;
 
     rect().coords( &x1, &y1, &x2, &y2 );	// get coordinates
 
-#define SAVE_PUSHBUTTON_PIXMAPS
-#if defined(SAVE_PUSHBUTTON_PIXMAPS)
-    QString pmkey;				// pixmap key
-    int kf = 0;
-    if ( isDown() )
-	kf |= 1;
-    if ( isOn() )
-	kf |= 2;
-    if ( isEnabled() )
-	kf |= 4;
-    if ( isToggleButton() )
-	kf |= 8;
-    if ( isDefault() )
-	kf |= 16;
     int w = x2 + 1;
     int h = y2 + 1;
-    pmkey.sprintf( "$qt_push_%d_%d_%d_%d_%d", gs, palette().serialNumber(),
-		   kf, w, h );
-    QPixmap *pm = QPixmapCache::find( pmkey );
-    QPainter pmpaint;
-    if ( pm ) {					// pixmap exists
-	QPixmap pm_direct = *pm;
-	pmpaint.begin( &pm_direct );
-	pmpaint.drawPixmap( 0, 0, *pm );
-	if ( text() )
-	    pmpaint.setFont( font() );
-	drawButtonLabel( &pmpaint );
-	pmpaint.end();
-	p->drawPixmap( 0, 0, pm_direct );
-	lastDown = isDown();
-	lastDef = defButton;
-	lastEnabled = isEnabled();
-	if ( hasFocus() ) {
-	    if ( style() == WindowsStyle ) {
-		p->drawWinFocusRect( x1+3, y1+3, x2-x1-5, y2-y1-5 );
-	    } else {
-		p->setPen( black );
-		p->drawRect( x1+3, y1+3, x2-x1-5, y2-y1-5 );
-	    }
-	}
-	return;
-    }
-    bool use_pm = TRUE;
-    if ( use_pm ) {
-	pm = new QPixmap( w, h );		// create new pixmap
-	CHECK_PTR( pm );
-	pmpaint.begin( pm );
-	p = &pmpaint;				// draw in pixmap
-	p->setBackgroundColor( fillcol );
-	p->eraseRect( 0, 0, w, h );
-    }
-#endif
 
     p->setPen( g.foreground() );
-    p->setBrush( QBrush(fillcol,NoBrush) );
+    p->setBrush( QBrush(g.background(),NoBrush) );
 
     if ( gs == WindowsStyle ) {		// Windows push button
 	if ( isDown() ) {
@@ -405,8 +360,9 @@ void QPushButton::drawButton( QPainter *paint )
 		qDrawWinButton( p, x1, y1, x2-x1+1, y2-y1+1, g, isOn() );
 	    }
 	}
+	// ### next two lines ignore backgroundPixmap()
 	if ( updated )
-	    p->fillRect( x1+1, y1+1, x2-x1-2, y2-y1-2, g.background() );
+	    p->fillRect( x1+2, y1+2, x2-x1-3, y2-y1-3, g.background() );
     } else if ( gs == MotifStyle ) {		// Motif push button
 	if ( defButton ) {			// default Motif button
 	    qDrawShadePanel( p, x1, y1, x2-x1+1, y2-y1+1, g, TRUE );
@@ -415,35 +371,33 @@ void QPushButton::drawButton( QPainter *paint )
 	    x2 -= extraMotifWidth/2;
 	    y2 -= extraMotifHeight/2;
 	}
-	QBrush fill( fillcol );
-	if ( isDown() ) {
-	    qDrawShadePanel( p, x1, y1, x2-x1+1, y2-y1+1, g, TRUE, 2,
-			     updated ? &fill : 0 );
-	} else if ( isToggleButton() && isOn() ) {
-	    QBrush fill( g.mid() );
-	    qDrawShadePanel( p, x1, y1, x2-x1+1, y2-y1+1, g, TRUE, 2,
-			     updated ? &fill : 0 );
-	} else {
-	    qDrawShadePanel( p, x1, y1, x2-x1+1, y2-y1+1, g, FALSE, 2,
-			     updated ? &fill : 0 );
-	}
+
+	QBrush fill;
+	if ( isDown() )
+	    fill = QBrush( g.mid() );
+	else if ( isOn() )
+	    fill = QBrush( g.mid(), Dense4Pattern );
+	else
+	    fill = QBrush( g.background() );
+
+	qDrawShadePanel( p, x1, y1, x2-x1+1, y2-y1+1, g, isOn() || isDown(),
+			 2, &fill );
     }
     if ( p->brush().style() != NoBrush )
 	p->setBrush( NoBrush );
 
-#if defined(SAVE_PUSHBUTTON_PIXMAPS)
-    if ( use_pm ) {
-	pmpaint.end();
-	p = paint;				// draw in default device
-	p->drawPixmap( 0, 0, *pm );
-	QPixmapCache::insert( pmkey, pm );	// save for later use
-    }
-#endif
     drawButtonLabel( p );
-    if ( gs == MotifStyle && hasFocus() ) {
-	p->setPen( black );
-	p->drawRect( x1+3, y1+3, x2-x1-5, y2-y1-5 );
+
+    if ( hasFocus() ) {
+	if ( style() == WindowsStyle ) {
+	    p->drawWinFocusRect( x1+3, y1+3, x2-x1-5, y2-y1-5, 
+				 backgroundColor() );
+	} else {
+	    p->setPen( black );
+	    p->drawRect( x1+3, y1+3, x2-x1-5, y2-y1-5 );
+	}
     }
+
     lastDown = isDown();
     lastDef = defButton;
     lastEnabled = isEnabled();
@@ -458,31 +412,20 @@ void QPushButton::drawButton( QPainter *paint )
 void QPushButton::drawButtonLabel( QPainter *paint )
 {
     register QPainter *p = paint;
-    GUIStyle	gs = style();
-    QColorGroup g  = colorGroup();
-    int		dt = 0;
-    switch ( gs ) {
-    case WindowsStyle:
-	dt = 1;
-	break;
-    case MotifStyle:
-	break;
-    default:
-	;
-    }
+
     QRect r = rect();
     int x, y, w, h;
     r.rect( &x, &y, &w, &h );
-    if ( isDown() || isOn() ) {			// shift pixmap/text
-	x += dt;
-	y += dt;
+    if ( (isDown() || isOn()) && style() == WindowsStyle ) {
+        // shift pixmap/text
+	x++;
+	y++;
     }
     x += 2;  y += 2;  w -= 4;  h -= 4;
-    qDrawItem( p, gs, x, y, w, h, 
+    qDrawItem( p, style(), x, y, w, h,
 	       AlignCenter|ShowPrefix,
 	       colorGroup(), isEnabled(),
 	       pixmap(), text() );
-
 }
 
 
