@@ -1,5 +1,5 @@
 /****************************************************************************
-** $Id: qradiobt.cpp,v 2.3 1997/03/25 13:20:43 hanord Exp $
+** $Id: qradiobt.cpp,v 2.19 1997/09/29 11:34:51 eiriken Exp $
 **
 ** Implementation of QRadioButton class
 **
@@ -7,13 +7,16 @@
 **
 ** Copyright (C) 1992-1997 Troll Tech AS.  All rights reserved.
 **
-** This file is part of the non-commercial distribution of Qt 1.2.
+** This file is part of the free edition of Qt 1.33.
 **
 ** See the file LICENSE included in the distribution for the usage
 ** and distribution terms, or http://www.troll.no/qt/license.html.
 **
-** (This file differs from the one in the commercial distribution of
-** Qt only by this comment.)
+** IMPORTANT NOTE: You may NOT copy this file or any part of it into
+** your own programs or libraries.
+**
+** (This file differs from the one in the commercial edition of Qt
+** only by this comment.)
 **
 *****************************************************************************/
 
@@ -24,8 +27,9 @@
 #include "qpixmap.h"
 #include "qpmcache.h"
 #include "qbitmap.h"
+#include "qkeycode.h"
 
-RCSTAG("$Id: qradiobt.cpp,v 2.3 1997/03/25 13:20:43 hanord Exp $");
+RCSTAG("$Id: qradiobt.cpp,v 2.19 1997/09/29 11:34:51 eiriken Exp $");
 
 
 /*!
@@ -40,6 +44,8 @@ RCSTAG("$Id: qradiobt.cpp,v 2.3 1997/03/25 13:20:43 hanord Exp $");
   switched on at a time.
 
   The QButtonGroup widget is very useful for defining groups of radio buttons.
+
+  <img src=qradiobt-m.gif> <img src=qradiobt-w.gif>
 */
 
 
@@ -131,9 +137,8 @@ QSize QRadioButton::sizeHint() const
     getSizeOfBitmap( style(), &wbm, &hbm );
     if ( h < hbm )
 	h = hbm;
-    w += wbm+6;
 
-    return QSize( w, h );
+    return QSize( w + wbm + 8, h + 4 );
 }
 
 
@@ -156,7 +161,7 @@ bool QRadioButton::hitButton( const QPoint &pos ) const
 
 void QRadioButton::drawButton( QPainter *paint )
 {
-    register QPainter *p = paint;
+    QPainter	*p = paint;
     GUIStyle	 gs = style();
     QColorGroup	 g  = colorGroup();
     int		 x, y, w, h;
@@ -221,18 +226,20 @@ void QRadioButton::drawButton( QPainter *paint )
 	p->drawPolyline( a );
 	a.setPoints( QCOORDARRLEN(pts3), pts3 );
 	a.translate( x, y );
-	p->setPen( g.background() );
+	p->setPen( g.midlight() );
 	p->drawPolyline( a );
 	a.setPoints( QCOORDARRLEN(pts4), pts4 );
 	a.translate( x, y );
-	p->setPen( white );
+	p->setPen( g.light() );
 	p->drawPolyline( a );
 	a.setPoints( QCOORDARRLEN(pts5), pts5 );
 	a.translate( x, y );
-	p->setPen( NoPen );
-	p->setBrush( isDown() ? g.background() : g.base() );
+	QColor fillColor = isDown() ? g.background() : g.base();
+	p->setPen( fillColor );
+	p->setBrush( fillColor );
 	p->drawPolygon( a );
 	if ( isOn() ) {
+	    p->setPen( NoPen );
 	    p->setBrush( g.foreground() );
 	    p->drawRect( x+5, y+4, 2, 4 );
 	    p->drawRect( x+4, y+5, 4, 2 );
@@ -268,7 +275,8 @@ void QRadioButton::drawButton( QPainter *paint )
 	p = paint;				// draw in default device
 	p->drawPixmap( wx, wy, *pm );
 	w += wx;
-	QPixmapCache::insert( pmkey, pm );	// save for later use
+	if (!QPixmapCache::insert(pmkey, pm) )	// save in cache
+	    delete pm;
     }
 #endif
     drawButtonLabel( p );
@@ -296,6 +304,27 @@ void QRadioButton::drawButtonLabel( QPainter *p )
 	       AlignLeft|AlignVCenter|ShowPrefix,
 	       colorGroup(), isEnabled(),
 	       pixmap(), text() );
+
+    if ( hasFocus() ) {
+	QRect br = qItemRect( p, gs, x, y, w, h, 
+			      AlignLeft|AlignVCenter|ShowPrefix,
+			      isEnabled(),
+			      pixmap(), text() );
+	br.setLeft( br.left()-3 );
+	br.setRight( br.right()+2 );
+	br.setTop( br.top()-2 );
+	br.setBottom( br.bottom()+2);
+	br = br.intersect( QRect(0,0,width(),height()) );
+
+	if ( gs == WindowsStyle ) {
+	    p->drawWinFocusRect( br, backgroundColor() );
+	} else {
+	    p->setPen( black );
+	    p->drawRect( br );
+	}
+    }
+
+
 }
 
 
@@ -310,3 +339,15 @@ void QRadioButton::mouseReleaseEvent( QMouseEvent *e )
     QButton::mouseReleaseEvent( e );		// send to button handler
     noHit = FALSE;
 }
+
+
+/*!  Reimplemented to ignore toggle-off key events. */
+
+void QRadioButton::keyPressEvent( QKeyEvent * e )
+{
+    if ( isOn() && e->key() == Key_Space ) {
+	e->accept();
+	return;
+    }
+    QButton::keyPressEvent( e );
+}
