@@ -1,5 +1,5 @@
 /****************************************************************************
-** $Id: qradiobt.h,v 2.0 1996/09/24 10:16:21 agulbra Exp $
+** $Id: qradiobt.h,v 2.4 1997/09/29 11:34:52 eiriken Exp $
 **
 ** Definition of QRadioButton class
 **
@@ -7,13 +7,16 @@
 **
 ** Copyright (C) 1992-1997 Troll Tech AS.  All rights reserved.
 **
-** This file is part of the non-commercial distribution of Qt 1.2.
+** This file is part of the free edition of Qt 1.33.
 **
 ** See the file LICENSE included in the distribution for the usage
 ** and distribution terms, or http://www.troll.no/qt/license.html.
 **
-** (This file differs from the one in the commercial distribution of
-** Qt only by this comment.)
+** IMPORTANT NOTE: You may NOT copy this file or any part of it into
+** your own programs or libraries.
+**
+** (This file differs from the one in the commercial edition of Qt
+** only by this comment.)
 **
 *****************************************************************************/
 
@@ -41,6 +44,7 @@ protected:
     void    drawButtonLabel( QPainter * );
 
     void    mouseReleaseEvent( QMouseEvent * );
+    void    keyPressEvent( QKeyEvent * );
 
 private:
     void    init();
