@@ -1,5 +1,5 @@
 /****************************************************************************
-** $Id: qrangect.cpp,v 2.1 1996/11/29 15:00:18 eiriken Exp $
+** $Id: qrangect.cpp,v 2.2 1997/08/06 15:06:01 hanord Exp $
 **
 ** Implementation of QRangeControl class
 **
@@ -7,20 +7,23 @@
 **
 ** Copyright (C) 1992-1997 Troll Tech AS.  All rights reserved.
 **
-** This file is part of the non-commercial distribution of Qt 1.2.
+** This file is part of the free edition of Qt 1.33.
 **
 ** See the file LICENSE included in the distribution for the usage
 ** and distribution terms, or http://www.troll.no/qt/license.html.
 **
-** (This file differs from the one in the commercial distribution of
-** Qt only by this comment.)
+** IMPORTANT NOTE: You may NOT copy this file or any part of it into
+** your own programs or libraries.
+**
+** (This file differs from the one in the commercial edition of Qt
+** only by this comment.)
 **
 *****************************************************************************/
 
 #include "qrangect.h"
 #include "qglobal.h"
 
-RCSTAG("$Id: qrangect.cpp,v 2.1 1996/11/29 15:00:18 eiriken Exp $");
+RCSTAG("$Id: qrangect.cpp,v 2.2 1997/08/06 15:06:01 hanord Exp $");
 
 
 /*!
