@@ -1,5 +1,5 @@
 /****************************************************************************
-** $Id: qscrbar.cpp,v 2.10 1997/03/14 21:20:55 hanord Exp $
+** $Id: qscrbar.cpp,v 2.28 1997/09/30 13:31:45 agulbra Exp $
 **
 ** Implementation of QScrollBar class
 **
@@ -7,23 +7,25 @@
 **
 ** Copyright (C) 1992-1997 Troll Tech AS.  All rights reserved.
 **
-** This file is part of the non-commercial distribution of Qt 1.2.
+** This file is part of the free edition of Qt 1.33.
 **
 ** See the file LICENSE included in the distribution for the usage
 ** and distribution terms, or http://www.troll.no/qt/license.html.
 **
-** (This file differs from the one in the commercial distribution of
-** Qt only by this comment.)
+** IMPORTANT NOTE: You may NOT copy this file or any part of it into
+** your own programs or libraries.
+**
+** (This file differs from the one in the commercial edition of Qt
+** only by this comment.)
 **
 *****************************************************************************/
 
 #include "qscrbar.h"
 #include "qpainter.h"
-#include "qdrawutl.h"
 #include "qbitmap.h"
 #include "qkeycode.h"
 
-RCSTAG("$Id: qscrbar.cpp,v 2.10 1997/03/14 21:20:55 hanord Exp $");
+RCSTAG("$Id: qscrbar.cpp,v 2.28 1997/09/30 13:31:45 agulbra Exp $");
 
 
 /*!
@@ -45,6 +47,8 @@ RCSTAG("$Id: qscrbar.cpp,v 2.10 1997/03/14 21:20:55 hanord Exp $");
   enable keyboard focus.
 
   \ingroup realwidgets
+
+  <img src=qscrbar-m.gif> <img src=qscrbar-w.gif>
 */
 
 
@@ -61,8 +65,13 @@ RCSTAG("$Id: qscrbar.cpp,v 2.10 1997/03/14 21:20:55 hanord Exp $");
 
 /*!
   \fn void QScrollBar::sliderMoved( int value )
-  This signal is emitted when the slider is dragged, with the
-  new scroll bar value as an argument.
+
+  This signal is emitted when the slider is moved by the user, with
+  the new scroll bar value as an argument.
+
+  This signal is emitted even when tracking is turned off.
+
+  \sa tracking() valueChanged() nextLine() prevLine() nextPage() prevPage()
 */
 
 /*!
@@ -119,7 +128,7 @@ public:
 #define PRIV	((QScrollBar_Private *)this)
 
 static const int thresholdTime = 500;
-static const int repeatTime	= 100;
+static const int repeatTime	= 10;
 
 #define HORIZONTAL	(orientation() == Horizontal)
 #define VERTICAL	!HORIZONTAL
@@ -187,7 +196,9 @@ void QScrollBar::init()
     clickedAt	     = FALSE;
     setFocusPolicy( NoFocus );
     if ( style() == MotifStyle )
-	setBackgroundColor( colorGroup().mid() );
+	setBackgroundMode( PaletteMid );
+    else
+	setBackgroundMode( PaletteBackground );
 }
 
 
@@ -256,7 +267,9 @@ void QScrollBar::setPalette( const QPalette &p )
 {
     QWidget::setPalette( p );
     if ( style() == MotifStyle )
-	setBackgroundColor( colorGroup().mid() );
+	setBackgroundMode( PaletteMid );
+    else
+	setBackgroundMode( PaletteBackground );
 }
 
 
@@ -308,7 +321,8 @@ void QScrollBar::stepChange()
 void QScrollBar::rangeChange()
 {
     positionSliderFromValue();
-    PRIV->drawControls( ADD_PAGE | SLIDER | SUB_PAGE, pressedControl );
+    PRIV->drawControls( ADD_LINE | ADD_PAGE | SLIDER | SUB_PAGE | SUB_LINE,
+			pressedControl );
 }
 
 
@@ -339,19 +353,27 @@ void QScrollBar::keyPressEvent( QKeyEvent *e )
     switch ( e->key() ) {
     case Key_Left:
 	if ( orient == Horizontal )
-	    setValue( value() - 1 );
+	    setValue( value() - lineStep() );
 	break;
     case Key_Right:
 	if ( orient == Horizontal )
-	    setValue( value() + 1 );
+	    setValue( value() + lineStep() );
 	break;
     case Key_Up:
 	if ( orient == Vertical )
-	    setValue( value() - 1 );
+	    setValue( value() - lineStep() );
 	break;
     case Key_Down:
 	if ( orient == Vertical )
-	    setValue( value() + 1 );
+	    setValue( value() + lineStep() );
+	break;
+    case Key_PageUp:
+	if ( orient == Vertical )
+	    setValue( value() - pageStep() );
+	break;
+    case Key_PageDown:
+	if ( orient == Vertical )
+	    setValue( value() + pageStep() );
 	break;
     case Key_Home:
 	setValue( minValue() );
@@ -380,10 +402,12 @@ void QScrollBar::resizeEvent( QResizeEvent * )
   Handles paint events for the scroll bar.
 */
 
-void QScrollBar::paintEvent( QPaintEvent * )
+void QScrollBar::paintEvent( QPaintEvent *event )
 {
-    QPainter p;
-    p.begin( this );
+    QPainter p( this );
+    if ( event )
+	p.setClipRect( event->rect() );
+
     qDrawShadePanel( &p, rect(), colorGroup(), TRUE );
     if ( hasFocus() ) {
 	if ( style() != WindowsStyle ) {
@@ -393,7 +417,6 @@ void QScrollBar::paintEvent( QPaintEvent * )
     }
     PRIV->drawControls( ADD_LINE | SUB_LINE | ADD_PAGE | SUB_PAGE | SLIDER,
 			pressedControl, &p );
-    p.end();
 }
 
 
@@ -405,27 +428,42 @@ static QCOORD sliderStartPos = 0;
 
 void QScrollBar::mousePressEvent( QMouseEvent *e )
 {
-    if ( e->button() != LeftButton )
+    if ( !(e->button() == LeftButton ||
+	   (style() == MotifStyle && e->button() == MidButton) ) )
 	return;
+
+    if ( maxValue() == minValue() ) // nothing to be done
+	return;
+
     clickedAt	   = TRUE;
     pressedControl = PRIV->pointOver( e->pos() );
-    switch( pressedControl ) {
-	case SLIDER:
-	    clickOffset = (QCOORD)( (HORIZONTAL ? e->pos().x() : e->pos().y())
-				    - sliderPos );
-	    slidePrevVal   = value();
-	    sliderStartPos = sliderPos;
-	    emit sliderPressed();
-	    break;
-	case NONE:
-	    break;
-	default:
-	    PRIV->drawControls( pressedControl, pressedControl );
-	    PRIV->action( (ScrollControl) pressedControl );
-	    thresholdReached = FALSE;	// wait before starting repeat
-	    startTimer(thresholdTime);
-	    isTiming = TRUE;
-	    break;
+
+    if ( (pressedControl == ADD_PAGE ||
+	  pressedControl == SUB_PAGE ||
+	  pressedControl == SLIDER ) &&
+	 style() == MotifStyle &&
+	 e->button() == MidButton ) {
+	int dummy1, dummy2, sliderLength;
+	PRIV->metrics( &dummy1, &dummy2, &sliderLength );
+	int newSliderPos = (HORIZONTAL ? e->pos().x() : e->pos().y())
+			   - sliderLength/2;
+	setValue( PRIV->sliderPosToRangeValue(newSliderPos) );
+	sliderPos = newSliderPos;
+	pressedControl = SLIDER;
+    }
+
+    if ( pressedControl == SLIDER ) {
+	clickOffset = (QCOORD)( (HORIZONTAL ? e->pos().x() : e->pos().y())
+				- sliderPos );
+	slidePrevVal   = value();
+	sliderStartPos = sliderPos;
+	emit sliderPressed();
+    } else if ( pressedControl != NONE ) {
+	PRIV->drawControls( pressedControl, pressedControl );
+	PRIV->action( (ScrollControl) pressedControl );
+	thresholdReached = FALSE;	// wait before starting repeat
+	startTimer(thresholdTime);
+	isTiming = TRUE;
     }
 }
 
@@ -436,7 +474,8 @@ void QScrollBar::mousePressEvent( QMouseEvent *e )
 
 void QScrollBar::mouseReleaseEvent( QMouseEvent *e )
 {
-    if ( e->button() != LeftButton || !clickedAt )
+    if ( !clickedAt || !(e->button() == LeftButton ||
+			 (style() == MotifStyle && e->button() == MidButton)) )
 	return;
     ScrollControl tmp = (ScrollControl) pressedControl;
     clickedAt = FALSE;
@@ -472,7 +511,8 @@ void QScrollBar::mouseMoveEvent( QMouseEvent *e )
 	clickedAt = FALSE;
 	return;
     }
-    if ( !(e->state() & LeftButton) || !clickedAt )
+    if ( !clickedAt || !(e->state() & LeftButton ||
+			 ((e->state() & MidButton) && style() == MotifStyle)) )
 	return;
     int newSliderPos;
     if ( pressedControl == SLIDER ) {
@@ -660,10 +700,8 @@ void QScrollBar_Private::action( ScrollControl control )
 void QScrollBar_Private::drawControls( uint controls,
 				       uint activeControl ) const
 {
-    QPainter p;
-    p.begin( this );
+    QPainter p ( this );
     drawControls( controls, activeControl, &p );
-    p.end();
 }
 
 
@@ -715,245 +753,77 @@ void QScrollBar_Private::drawControls( uint controls, uint activeControl,
 	sliderR .setRect( b, sliderStart(), sliderW, sliderLength );
     }
 
-    switch ( style() ) {
-	case WindowsStyle:
-	    if ( controls & ADD_LINE ) {
-		qDrawWinPanel( p, addB.x(), addB.y(),
-			       addB.width(), addB.height(), g,
-			       ADD_LINE_ACTIVE );
-		qDrawArrow( p, VERTICAL ? DownArrow : RightArrow,WindowsStyle,
-			    ADD_LINE_ACTIVE, addB.x()+2, addB.y()+2,
-			    addB.width()-4, addB.height()-4, g );
-	    }
-	    if ( controls & SUB_LINE ) {
-		qDrawWinPanel( p, subB.x(), subB.y(),
-			       subB.width(), subB.height(), g,
-			       SUB_LINE_ACTIVE );
-		qDrawArrow( p, VERTICAL ? UpArrow : LeftArrow, WindowsStyle,
-			    SUB_LINE_ACTIVE, subB.x()+2, subB.y()+2,
-			    subB.width()-4, subB.height()-4, g );
-	    }
-	    p->setBrush( QBrush(white,Dense4Pattern) );
-	    p->setPen( NoPen );
-	    p->setBackgroundMode( OpaqueMode );
+    if ( style() == WindowsStyle ) {
+	bool maxedOut = (maxValue() == minValue());
+	if ( controls & ADD_LINE ) {
+	    qDrawWinPanel( p, addB.x(), addB.y(),
+			   addB.width(), addB.height(), g,
+			   ADD_LINE_ACTIVE );
+	    qDrawArrow( p, VERTICAL ? DownArrow : RightArrow,
+			WindowsStyle, ADD_LINE_ACTIVE, addB.x()+2, addB.y()+2,
+			addB.width()-4, addB.height()-4, g, !maxedOut );
+	}
+	if ( controls & SUB_LINE ) {
+	    qDrawWinPanel( p, subB.x(), subB.y(),
+			   subB.width(), subB.height(), g,
+			   SUB_LINE_ACTIVE );
+	    qDrawArrow( p, VERTICAL ? UpArrow : LeftArrow,
+			WindowsStyle, SUB_LINE_ACTIVE, subB.x()+2, subB.y()+2,
+			subB.width()-4, subB.height()-4, g, !maxedOut );
+	}
+	p->setBrush( QBrush(white,Dense4Pattern) );
+	p->setPen( NoPen );
+	p->setBackgroundMode( OpaqueMode );
+	if ( maxedOut ) {
+	    p->drawRect( sliderR );
+	} else {
 	    if ( controls & SUB_PAGE )
 		p->drawRect( subPageR );
 	    if ( controls & ADD_PAGE )
 		p->drawRect( addPageR );
 	    if ( controls & SLIDER ) {
-		QBrush fill( g.background() );
-		qDrawWinPanel( p, sliderR.x(), sliderR.y(),
-			       sliderR.width(), sliderR.height(), g,
-			       FALSE, &fill );
-		if ( hasFocus() ) {
-		    //### drawWinFocusFrame
-		    qDrawPlainRect( p, sliderR.x() + 2 , sliderR.y() + 2,
-		    	       sliderR.width() - 5, sliderR.height() - 5, black );
+		if ( !maxedOut ) {
+		    QBrush fill( g.background() );
+		    qDrawWinPanel( p, sliderR.x(), sliderR.y(),
+				   sliderR.width(), sliderR.height(), g,
+				   FALSE, &fill );
 		}
 	    }
-	    break;
-	default:
-	case MotifStyle:
-	    if ( controls & ADD_LINE )
-		qDrawArrow( p, VERTICAL ? DownArrow : RightArrow, MotifStyle,
-			    ADD_LINE_ACTIVE, addB.x(), addB.y(),
-			    addB.width(), addB.height(), g );
-	    if ( controls & SUB_LINE )
-		qDrawArrow( p, VERTICAL ? UpArrow : LeftArrow, MotifStyle,
-			    SUB_LINE_ACTIVE, subB.x(), subB.y(),
-			    subB.width(), subB.height(), g );
-	    if ( controls & SUB_PAGE )
-		p->fillRect( subPageR, g.mid() );
-	    if ( controls & ADD_PAGE )
-		p->fillRect( addPageR, g.mid() );
-	    if ( controls & SLIDER ) {
-		QBrush fill( g.background() );
-		qDrawShadePanel( p, sliderR, g, FALSE, 2, &fill );
-	    }
-	    break;
+	}
+	// ### perhaps this should not be able to accept focus if maxedOut?
+	if ( hasFocus() && (controls & SLIDER) )
+	    p->drawWinFocusRect( sliderR.x()+2, sliderR.y()+2,
+				 sliderR.width()-5, sliderR.height()-5, 
+				 backgroundColor() );
+    } else {
+	if ( controls & ADD_LINE )
+	    qDrawArrow( p, VERTICAL ? DownArrow : RightArrow, MotifStyle,
+			ADD_LINE_ACTIVE, addB.x(), addB.y(),
+			addB.width(), addB.height(), g, value()<maxValue() );
+	if ( controls & SUB_LINE )
+	    qDrawArrow( p, VERTICAL ? UpArrow : LeftArrow, MotifStyle,
+			SUB_LINE_ACTIVE, subB.x(), subB.y(),
+			subB.width(), subB.height(), g, value()>minValue() );
+	if ( controls & SUB_PAGE )
+	    p->fillRect( subPageR, g.mid() );
+	if ( controls & ADD_PAGE )
+	    p->fillRect( addPageR, g.mid() );
+	if ( controls & SLIDER ) {
+	    QBrush fill( g.background() );
+	    qDrawShadePanel( p, sliderR, g, FALSE, 2, &fill );
+	}
+
     }
+}
+
+
 #undef ADD_LINE_ACTIVE
 #undef SUB_LINE_ACTIVE
-}
-
-
-static void qDrawWinArrow( QPainter *p, ArrowType type, bool down,
-			   int x, int y, int w, int h,
-			   const QColorGroup &g )
-{
-    QPointArray a;				// arrow polygon
-    switch ( type ) {
-	case UpArrow:
-	    a.setPoints( 7, -3,1, 3,1, -2,0, 2,0, -1,-1, 1,-1, 0,-2 );
-	    break;
-	case DownArrow:
-	    a.setPoints( 7, -3,-1, 3,-1, -2,0, 2,0, -1,1, 1,1, 0,2 );
-	    break;
-	case LeftArrow:
-	    a.setPoints( 7, 1,-3, 1,3, 0,-2, 0,2, -1,-1, -1,1, -2,0 );
-	    break;
-	case RightArrow:
-	    a.setPoints( 7, -1,-3, -1,3, 0,-2, 0,2, 1,-1, 1,1, 2,0 );
-	    break;
-    }
-    if ( a.isNull() )
-	return;
-
-    if ( down ) {
-	x++;
-	y++;
-    }
-    a.translate( x+w/2, y+h/2 );
-
-    QPen savePen = p->pen();			// save current pen
-    p->fillRect( x, y, w, h, g.background() );
-    p->setPen( g.foreground() );
-    p->drawLineSegments( a, 0, 3 );		// draw arrow
-    p->drawPoint( a[6] );
-    p->setPen( savePen );			// restore pen
-}
-
-
-#if defined(_CC_MSVC_)
-#pragma warning(disable: 4244)
-#endif
-
-static void qDrawMotifArrow( QPainter *p, ArrowType type, bool down,
-			     int x, int y, int w, int h,
-			     const QColorGroup &g )
-{
-    QPointArray bFill;				// fill polygon
-    QPointArray bTop;				// top shadow.
-    QPointArray bBot;				// bottom shadow.
-    QPointArray bLeft;				// left shadow.
-    QWMatrix	matrix;				// xform matrix
-    bool vertical = type == UpArrow || type == DownArrow;
-    bool horizontal = !vertical;
-    int	 dim = w < h ? w : h;
-    int	 colspec = 0x0000;			// color specification array
-
-    if ( dim < 2 )				// too small arrow
-	return;
-
-    if ( dim > 3 ) {
-	if ( dim > 6 )
-	    bFill.resize( dim & 1 ? 3 : 4 );
-	bTop.resize( (dim/2)*2 );
-	bBot.resize( dim & 1 ? dim + 1 : dim );
-	bLeft.resize( dim > 4 ? 4 : 2 );
-	bLeft.putPoints( 0, 2, 0,0, 0,dim-1 );
-	if ( dim > 4 )
-	    bLeft.putPoints( 2, 2, 1,2, 1,dim-3 );
-	bTop.putPoints( 0, 4, 1,0, 1,1, 2,1, 3,1 );
-	bBot.putPoints( 0, 4, 1,dim-1, 1,dim-2, 2,dim-2, 3,dim-2 );
-
-	for( int i=0; i<dim/2-2 ; i++ ) {
-	    bTop.putPoints( i*2+4, 2, 2+i*2,2+i, 5+i*2, 2+i );
-	    bBot.putPoints( i*2+4, 2, 2+i*2,dim-3-i, 5+i*2,dim-3-i );
-	}
-	if ( dim & 1 )				// odd number size: extra line
-	    bBot.putPoints( dim-1, 2, dim-3,dim/2, dim-1,dim/2 );
-	if ( dim > 6 ) {			// dim>6: must fill interior
-	    bFill.putPoints( 0, 2, 1,dim-3, 1,2 );
-	    if ( dim & 1 )			// if size is an odd number
-		bFill.setPoint( 2, dim - 3, dim / 2 );
-	    else
-		bFill.putPoints( 2, 2, dim-4,dim/2-1, dim-4,dim/2 );
-	}
-    }
-    else {
-	if ( dim == 3 ) {			// 3x3 arrow pattern
-	    bLeft.setPoints( 4, 0,0, 0,2, 1,1, 1,1 );
-	    bTop .setPoints( 2, 1,0, 1,0 );
-	    bBot .setPoints( 2, 1,2, 2,1 );
-	}
-	else {					// 2x2 arrow pattern
-	    bLeft.setPoints( 2, 0,0, 0,1 );
-	    bTop .setPoints( 2, 1,0, 1,0 );
-	    bBot .setPoints( 2, 1,1, 1,1 );
-	}
-    }
-
-    if ( type == UpArrow || type == LeftArrow ) {
-	matrix.translate( x, y );
-	if ( vertical ) {
-	    matrix.translate( 0, h - 1 );
-	    matrix.rotate( -90 );
-	} else {
-	    matrix.translate( w - 1, h - 1 );
-	    matrix.rotate( 180 );
-	}
-	if ( down )
-	    colspec = horizontal ? 0x2334 : 0x2343;
-	else
-	    colspec = horizontal ? 0x1443 : 0x1434;
-    }
-    else if ( type == DownArrow || type == RightArrow ) {
-	matrix.translate( x, y );
-	if ( vertical ) {
-	    matrix.translate( w-1, 0 );
-	    matrix.rotate( 90 );
-	}
-	if ( down )
-	    colspec = horizontal ? 0x2443 : 0x2434;
-	else
-	    colspec = horizontal ? 0x1334 : 0x1343;
-    }
-
-    QColor *cols[5];
-    cols[0] = 0;
-    cols[1] = (QColor *)&g.background();
-    cols[2] = (QColor *)&g.mid();
-    cols[3] = (QColor *)&g.light();
-    cols[4] = (QColor *)&g.dark();
-#define CMID	*cols[ (colspec>>12) & 0xf ]
-#define CLEFT	*cols[ (colspec>>8) & 0xf ]
-#define CTOP	*cols[ (colspec>>4) & 0xf ]
-#define CBOT	*cols[ colspec & 0xf ]
-
-    QPen     savePen   = p->pen();		// save current pen
-    QBrush   saveBrush = p->brush();		// save current brush
-    QWMatrix wxm = p->worldMatrix();
-    QPen     pen( NoPen );
-    QBrush   brush( CMID );
-
-    p->setPen( pen );
-    p->setBrush( brush );
-    p->setWorldMatrix( matrix, TRUE );		// set transformation matrix
-    p->drawPolygon( bFill );			// fill arrow
-    p->setBrush( NoBrush );			// don't fill
-
-    p->setPen( CLEFT );
-    p->drawLineSegments( bLeft );
-    p->setPen( CTOP );
-    p->drawLineSegments( bTop );
-    p->setPen( CBOT );
-    p->drawLineSegments( bBot );
-
-    p->setWorldMatrix( wxm );
-    p->setBrush( saveBrush );			// restore brush
-    p->setPen( savePen );			// restore pen
-
-#undef CMID
-#undef CLEFT
-#undef CTOP
-#undef CBOT
-}
 
 
 void qDrawArrow( QPainter *p, ArrowType type, GUIStyle style, bool down,
-		 int x, int y, int w, int h, const QColorGroup &g )
+		 int x, int y, int w, int h,
+		 const QColorGroup & g )
 {
-    switch ( style ) {
-	case WindowsStyle:
-	    qDrawWinArrow( p, type, down, x, y, w, h, g );
-	    break;
-	case MotifStyle:
-	    qDrawMotifArrow( p, type, down, x, y, w, h, g );
-	    break;
-	default:
-#if defined(CHECK_RANGE)
-	    warning( "qDrawArrow: Requested GUI style not supported" );
-#endif
-    }
+    qDrawArrow( p, type, style, down, x, y, w, h, g, TRUE );
 }
