@@ -1,5 +1,5 @@
 /****************************************************************************
-** $Id: qslider.cpp,v 2.30 1997/04/07 12:52:41 paul Exp $
+** $Id: qslider.cpp,v 2.44 1997/09/26 10:57:41 eiriken Exp $
 **
 ** Implementation of QSlider class
 **
@@ -7,13 +7,16 @@
 **
 ** Copyright (C) 1992-1997 Troll Tech AS.  All rights reserved.
 **
-** This file is part of the non-commercial distribution of Qt 1.2.
+** This file is part of the free edition of Qt 1.33.
 **
 ** See the file LICENSE included in the distribution for the usage
 ** and distribution terms, or http://www.troll.no/qt/license.html.
 **
-** (This file differs from the one in the commercial distribution of
-** Qt only by this comment.)
+** IMPORTANT NOTE: You may NOT copy this file or any part of it into
+** your own programs or libraries.
+**
+** (This file differs from the one in the commercial edition of Qt
+** only by this comment.)
 **
 *****************************************************************************/
 
@@ -23,12 +26,12 @@
 #include "qtimer.h"
 #include "qkeycode.h"
 
-RCSTAG("$Id: qslider.cpp,v 2.30 1997/04/07 12:52:41 paul Exp $");
+RCSTAG("$Id: qslider.cpp,v 2.44 1997/09/26 10:57:41 eiriken Exp $");
 
 
 static const int motifBorder = 2;
 static const int motifLength = 30;
-static const int winLength = 10;
+static const int winLength = 9; // Must be odd
 static const int thresholdTime = 500;
 static const int repeatTime    = 100;
 
@@ -52,10 +55,9 @@ static int sliderStartVal = 0; //##### class member?
 
   Tickmarks may be added using setTickmarks().
 
-  A slider can be controlled by the keyboard, but it has a
-  default focusPolicy() of \a NoFocus. Use setFocusPolicy() to
-  enable keyboard focus.
-*/
+  A slider has a default focusPolicy() of \a TabFocus.
+
+  <img src=qslider-m.gif> <img src=qslider-w.gif> */
 
 
 /*!
@@ -121,8 +123,10 @@ void QSlider::init()
     ticks = NoMarks;
     tickInt = 0;
     if ( style() == MotifStyle )
-	setBackgroundColor( colorGroup().mid() );
-    setFocusPolicy( NoFocus );
+	setBackgroundMode( PaletteMid );
+    else
+	setBackgroundMode( PaletteBackground );
+    setFocusPolicy( TabFocus );
     initTicks();
 }
 
@@ -285,9 +289,11 @@ void QSlider::resizeEvent( QResizeEvent * )
 
 void QSlider::setPalette( const QPalette &p )
 {
-    QWidget::setPalette( p );
     if ( style() == MotifStyle )
-	setBackgroundColor( colorGroup().mid() );
+	setBackgroundMode( PaletteMid );
+    else
+	setBackgroundMode( PaletteBackground );
+    QWidget::setPalette( p );
 }
 
 
@@ -349,10 +355,21 @@ static void drawWinPointedSlider( QPainter *p,
 				  const QColorGroup &g,
 				  SlDir dir)
 {
-    const QColor c1 = g.light();
-    const QColor c2 = black;
-    const QColor c3 = g.background();
-    const QColor c4 = g.dark();
+    // 3333330
+    // 3444410
+    // 3422210
+    // 3422210
+    // 3422210
+    // 3422210
+    //  34210
+    //   340
+    //    0
+
+    const QColor c0 = black;
+    const QColor c1 = g.dark();
+    const QColor c2 = g.background();
+    const QColor c3 = g.midlight();
+    const QColor c4 = g.light();
 
     int x1 = r.left();
     int x2 = r.right();
@@ -361,7 +378,7 @@ static void drawWinPointedSlider( QPainter *p,
 
 
     QBrush oldBrush = p->brush();
-    p->setBrush( c3 );
+    p->setBrush( c2 );
     p->setPen( NoPen );
     p->drawRect( r );
     p->setBrush( oldBrush );
@@ -383,72 +400,76 @@ static void drawWinPointedSlider( QPainter *p,
     }
 
     if ( dir != SlUp ) {
-	p->setPen( c1 );
+	p->setPen( c4 );
 	p->drawLine( x1, y1, x2, y1 );
+	p->setPen( c3 );
+	p->drawLine( x1, y1+1, x2, y1+1 );
     }
     if ( dir != SlLeft ) {
-	p->setPen( c1 );
+	p->setPen( c3 );
+	p->drawLine( x1+1, y1+1, x1+1, y2 );
+	p->setPen( c4 );
 	p->drawLine( x1, y1, x1, y2 );
     }
     if ( dir != SlRight ) {
-	p->setPen( c2 );
+	p->setPen( c0 );
 	p->drawLine( x2, y1, x2, y2 );
-	p->setPen( c4 );
+	p->setPen( c1 );
 	p->drawLine( x2-1, y1+1, x2-1, y2-1 );
     }
     if ( dir != SlDown ) {
-	p->setPen( c2 );
+	p->setPen( c0 );
 	p->drawLine( x1, y2, x2, y2 );
-	p->setPen( c4 );
+	p->setPen( c1 );
 	p->drawLine( x1+1, y2-1, x2-1, y2-1 );
     }
 
-    int d = 0;
+    int d;
     switch ( dir ) {
-    case SlUp:
-	p->setPen( c1 );
-	d =  (r.width() + 1) / 2 - 1;
-	p->drawLine( x1, y1, x1+d, y1-d);
-	p->setPen( c2 );
-	d = r.width() - d - 1;
-	p->drawLine( x2, y1, x2-d, y1-d);
-	p->setPen( c4 );
-	d--;
-	p->drawLine( x2-1, y1, x2-1-d, y1-d);
-	break;
-    case SlDown:
-	p->setPen( c1 );
-	d =  (r.width() + 1) / 2 - 1;
-	p->drawLine( x1, y2, x1+d, y2+d);
-	p->setPen( c2 );
-	d = r.width() - d - 1;
-	p->drawLine( x2, y2, x2-d, y2+d);
-	p->setPen( c4 );
-	d--;
-	p->drawLine( x2-1, y2, x2-1-d, y2+d);
-	break;
-    case SlLeft:
-	p->setPen( c1 );
-	d =  (r.height() + 1) / 2 - 1;
-	p->drawLine( x1, y1, x1-d, y1+d);
-	p->setPen( c2 );
-	d = r.height() - d - 1;
-	p->drawLine( x1, y2, x1-d, y2-d);
-	p->setPen( c4 );
-	d--;
-	p->drawLine( x1, y2-1, x1-d, y2-1-d);
-	break;
-    case SlRight:
-	p->setPen( c1 );
-	d =  (r.height() + 1) / 2 - 1;
-	p->drawLine( x2, y1, x2+d, y1+d);
-	p->setPen( c2 );
-	d = r.height() - d - 1;
-	p->drawLine( x2, y2, x2+d, y2-d);
-	p->setPen( c4 );
-	d--;
-	p->drawLine( x2, y2-1, x2+d, y2-1-d);
-	break;
+	case SlUp:
+	    p->setPen( c4 );
+	    d =  (r.width() + 1) / 2 - 1;
+	    p->drawLine( x1, y1, x1+d, y1-d);
+	    p->setPen( c0 );
+	    d = r.width() - d - 1;
+	    p->drawLine( x2, y1, x2-d, y1-d);
+	    p->setPen( c1 );
+	    d--;
+	    p->drawLine( x2-1, y1, x2-1-d, y1-d);
+	    break;
+	case SlDown:
+	    p->setPen( c4 );
+	    d =  (r.width() + 1) / 2 - 1;
+	    p->drawLine( x1, y2, x1+d, y2+d);
+	    p->setPen( c0 );
+	    d = r.width() - d - 1;
+	    p->drawLine( x2, y2, x2-d, y2+d);
+	    p->setPen( c1 );
+	    d--;
+	    p->drawLine( x2-1, y2, x2-1-d, y2+d);
+	    break;
+	case SlLeft:
+	    p->setPen( c4 );
+	    d =  (r.height() + 1) / 2 - 1;
+	    p->drawLine( x1, y1, x1-d, y1+d);
+	    p->setPen( c0 );
+	    d = r.height() - d - 1;
+	    p->drawLine( x1, y2, x1-d, y2-d);
+	    p->setPen( c1 );
+	    d--;
+	    p->drawLine( x1, y2-1, x1-d, y2-1-d);
+	    break;
+	case SlRight:
+	    p->setPen( c4 );
+	    d =  (r.height() + 1) / 2 - 1;
+	    p->drawLine( x2, y1, x2+d, y1+d);
+	    p->setPen( c0 );
+	    d = r.height() - d - 1;
+	    p->drawLine( x2, y2, x2+d, y2-d);
+	    p->setPen( c1 );
+	    d--;
+	    p->drawLine( x2, y2-1, x2+d, y2-1-d);
+	    break;
     }
 
 }
@@ -480,11 +501,11 @@ void QSlider::paintSlider( QPainter *p, const QRect &r )
     case MotifStyle:
 	qDrawShadePanel( p, r, g, FALSE, 2, &fill );
 	if ( orient == Horizontal ) {
-	    QCOORD mid = ( r.left() + r.right() ) / 2;
+	    QCOORD mid = ( r.left() + r.right() + 1) / 2;
 	    qDrawShadeLine( p, mid,  r.top(), mid,  r.bottom() - 1,
 			    g, TRUE, 1);
 	} else {
-	    QCOORD mid = ( r.top() + r.bottom() ) / 2;
+	    QCOORD mid = ( r.top() + r.bottom() + 1) / 2;
 	    qDrawShadeLine( p, r.left(), mid,  r.right() - 1, mid,
 			    g, TRUE, 1);
 	}
@@ -498,10 +519,24 @@ void QSlider::paintSlider( QPainter *p, const QRect &r )
 
 void QSlider::reallyMoveSlider( int newPos )
 {
-    QRect updateR = sliderRect();
+    QRect oldR = sliderRect();
     sliderPos = newPos;
-    updateR.unite( sliderRect() );
-    repaint( updateR );
+    QRect newR = sliderRect();
+    //since sliderRect isn't virtual, I know that oldR and newR
+    // are the same size.
+    if ( orient == Horizontal ) {
+	if ( oldR.left() < newR.left() )
+	    oldR.setRight( QMIN ( oldR.right(), newR.left()));
+	else           //oldR.right() >= newR.right() 
+	    oldR.setLeft( QMAX ( oldR.left(), newR.right()));
+    } else {
+	if ( oldR.top() < newR.top() )
+	    oldR.setBottom( QMIN ( oldR.bottom(), newR.top()));
+	else           //oldR.bottom() >= newR.bottom() 
+	    oldR.setTop( QMAX ( oldR.top(), newR.bottom()));
+    }
+    repaint( oldR );
+    repaint( newR, FALSE );
 }
 
 
@@ -529,13 +564,13 @@ void QSlider::drawWinGroove( QPainter *p, QCOORD c )
   Handles paint events for the slider.
 */
 
-void QSlider::paintEvent( QPaintEvent * )
+void QSlider::paintEvent( QPaintEvent *e )
 {
-    //QRect paintRect = e->rect();
     
     QPainter p;
     p.begin( this );
-
+    QRect paintRect = e->rect();
+    p.setClipRect( paintRect );
     QRect sliderR = sliderRect();
     QColorGroup g = colorGroup();
     switch ( style() ) {
@@ -548,7 +583,7 @@ void QSlider::paintEvent( QPaintEvent * )
 		r.setRect( tickOffset-1, 0, thickness()+2, height() );
 	    r = r.intersect( rect() );
 	    qDrawPlainRect( &p, r, g.background() );
-	    p.drawWinFocusRect( r );
+	    p.drawWinFocusRect( r, backgroundColor() );
 	}
 	{
 	    int mid = tickOffset + thickness()/2;
@@ -697,6 +732,14 @@ void QSlider::mouseReleaseEvent( QMouseEvent *e )
     resetState();
 }
 
+/*!
+  Handles focus in events for the slider.
+*/
+
+void QSlider::focusInEvent( QFocusEvent * )
+{
+    repaint( FALSE );
+}
 
 /*!
   Moves the left (or top) edge of the slider to position 
@@ -938,7 +981,7 @@ void QSlider::drawTicks( QPainter *p, int d, int w, int i ) const
 {
     p->setPen( colorGroup().foreground() );
     int v = minValue();
-    int fudge = slideLength() / 2 - 1;
+    int fudge = slideLength() / 2 + 1;
     while ( v <= maxValue() + 1 ) {
 	int pos = positionFromValue( v ) + fudge;
 	if ( orient == Horizontal )
@@ -972,6 +1015,15 @@ void QSlider::setTickmarks( TickSetting s )
     update();
 }
 
+
+/*!
+  \fn QSlider::TickSetting QSlider::tickmarks() const
+
+  Returns the tickmark settings for this slider.
+
+  \sa setTickmarks()
+  */
+
 /*!
   \fn QSlider::TickSetting tickmarks() const
   Returns the way tickmarks are displayed by the slider.
@@ -994,7 +1046,7 @@ void QSlider::setTickInterval( int i )
 
 
 /*!
-  \fn int tickmarks()
+  \fn int QSlider::tickInterval() const
   Returns the interval between tickmarks. Returns 0 if the slider
   chooses between pageStep() and lineStep().
   \sa setTickInterval()
